#include <stdio.h>
#include <stdlib.h>
#include "./inc/ComputeScore.h"
#include "./inc/ReadInput.h"
#include "./inc/RetrieveScore.h"
#include "./inc/BuildHashes.h"
#include "./inc/Parameter.h"

void AddMatchPairs(MATCH *matchPairs, int *pair, MATCH **match){
	int	i, k;
	k = matchPairs->numbasepair;
	/*add matched stacks in match to matchPairs*/
	matchPairs->pair[2 * k] = pair[0];
	matchPairs->pair[2 * k + 1] = pair[1];
	matchPairs->numbasepair ++;
	//printf("%d	|	%d\n", matchPairs->pair[2 * k], matchPairs->pair[2 * k + 1]);
	/*recursively add all stacks enclosed by pair*/
	for(i = 0; i < match[pair[0]][pair[1]].numbasepair; i ++)	{
		AddMatchPairs(matchPairs, &(match[pair[0]][pair[1]].pair[2 * i]), match);
	}
	return;
}




double AlignRNAPairs(char *seq1, int len_seq1, char *seq2, int len_seq2, int *pair1,
		int numPairs1, int *pair2, int numPairs2, int numCleavage, int *Cleavages, MATCH *matchPairs){
	extern	double **HairpinSim, **BasepairSim;	
	int	i, j, k, l, h, m, n, m1, m2;
	/*
	for(i = 0; i < numPairs1; ++ i)	{
		printf("%d	%d	%d	%d	%d	%d\n", pair1[i * 6], pair1[i * 6  + 1], pair1[i * 6 + 2], pair1[i * 6 + 3], pair1[i * 6 + 4], pair1[i * 6 + 5]);
	}
	printf("\n");
	for(i = 0; i < numPairs2; ++ i)	{
		printf("%d	%d	%d	%d	%d	%d\n", pair2[i * 6], pair2[i * 6  + 1], pair2[i * 6 + 2], pair2[i * 6 + 3], pair2[i * 6 + 4], pair2[i * 6 + 5]);
	}
	*/
	int	len1, len2;
	char	*seq_b1, *seq_b2;
	int	level1, level2;
	int	**levelindex1, **levelindex2, *sizelevel1, *sizelevel2;
	int	**downarray1, **downarray2, *sizedownarray1, *sizedownarray2, **uparray1, *sizeuparray1, **uparray2,
		*sizeuparray2;
	int	*index1, *index2;
	double	score, chainscore, mscore0;
	MATCH	matchchain, **match;
	int	**numContStacking;
	/* transform the sequence from character into integer*/
	seq_b1 = (char *) calloc(len_seq1 + 1000, sizeof(char));
	seq_b2 = (char *) calloc(len_seq2 + 1000, sizeof(char));
	for(i = 0; i < len_seq1; i ++)	{
		seq_b1[i] = Char2int(seq1[i]);
	}
	for(i = 0; i < len_seq2; i ++)	{
		seq_b2[i] = Char2int(seq2[i]);
	}
	/*
	printf("Pair1\n");
	for(i=0; i<numPairs1; ++i){
		printf("%d:	", i);
		for(j=0; j<6; j++){
			printf("%d	", pair1[6 * i + j]);
		}
		printf("\n");
	}
	printf("Pair2\n");
	for(i=0; i<numPairs2; ++i){
		printf("%d:	", i);
		for(j=0; j<6; j++){
			printf("%d	", pair2[6 * i + j]);
		}
		printf("\n");
	}
	*/
	/* fill the matrix of hairpin loop similarity*/
	FillHairpinSim(seq_b1, seq_b2, pair1, numPairs1, pair2, numPairs2, numCleavage, Cleavages);
	/* fill the matrix of para score*/
	FillParaScore(seq_b1, len_seq1, seq_b2, len_seq2, pair1, numPairs1, pair2, numPairs2, numCleavage, Cleavages);
	for(i=0; i<numPairs1; ++i){
		for(j=0; j<numPairs2; ++j){
			PARASCORE *cindex = GetParaHead(i, j);
			ListAllParaScore(cindex);
		}
	}
	/* fill the matrix of basepair similarity*/
	FillBasepairSim(seq_b1, seq_b2, pair1, numPairs1, pair2, numPairs2);
	/* allocate memories for index, used for traceback*/
	/* initialize matchchain, used for traceback*/
	matchchain.numbasepair = 0;

	/* allocate memories for downarray, sizedownarray, uparray and sizeuparray, for each sequence*/
	downarray1 = (int **) calloc(numPairs1, sizeof(int *));
	uparray1 = (int **) calloc(numPairs1, sizeof(int *));
	sizedownarray1 = (int *) calloc(numPairs1, sizeof(int));
	sizeuparray1 = (int *) calloc(numPairs1, sizeof(int));
	for(i = 0; i < numPairs1; i ++)	{
		downarray1[i] = (int *) calloc(numPairs1, sizeof(int));
		uparray1[i] = (int *) calloc(numPairs1, sizeof(int));
	}
	downarray2 = (int **) calloc(numPairs2, sizeof(int *));
	uparray2 = (int **) calloc(numPairs2, sizeof(int *));
	sizedownarray2 = (int *) calloc(numPairs2, sizeof(int));
	sizeuparray2 = (int *) calloc(numPairs2, sizeof(int));
	for(i = 0; i < numPairs2; i ++)	{
		downarray2[i] = (int *) calloc(numPairs2, sizeof(int));
		uparray2[i] = (int *) calloc(numPairs2, sizeof(int));
	}

	/* build downarray and uparray of the basepairs, used in building levels of basepairs*/
	BuildArrays(pair1, numPairs1, downarray1, sizedownarray1, uparray1, sizeuparray1);
	BuildArrays(pair2, numPairs2, downarray2, sizedownarray2, uparray2, sizeuparray2);
	//printf("%d\n", numPairs1);
	/*
	for(i = 0; i < numPairs1; ++ i) {
		for(j = 0; j < sizedownarray1[i]; ++ j) {
			printf("%d	", downarray1[i][j]);
		}
		printf("\n");
	}
	
	for(i = 0; i < numPairs2; ++ i) {
		for(j = 0; j < sizedownarray2[i]; ++ j) {
			printf("%d	", downarray2[i][j]);
		}
		printf("\n");
	}
	*/
	/* build levels for comparison*/
	/* allocate memory for levelindex and sizelevel, for each sequence*/
	levelindex1 = (int **) calloc(numPairs1, sizeof(int *));
	sizelevel1 = (int *) calloc(numPairs1, sizeof(int));
	for(i = 0; i < numPairs1; i ++)	{
		levelindex1[i] = (int *) calloc(numPairs1, sizeof(int));
	}
	levelindex2 = (int **) calloc(numPairs2, sizeof(int *));
	sizelevel2 = (int *) calloc(numPairs2, sizeof(int));
	for(i = 0; i < numPairs2; i ++)	{
		levelindex2[i] = (int *) calloc(numPairs2, sizeof(int));
	}
	/* build levels, the number of levels is returned, stored in level1 and level2*/
	level1 = BuildLevels(pair1, numPairs1, sizedownarray1, uparray1, sizeuparray1, levelindex1, sizelevel1);
	level2 = BuildLevels(pair2, numPairs2, sizedownarray2, uparray2, sizeuparray2, levelindex2, sizelevel2);
	//for(i = 0; i < numPairs1; ++ i) {
	//	for(j = 0; j < sizelevel1[i]; ++ j) {
	//		printf("%d	", levelindex1[i][j]);
	//	}
	//	printf("\n");
	//}
	
	/* iterate process of dynamic programming*/
	
	
	/* allocate memory to the matrix to be filled in dynamic programming (match)*/
	match = (MATCH **) calloc(numPairs1, sizeof(MATCH *));
	numContStacking = (int **) calloc(numPairs1, sizeof(int *));
	for(i = 0; i < numPairs1; i ++){
		match[i] = (MATCH *) calloc(numPairs2, sizeof(MATCH));
		numContStacking[i] = (int *) calloc(numPairs2, sizeof(int));
	}
	/* boundary case 1: the lowest level in motif matching other basepairs in target*/
	for(i = 0; i < sizelevel1[0]; i ++){
		for(j = 0; j < level2; j ++){
			/* adds up the target missing penalty in previous levels*/
			for(k = 0; k < sizelevel2[j]; k ++){
				m1 = levelindex1[0][i];
				m2 = levelindex2[j][k];
				/* one pair only for boundary condition, assuming no pair in lower level to be matched*/
				/* adds up to the score*/
				match[m1][m2].score = rweight2() * GetBasepairSim(m1, m2) 
						+ rweight3() * RetrievePairScore(seq_b1, seq_b2, &pair1[6 * m1], &pair2[6 * m2]);
				/*
				printf("initial calculation of match score: %f\n", match[m1][m2].score);
				*/
				double PenaltyScore = rweight3() * (sizedownarray1[m1] * rPENMissMotif() + sizedownarray2[m2] * rPENMissTarget());
				match[m1][m2].score += rweight1() * GetHairpinSim(m1, m2) - PenaltyScore;
				/*
				printf("match score after adding mid part sequence: %f\n", match[m1][m2].score);
				printf("m1: %d	m2: %d	hairpin score: %f\n", m1, m2, GetHairpinSim(m1, m2));
				printf("penalty score: %f\n", PenaltyScore);
				*/
				/* in this case we should not consider the region because it does not give us bonus if a cleavage site presents*/ 
				if(PenaltyScore > rweight1() * GetHairpinSim(m1, m2)){
					/* see if we have a cleavage site*/
					for(l = 0 ; l < numCleavage; ++l){
						/* if a cleavage site presents*/
						if(pair1[6 * m1] < Cleavages[l] && pair1[6 * m1 + 3] >= Cleavages[l]){
							/* add back the penalty and substract the sequence similarity*/
							match[m1][m2].score += PenaltyScore - rweight1() * GetHairpinSim(m1, m2);
						}
					}
				}
				/*				
				printf("m1: %d	m2: %d\n", m1, m2);
				printf("match score is %f\n", match[m1][m2].score);
				printf("HairpinScore: %f\n", rweight1() * GetHairpinSim(m1, m2));
				printf("BasepairScore: %f\n", rweight2() * GetBasepairSim(m1, m2));
				printf("PairScore: %f\n", rweight3() * RetrievePairScore(seq_b1, seq_b2, &pair1[6 * m1], &pair2[6 * m2]));
				printf("\n");
				*/
				/* if the score is negative, make it 0 for semi-global alignment*/				
				if(match[m1][m2].score < 0){
					for(l=0; l<numCleavage; ++l){
						if(pair1[6 * m1] < Cleavages[l] && pair1[6 * m1 + 3] >= Cleavages[l]){
							match[m1][m2].score = 0.0;
						}
					}
				}
				/*
				printf("M1: %d	M2: %d\n", m1, m2);
				printf("final match score: %f\n", match[m1][m2].score);
				*/
				/* the other part of achieving semi-global alignment is traceback from the optimal score
				   instead of the last score*/
			}
		}
	}
	
	/* boundary case 2: the lowest level in target matching other basepairs in motif*/
	for(j = 0; j < sizelevel2[0]; j++){
		for(i = 1; i < level1; i++){
			/* adds up the motif missing penalty in previous levels*/
			for(k = 0; k < sizelevel1[k]; k++){
				m1 = levelindex1[i][k];
				m2 = levelindex2[0][j];
				/* one pair only for boundary condition, assuming no pair in lower level to be matched*/
				/* adds up to the score*/
				match[m1][m2].score = rweight2() * GetBasepairSim(m1, m2)
						+ rweight3() * RetrievePairScore(seq_b1, seq_b2, &pair1[6 * m1], &pair2[6 * m2]);
				/*
				printf("initial calculation of match score: %f\n", match[m1][m2].score);
				*/
				double PenaltyScore = rweight3() * (sizedownarray1[m1] * rPENMissMotif() + sizedownarray2[m2] * rPENMissTarget());
				match[m1][m2].score += rweight1() * GetHairpinSim(m1, m2) - PenaltyScore;
				/*
				printf("match score after adding mid part sequence: %f\n", match[m1][m2].score);
				printf("m1: %d	m2: %d	hairpin score: %f\n", m1, m2, GetHairpinSim(m1, m2));
				printf("penalty score: %f\n", PenaltyScore);
				*/
				/* in this case we should not consider the region because it does not give us bonus if a cleavage site presents*/ 
				if(PenaltyScore > rweight1() * GetHairpinSim(m1, m2)){
					/* see if we have a cleavage site*/
					for(l = 0 ; l < numCleavage; ++l){
						/* if a cleavage site presents*/
						if(pair1[6 * m1] < Cleavages[l] && pair1[6 * m1 + 3] >= Cleavages[l]){
							/* add back the penalty and substract the sequence similarity*/
							match[m1][m2].score += PenaltyScore - rweight1() * GetHairpinSim(m1, m2);
						}
					}
				}
				/*				
				printf("m1: %d	m2: %d\n", m1, m2);
				printf("match score is %f\n", match[m1][m2].score);
				printf("HairpinScore: %f\n", rweight1() * GetHairpinSim(m1, m2));
				printf("BasepairScore: %f\n", rweight2() * GetBasepairSim(m1, m2));
				printf("PairScore: %f\n", rweight3() * RetrievePairScore(seq_b1, seq_b2, &pair1[6 * m1], &pair2[6 * m2]));
				printf("\n");
				*/
				/* if the score is negative, make it 0 for semi-global alignment*/
				if(match[m1][m2].score < 0){
					for(l=0; l<numCleavage; ++l){
						if(pair1[6 * m1] < Cleavages[l] && pair1[6 * m1 + 3] >= Cleavages[l]){
							match[m1][m2].score = 0.0;
						}
					}
				}
				/*
				printf("M1: %d	M2: %d\n", m1, m2);
				printf("final match score: %f\n", match[m1][m2].score);
				*/
				/* the other part of achieving semi-global alignment is traceback from the optimal score
				   instead of the last score*/ 
			}
		}
	}
	/* dynamic programming strategy filling the 'match' matrix*/
	for(i = 1; i < level1; i ++){
		for(k = 0; k < sizelevel1[i]; k ++){
			for(j = 1; j < level2; j ++){
				for(l = 0; l < sizelevel2[j]; l ++){					
					m1 = levelindex1[i][k];
					m2 = levelindex2[j][l];
					/* calculate the score contribution of the two basepair themselves*/
					score =  rweight2() * GetBasepairSim(m1, m2)
						+ rweight3() * RetrievePairScore(seq_b1, seq_b2, &pair1[6 * m1], &pair2[6 * m2]);
					/*
					printf("\nassume no pair matched: %f\n\n", score);
					*/
					/* compute the score contribution of the regions enclosed by the two pairs*/
					/* assuming there is no basepair matched*/
					mscore0 = rweight1() * GetHairpinSim(m1, m2)
							- rweight3() * (sizedownarray1[m1] * rPENMissMotif() + sizedownarray2[m2] * rPENMissTarget());
					//printf("m1: %d	m2: %d	hairpin score: %f\n", m1, m2, GetHairpinSim(m1, m2));
					/* if we do not want to consider the region*/					
					if(mscore0 < 0){
						/* check whether a cleavage site presents*/
						for(h = 0; h < numCleavage; ++h){
							if(pair1[6 * m1] < Cleavages[h] && pair1[6 * m1 + 3] >= Cleavages[h]){
								mscore0 = 0;
							}
						}
					}
					/* return the optimum score between 3 cases: 
					   no basepair matched, 1 basepair matched, and multiple basepair matched*/
					/*
					printf("\nm1: %d\nm2: %d\n", m1, m2);
					printf("score: %f\n", score);
					*/
					chainscore = CalChainScore(m1, m2, pair1, downarray1[m1], sizedownarray1[m1],
						numPairs1, pair2, downarray2[m2], sizedownarray2[m2],
						numPairs2, match, numContStacking, &matchchain, mscore0,
						pair1[6 * m1], pair1[6 * m1 + 3],
						pair2[6 * m2], pair2[6 * m2 + 3], seq_b1, seq_b2,
						len_seq1, len_seq2, numCleavage, Cleavages, 0);
					/* adds up the score*/
					match[m1][m2].score = score + chainscore;
					/*
					printf("chainscore: %f\n\n", chainscore);
					*/
					/* record the information for trace back*/
					for(m = 0; m < matchchain.numbasepair; m ++){
						match[m1][m2].pair[2 * m] = matchchain.pair[2 * (matchchain.numbasepair - m - 1)];
						match[m1][m2].pair[2 * m + 1] = matchchain.pair[2 * (matchchain.numbasepair - m - 1) + 1];
						/* update the matrix recording the stacking base-pairs*/
						if(pair1[6 * match[m1][m2].pair[2 * m]] == pair1[6 * m1] + 1 &&
						   pair1[6 * match[m1][m2].pair[2 * m] + 3] == pair1[6 * m1 + 3] - 1 &&
						   pair2[6 * match[m1][m2].pair[2 * m + 1]] == pair2[6 * m2] + 1 &&
						   pair2[6 * match[m1][m2].pair[2 * m + 1] + 3] == pair2[6 * m2 + 3] - 1){
							numContStacking[m1][m2] = 
								numContStacking[match[m1][m2].pair[2 * m]][match[m1][m2].pair[2 * m + 1]] + 1;
						}
					}
					match[m1][m2].numbasepair = matchchain.numbasepair;
					/* check whether the matching score is smaller than 0*/
					if(match[m1][m2].score < 0){
						/* set the score and number of matched base-pairs to 0*/
						for(h=0; h<numCleavage; ++h){
							if(pair1[6 * m1] < Cleavages[h] && pair1[6 * m1 + 3] >= Cleavages[h]){
								match[m1][m2].score = 0.0;
								match[m1][m2].numbasepair = 0;
							}
						}
					}
					/*
					printf("M1: %d	M2: %d\n", m1, m2);
					printf("final match score: %f\n", match[m1][m2].score);
					*/
				}
			}
		}
	}
	//printf("finishing calchainscore in AlignRNAPairs\n");
	for(i = 0; i < numPairs1; i ++)	{
		free((void *) levelindex1[i]);
	}
	free((void **) levelindex1);
	free((void *) sizelevel1);
	for(i = 0; i < numPairs2; i ++)	{
		free((void *) levelindex2[i]);
	}
	free((void **) levelindex2);
	free((void *) sizelevel2);

	for(i = 0; i < numPairs1; i ++)	{
		free((void *) downarray1[i]);
		free((void *) uparray1[i]);
	}
	free((void **) downarray1);
	free((void **) uparray1);
	free((void *) sizedownarray1);
	free((void *) sizeuparray1);

	for(i = 0; i < numPairs2; i ++)	{
		free((void *) downarray2[i]);
		free((void *) uparray2[i]);
	}
	free((void **) downarray2);
	free((void **) uparray2);
	free((void *) sizedownarray2);
	free((void *) sizeuparray2);
	/* traceback*/
	mscore0 = -100.0;
	index1 = (int *) calloc(numPairs1, sizeof(int));
	index2 = (int *) calloc(numPairs2, sizeof(int));
	for(i = 0; i < numPairs1; i ++){
		/* only used in the last part of the program, mimic the downarray*/
		index1[i] = i;
	}
	for(i = 0; i < numPairs2; i ++){
		/* only used in the last part of the program, mimic the downarray*/
		index2[i] = i;
	}
	/*
	printf("printing pair1\n");
	for(i=0; i<numPairs1; ++i){
		printf("%d	|	%d\n", pair1[6 * i], pair1[6 * i + 3]);
	}
	printf("printing pair2\n");
	for(i=0; i<numPairs2; ++i){
		printf("%d	|	%d\n", pair2[6 * i], pair2[6 * i + 3]);
	}
	*/
	/* assuming there is an putative basepair that encloses the whole basepair, calculate the score*/
	/*
	printf("\n\nThe last call of calchainscore!\n\n");
	*/
	pair1[6 * numPairs1] = -1;
	pair1[6 * numPairs1 + 3] = len_seq1;
	pair2[6 * numPairs2] = -1;
	pair2[6 * numPairs2 + 3] = len_seq2;
	chainscore = CalChainScore(numPairs1, numPairs2, pair1, index1, numPairs1, numPairs1 + 1,
			pair2, index2, numPairs2, numPairs2 + 1,
			match, numContStacking, &matchchain, mscore0, -1, -1, -1, -1, seq_b1, seq_b2, len_seq1, len_seq2, numCleavage, Cleavages, 1);
	//printf("finish traceback calchainscore\n");
	/*
	printf("numContStacking\n");
	for(i=0; i<numPairs1; ++i){
		for(j=0; j<numPairs2; ++j){
			printf("%d	", numContStacking[i][j]);
		}
		printf("\n");
	}
	*/
	/* add matched basepairs to matchPairs, which is an array passed in as an argument, and used to store the result*/
	//printf("finish last call of calchainscore\n");
	matchPairs->score = chainscore;
	for(l = 0; l < matchchain.numbasepair; l ++)	{
		AddMatchPairs(matchPairs, &(matchchain.pair[2 * (matchchain.numbasepair - 1 - l)]), match);
	}
	//printf("finish adding pairs\n");
	FreeLoopsim(numPairs1, numPairs2);
	/*
	printf("score:\n");
	for(i=0; i<numPairs1; ++i){
		for(j=0; j<numPairs2; ++j){
			printf("%f	", match[i][j].score);
		}
		printf("\n");
	}
	*/
	//printf("finish freeing loopsim\n");
	for(i = 0; i < numPairs1; i ++)	{
		free((void *) match[i]);
		free((void *) numContStacking[i]);
	}
	free((void **) match);
	free((void **) numContStacking);
	free((void *) index1);
	free((void *) index2);
	free((void *) seq_b1);
	free((void *) seq_b2);
	return(chainscore);
}

