#!/usr/bin/perl -w
use strict;

# reads in the BPSEQ file
my $bpseqFile = shift;
open my $IN, "<$bpseqFile" or die "Cannot open file: $!\n";
my $header = <$IN>;
my $seq = <$IN>;
chomp $header;
chomp $seq;
my $temp = <$IN>;	# remove format characters
my @basepairs;
while(<$IN>){
	chomp;
	my @decom = split /\s+/, $_;
	push @basepairs, [@decom];
}
close $IN;

my $i;
my $j;
my @mutualEx;
for($i = 0; $i < scalar(@basepairs); ++ $i){
	for($j = 0; $j < scalar(@basepairs); ++ $j){
		$mutualEx[$i][$j] = 0;
	}
}

for($i = 0; $i < scalar(@basepairs) - 1; ++ $i){
	my @pLocA = split /-/, $basepairs[$i][1];
	for($j = $i + 1; $j < scalar(@basepairs); ++ $j){
		my @pLocB = split /-/, $basepairs[$j][1];
		if($pLocA[0] < $pLocB[0] && $pLocA[1] > $pLocB[0] && $pLocA[1] < $pLocB[1] ||
		   $pLocB[0] < $pLocA[0] && $pLocB[1] > $pLocA[0] && $pLocB[1] < $pLocA[1]){
			$mutualEx[$i][$j] = 1;
			$mutualEx[$j][$i] = 1;
		}		
	}		
}

# greedy algorithm finding maximum number of mutual exclusion
# remove the correspinding basepair and update the array accordingly
my $maxEx;
my $maxExid;
my %crossing;
do{
	$maxEx = -1;	
	for($i = 0; $i < scalar(@basepairs); ++ $i){
		my $numEx = 0;
		for($j = 0; $j < scalar(@basepairs); ++ $j){
			$numEx += $mutualEx[$i][$j];
		}
		if($numEx > $maxEx){
			$maxEx = $numEx;
			$maxExid = $i;
		}
	}
	if($maxEx > 0){
		$crossing{$maxExid} = 1;
		for($i = 0; $i < scalar(@basepairs); ++ $i){
			$mutualEx[$maxExid][$i] = 0;
			$mutualEx[$i][$maxExid] = 0;
		}
	}
}while($maxEx > 0);

my $betweenStrand;
for($i = 0; $i < length($seq); ++ $i){
	my $c = substr($seq, $i, 1);
	if($c eq '.'){
		$betweenStrand = $i - 1;
		last;
	}
}

# subroutine for generating brackets format output
sub genBrackets(@){
	my @out;
	push @out, '.' x length($seq);
	push @out, '.' x length($seq);
	my $k;
	foreach(@_){
		my @pLoc = split /-/, ${$_}[1];
		my @pEdge = split /\//, ${$_}[4];
		my $foundTag = 0;	# records whether there is a suitable place for the pair
		# considering the '...' which indicating a break of strand in the original sequence
		if($pLoc[0] > $betweenStrand){
			$pLoc[0] += 3;
		}
		if($pLoc[1] > $betweenStrand){
			$pLoc[1] += 3;
		}
		for($k = 0; $k < scalar(@out); ++ $k){
			
			# if there is a suitable place
			if(substr($out[$k], $pLoc[0], 1) eq '.' && substr($out[$k], $pLoc[1], 1) eq '.'){
				$foundTag = 1;
				substr($out[$k], $pLoc[0], 1) = '(';
				substr($out[$k], $pLoc[1], 1) = ')';
				substr($out[$k + 1], $pLoc[0], 1) = $pEdge[0];
				substr($out[$k + 1], $pLoc[1], 1) = $pEdge[1];
				last;
			}
		}
		if($foundTag == 0){
			push @out, '.' x length($seq);
			push @out, '.' x length($seq);
			substr($out[$k], $pLoc[0], 1) = '(';
			substr($out[$k], $pLoc[1], 1) = ')';
			substr($out[$k + 1], $pLoc[0], 1) = $pEdge[0];
			substr($out[$k + 1], $pLoc[1], 1) = $pEdge[1];
		}
	}
	return @out;
}

# separate the base-pairs to crossing and non-crossing cases
my @nestedpairs;
my @crosspairs;
for($i = 0; $i < scalar(@basepairs); ++ $i){
	if(exists $crossing{$i}){
		push @crosspairs, $basepairs[$i];
	}else{
		push @nestedpairs, $basepairs[$i];
	}
}

my @nestedOut = genBrackets(@nestedpairs);
my @crossOut = genBrackets(@crosspairs);

# print the formatted segments
print "$header\n";
print "$seq\n";
print "\/\/\n";
foreach(@nestedOut){
	print "$_\n";
}
print "@@\n";
foreach(@crossOut){
	print "$_\n";
}
