/*
 * AddPair
 *
 * The input is the pairs, the index of the enclosing pair, the index of the enclosed pair, downarray array, downarray size array
 * uparray array and uparray size array
 * AddPair is a function called by BuildArrays
 * The function of it is adding specified pair into the 'uparray' and 'downarray'
*/
void AddPair(int *pairs, int index1, int index2, int **downarray, int *sizedownarray, int **uparray, int *sizeuparray)
{
	int	i, j, k, l;
	
	/* Find the closest stack that enclosed the base-pair 'index2'*/
	for(i = 0; i < sizeuparray[index2]; i ++)	{
		/* If some index has already enclosed the base-pair has the index 'index2' and the base-pair is enclosed by the base-pair 'index1'*/
		if(pairs[6 * uparray[index2][i] + 3] <= pairs[6 * index1 + 3] &&
		   pairs[6 * uparray[index2][i]] >= pairs[6 * index1])	{
			break;
		} else if(pairs[6 * uparray[index2][i] + 3] >= pairs[6 * index1 + 3] &&
		   	pairs[6 * uparray[index2][i]] <= pairs[6 * index1])	{
			/* If the base-pair 'index2' has some other base-pair that enclose itself and 'index1' at the same time, replace that base-pair*/
			uparray[index2][i] = index1;
			break;
		}
	}
	/*if enclosion is not found in the existed base-pairs*/
	/*add directly*/
	if(i == sizeuparray[index2])	{
		uparray[index2][sizeuparray[index2] ++] = index1;
	}
	downarray[index1][sizedownarray[index1] ++] = index2;
	return;
}

/*
 * BuildArrays
 *
 * The input is pairs, number of pairs, four arrays to store the result (downarray, downarray size, uparray and uparray size)
 * BuildArrays build arrays according to their relative enclosing relationships
 * uparray records the pairs that enclosing the pair
 * downarray records the pairs that enclosed by the pair
*/
void BuildArrays(int *pairs, int num_pairs, int **downarray, int *sizedownarray, int **uparray, int *sizeuparray)
{
	int	i, j, k, l;
	int	shift1, shift2;

	for(i = 0; i < num_pairs; i ++)	{
		for(j = i + 1; j < num_pairs; j ++)	{
			/* if the ith base-pair is enclosing the jth base-pair */
			if(pairs[6*i] <= pairs[6*j] && pairs[6*i+3] >= pairs[6*j+3])	{
				/*then separate them into two arrays*/
				AddPair(pairs, i, j, downarray, sizedownarray, uparray, sizeuparray);
			} else if(pairs[6*j] > pairs[6 * i + 3])	{
				/* if already go out of range */
				break;
			}
		}
	}
	return;
}


