#include <stdio.h>
#include <stdlib.h>
#include "./inc/BuildHashes.h"

/*
 * AddPair
 *
 * The input is the pairs, the index of the enclosing pair, the index of the enclosed pair, downarray array, downarray size array
 * uparray array and uparray size array
 * AddPair is a function called by BuildArrays
 * The function of it is adding specified pair into the 'uparray' and 'downarray'
*/
void AddPair(int *pairs, int index1, int index2, int **downarray, int *sizedownarray, int **uparray, int *sizeuparray)
{
	int	i, j, k, l;
	
	/* Find the closest stack that enclosed the base-pair 'index2'*/
	for(i = 0; i < sizeuparray[index2]; i ++)	{
		/* If some index has already enclosed the base-pair has the index 'index2' and the base-pair is enclosed by the base-pair 'index1'*/
		if(pairs[6 * uparray[index2][i] + 3] <= pairs[6 * index1 + 3] && pairs[6 * uparray[index2][i]] >= pairs[6 * index1]){
			break;
		} else if(pairs[6 * uparray[index2][i] + 3] >= pairs[6 * index1 + 3] && pairs[6 * uparray[index2][i]] <= pairs[6 * index1]){
			/* If the base-pair 'index2' has some other base-pair that enclose itself and 'index1' at the same time, replace that base-pair*/
			uparray[index2][i] = index1;
			break;
		}
	}
	/*if enclosion is not found in the existed base-pairs*/
	/*add directly*/
	if(i == sizeuparray[index2])	{
		uparray[index2][sizeuparray[index2] ++] = index1;
	}
	downarray[index1][sizedownarray[index1] ++] = index2;
	return;
}

/*
 * BuildArrays
 *
 * The input is pairs, number of pairs, four arrays to store the result (downarray, downarray size, uparray and uparray size)
 * BuildArrays build arrays according to their relative enclosing relationships
 * uparray records the pairs that enclosing the pair
 * downarray records the pairs that enclosed by the pair
*/
void BuildArrays(int *pairs, int num_pairs, int **downarray, int *sizedownarray, int **uparray, int *sizeuparray){
	int	i, j, k, l;
	int	shift1, shift2;
	for(i = 0; i < num_pairs; i ++)	{
		for(j = i + 1; j < num_pairs; j ++)	{
			/* if the ith base-pair is enclosing the jth base-pair */
			if(pairs[6 * i] <= pairs[6 * j] && pairs[6 * i + 3] >= pairs[6 * j + 3])	{
				/*then separate them into two arrays*/
				AddPair(pairs, i, j, downarray, sizedownarray, uparray, sizeuparray);
			} else if(pairs[6 * j] > pairs[6 * i + 3])	{
				/* if already go out of range */
				break;
			}
		}
	}
	return;
}

/*
 * BuildLastlist
 *
 * The function takes the base-pair array, downarray , sizedownarray and two arrays for storing results as input
 * The function builds up the list need to be checked in dynamic programming process
 * The function added the base-pairs that are possible positional preceed a base-pair to that base-pair
 * Called or used by function 'CalChainScore'
*/
void BuildLastlist(int *pairs, int *downarray, int sizedownarray, int **lastlist, int *sizelist){
	int	i, j, k, l;
	int	*rightmost;

	if(sizedownarray  == 0)	return;
	rightmost = (int *) calloc(sizedownarray, sizeof(int));
	/* initialize right most index of the chain to the last index of the current chain*/
	k = pairs[6 * downarray[sizedownarray - 1] + 3];
	for(i = 0; i < sizedownarray; i ++){
		rightmost[i] = k;
	}
	/* adding base-pairs before one base-pair to the lastlist of it*/
	for(i = 0; i < sizedownarray; i ++){
		/* k is the right index of the current base-pair*/
		k = pairs[6 * downarray[i] + 3];
		/* add the current base-pair to base-pairs that closely next to it in all levels*/
		for(j = i + 1; j < sizedownarray; j ++){
			/* if the base-pair is juxtapose to the latest added base-pair, continue adding lastlist of the next base-pair*/
			/* break j loop and go back to i loop*/
			if(pairs[6 * downarray[j]] >= rightmost[i]){
				break;
			}else{
				/* do not add thec++ bool current base-pair to those base-pairs that are enclosed by the current base-pair*/
				if(pairs[6 * downarray[j]] >= k){
					/* add to last list*/
					lastlist[j][sizelist[j] ++] = i;
					/* shrink the right most index back to the right index of the base-pair closely next ot the current base-pair*/
					if((l = pairs[6 * downarray[j] + 3]) < rightmost[i]){
						rightmost[i] = l;
						
					}
				}
			}
		}
	}
	free((void *) rightmost);
}

/*
 * GetLastList
 *
 * The function retrieves the indices of the base-pairs that have the highest score
 * Note that this function does not consider the base-pairs that are the same as the current base-pair
 * The reason is that this situation has been already considered priori calling this function
 * Separation of the retrieval due to complexity reason
 * The function takes the current base-pair1 index, current base-pair2 index, optimal i index, optimal j index, and an array 
 * for storing output as input
 * The result is finally stored in the array, and no explicit return value
*/
void GetLastlist(int currentIndex1, int currentIndex2, int **ilast, int **jlast, int *index2){
	/* ilast is the last i index of the base-pair that constructing the matched base-pair chain of optimal score in the region*/
	/* jlast is the last j index of the base-pair that constructing the matched base-pair chain of optimal score in the region*/
	int i, j;
	/* gets the index of the highest score of pairing two base-pairs that are different from both current base-pairs*/
	do{
		/* sets current base-pair*/
		i = currentIndex1;
		j = currentIndex2;
		/* find the index of the optimal score*/
		currentIndex1 = ilast[i][j];
		currentIndex2 = jlast[i][j];
	/* check whether the base-pair are the same as the current base-pair*/
	} while(currentIndex1 == i || currentIndex2 == j);
	/* puts the result into index2*/
	index2[0] = i;
	index2[1] = j;
	return;
}

/*
 * AddLevel
 *
 * The input is the level we are adding, index of the pair, uparray, sizeuparray, and the array recording each pair's level
 * AddLevel is a function called by BuildLevels
 * It recursively addes pairs into different array, according to their enclosing relationship
 * The result is recorded in array 'label'
*/
void AddLevel(int level, int index, int **uparray, int *sizeuparray, int *label){
	/*add all conflicting stacks to the next level*/
	int	i, j, k;
	int	maxlevel;
	
	/*if current stack is not in the last level, put it in the last level*/
	if(level > label[index])	{
		label[index] = level;
	}
	level ++;
	for(i = 0; i < sizeuparray[index]; i ++)	{
		/*recursively add conflicting stacks*/
		AddLevel(level, uparray[index][i], uparray, sizeuparray, label);
	}
	level --;
	return;
}

/*
 * BuildLevels
 *
 * The input is pairs (actually I think I can remove it), number of pairs, downarray size, uparray, uparray size, array storing pairs levelwise
 * and finally the size of each level
 * BuildLevels builds levels according to their enclosing relationship
 * The number of levels is returned
*/
int BuildLevels(int *pairs, int numPairs, int *sizedownarray, int **uparray, int *sizeuparray, int **levelindex, int *sizelevel){
	int	i, j, k;
	int	level;
	/*label indicating each preicted stack belongs to which level*/
	int	*label;

	label = (int *) calloc(numPairs, sizeof(int));
	/*if the stack dose not conflicting with any stacks with larger index*/
	for(i = 0; i < numPairs; i ++)	{
		if(sizedownarray[i] == 0)	{
			/*levels added to label; label indicates each base-pair belongs to which level*/
			AddLevel(0, i, uparray, sizeuparray, label);
		}
	}
	/*copy the base-pairs according to label into levelindex*/
	for(i = 0; i < numPairs; i ++)	{
		/*adding information to levelindex*/
		levelindex[label[i]][sizelevel[label[i]] ++] = i;
	}

	/*finds how many levels are created*/
	level = 0;
	for(i = 0; i < numPairs; i ++)	{
		if(sizelevel[i] == 0)	{
			/*record how many levels are here*/
			level = i;
			break;
		}
	}
	free((void *) label);
	/*if there are no levels*/
	if(level == 0)	{
		/*then put each stack in one level*/
		level = numPairs;
	}
	return(level);
}


