/*
 * BuildLastlist
 *
 * The function takes the base-pair array, downarray , sizedownarray and two arrays for storing results as input
 * The function builds up the list need to be checked in dynamic programming process
 * The function added the base-pairs that are possible positional preceed a base-pair to that base-pair
 * Called or used by function 'CalChainScore'
*/
void BuildLastlist(int *pairs, int *downarray, int sizedownarray, int **lastlist, int *sizelist){
	int	i, j, k, l;
	int	*rightmost;

	if(sizedownarray  == 0)	return;
	rightmost = (int *) malloc(sizedownarray * sizeof(int));
	/* initialize right most index of the chain to the last index of the current chain*/
	k = pairs[6 * downarray[sizedownarray - 1] + 3];
	for(i = 0; i < sizedownarray; i ++){
		rightmost[i] = k;
	}
	/* adding base-pairs before one base-pair to the lastlist of it*/
	for(i = 0; i < sizedownarray; i ++){
		/* k is the right index of the current base-pair*/
		k = pairs[6 * downarray[i] + 3];
		/* add the current base-pair to base-pairs that closely next to it in all levels*/
		for(j = i + 1; j < sizedownarray; j ++){
			/* if the base-pair is juxtapose to the latest added base-pair, continue adding lastlist of the next base-pair*/
			/* break j loop and go back to i loop*/
			if(pairs[6 * downarray[j]] >= rightmost[i]){
				break;
			}else{
				/* do not add the current base-pair to those base-pairs that are enclosed by the current base-pair*/
				if(pairs[6 * downarray[j]] >= k){
					/* add to last list*/
					lastlist[j][sizelist[j] ++] = i;
					/* shrink the right most index back to the right index of the base-pair closely next ot the current base-pair*/
					if((l = pairs[6 * downarray[j] + 3]) < rightmost[i]){
						rightmost[i] = l;
					}
				}
			}
		}
	}
	free((void *) rightmost);
}

/*
 * GetLastList
 *
 * The function retrieves the indices of the base-pairs that have the highest score
 * Note that this function does not consider the base-pairs that are the same as the current base-pair
 * The reason is that this situation has been already considered priori calling this function
 * Separation of the retrieval due to complexity reason
 * The function takes the current base-pair1 index, current base-pair2 index, optimal i index, optimal j index, and an array 
 * for storing output as input
 * The result is finally stored in the array, and no explicit return value
*/
void GetLastlist(int currentIndex1, int currentIndex2, int **ilast, int **jlast, int *index2){
	/* ilast is the last i index of the base-pair that constructing the matched base-pair chain of optimal score in the region*/
	/* jlast is the last j index of the base-pair that constructing the matched base-pair chain of optimal score in the region*/
	int i, j;
	/* gets the index of the highest score of pairing two base-pairs that are different from both current base-pairs*/
	do{
		/* sets current base-pair*/
		i = currentIndex1;
		j = currentIndex2;
		/* find the index of the optimal score*/
		currentIndex1 = ilast[i][j];
		currentIndex2 = jlast[i][j];
	/* check whether the base-pair are the same as the current base-pair*/
	} while(currentIndex1 == i || currentIndex2 == j);
	/* puts the result into index2*/
	index2[0] = i;
	index2[1] = j;
	return;
}
