#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "./inc/ComputeScore.h"
#include "./inc/ALIGN.h"
#include "./inc/ReadInput.h"
#include "./inc/RetrieveScore.h"
#include "./inc/OutputMPairs.h"

extern int W[15][15];

int AssignPairs(char *seq_b1, char *seq_b2, int *pair1, int *pair2, int numPairs, int *OutputIndex, char **OutLayers, char **OutEdges1, char **OutEdges2, char **OutIsos, char symbol1, char symbol2, int startLayer){
	char EdgeHash[4] = {'N', 'W', 'H', 'S'};
	int i, j, k;	
	int currentLayer;
	int maxLayer;
	maxLayer = startLayer;
	for(i=0; i<numPairs; ++i){
		currentLayer = startLayer;
		while(OutLayers[currentLayer][OutputIndex[pair1[6 * i]]] != '.' || OutLayers[currentLayer][OutputIndex[pair1[6 * i + 3]]] != '.'){
			++currentLayer;
			if(currentLayer >= 10){
				printf("Warnning: number of layers exceeds maximum layer! Base-pair may lost\n");
				break;
			}
		}
		if(maxLayer < currentLayer){
			maxLayer = currentLayer;
		}
		OutLayers[currentLayer][OutputIndex[pair1[6 * i]]] = symbol1;
		OutLayers[currentLayer][OutputIndex[pair1[6 * i + 3]]] = symbol2;
		if(pair1[6 * i + 2] == 0){
			/* cis for upper case letters*/
			OutEdges1[currentLayer][OutputIndex[pair1[6 * i]]] = EdgeHash[pair1[6 * i + 1]];
			OutEdges1[currentLayer][OutputIndex[pair1[6 * i + 3]]] = EdgeHash[pair1[6 * i + 4]];
		}else if(pair1[6 * i + 2] == 1){
			/* trans for lower case letters*/
			OutEdges1[currentLayer][OutputIndex[pair1[6 * i]]] = tolower(EdgeHash[pair1[6 * i + 1]]);
			OutEdges1[currentLayer][OutputIndex[pair1[6 * i + 3]]] = tolower(EdgeHash[pair1[6 * i + 4]]);
		}
		if(pair2[6 * i + 2] == 0){
			/* cis for upper case letters*/
			OutEdges2[currentLayer][OutputIndex[pair1[6 * i]]] = EdgeHash[pair2[6 * i + 1]];
			OutEdges2[currentLayer][OutputIndex[pair1[6 * i + 3]]] = EdgeHash[pair2[6 * i + 4]];
		}else if(pair2[6 * i + 2] == 1){
			/* trans for lower case letters*/
			OutEdges2[currentLayer][OutputIndex[pair1[6 * i]]] = tolower(EdgeHash[pair2[6 * i + 1]]);
			OutEdges2[currentLayer][OutputIndex[pair1[6 * i + 3]]] = tolower(EdgeHash[pair2[6 * i + 4]]);
		}
		if(IsSameIsoGroup(seq_b1, seq_b2, &pair1[6 * i], &pair2[6 * i])){
			OutIsos[currentLayer][OutputIndex[pair1[6 * i]]] = '*';
			OutIsos[currentLayer][OutputIndex[pair1[6 * i + 3]]] = '*';
		}else{
			OutIsos[currentLayer][OutputIndex[pair1[6 * i]]] = '+';
			OutIsos[currentLayer][OutputIndex[pair1[6 * i + 3]]] = '+';
		}		
	}
	/* shift 1 because the layers inside the program are indexed starting from 0*/
	return maxLayer+1;
}

int AlignIntervals(int startMotif, int endMotif, int startTarget, int endTarget, int OutTag, char *seq1, int seqLeng1, char *seq2, int seqLeng2, char *OutMotif, char *OutSymbol, char *OutTarget){
	char *tempSeq1, *tempSeq2, *tempOutMotif, *tempOutSymbol, *tempOutTarget;
	int tempSeqLeng1, tempSeqLeng2;
	int *AlignOps;
	int l;
	/* allocate memories*/
	tempSeq1 = (char *) calloc(seqLeng1, sizeof(char));
	tempSeq2 = (char *) calloc(seqLeng2, sizeof(char));
	/* gets the sequence need to be aligned*/
	tempSeqLeng1 = 0;
	tempSeqLeng2 = 0;
	for(l=startMotif+1; l<endMotif; ++l){
		tempSeq1[tempSeqLeng1] = Char2int(seq1[l]);
		++tempSeqLeng1;
	}
	for(l=startTarget+1; l<endTarget; ++l){
		tempSeq2[tempSeqLeng2] = Char2int(seq2[l]);
		++tempSeqLeng2;
	}
	/* allocate memories for storage of alignment results*/
	AlignOps = (int *) calloc((tempSeqLeng1 + tempSeqLeng2 + 1), sizeof(int));
	tempOutMotif = (char *) calloc((tempSeqLeng1 + tempSeqLeng2 + 1), sizeof(char));
	tempOutSymbol = (char *) calloc((tempSeqLeng1 + tempSeqLeng2 + 1), sizeof(char));
	tempOutTarget = (char *) calloc((tempSeqLeng1 + tempSeqLeng2 + 1), sizeof(char));
	/*
	printf("call of AlignIntervals\n");
	for(l=0; l<tempSeqLeng1; ++l){
		printf("%d", tempSeq1[l]);
	}
	printf("\n");
	for(l=0; l<tempSeqLeng2; ++l){
		printf("%d", tempSeq2[l]);
	}
	printf("\n");
	*/
	/* do the alignment*/
	ALIGN(&tempSeq1[-1], &tempSeq2[-1], tempSeqLeng1, tempSeqLeng2, W, rGAPST(), rGAPET(), AlignOps);
	/* do the trace back*/
	TraceBack(&tempSeq1[-1], &tempSeq2[-1], tempSeqLeng1, tempSeqLeng2, AlignOps, tempOutMotif, tempOutSymbol, tempOutTarget);
	l=0;
	/* copy results*/
	while(tempOutSymbol[l] != '\0'){
		OutMotif[OutTag] = tempOutMotif[l];
		OutSymbol[OutTag] = tempOutSymbol[l];
		OutTarget[OutTag] = tempOutTarget[l];
		++OutTag;
		++l;
	}
	free((void *) tempSeq1);
	free((void *) tempSeq2);
	free((void *) AlignOps);
	free((void *) tempOutMotif);
	free((void *) tempOutSymbol);
	free((void *) tempOutTarget);
	return OutTag;
}
/*
 *	OutputMPairs
 *
 * 	The function is used as the output module of the program
 * 	It takes the matched basepairs, and the array indicating the basepair's location as input,
 *	and prints out the result
 *
 *
*/
void OutputMPairs(	/* information about the two sequences, the sequence, the name of the sequence and the length of the sequence*/	
			char *seq1, char *seqname1, int seqLeng1, 
			char *seq2, char *seqname2, int seqLeng2, 
			/* information of the pairs, the two pair locations, number of pairs and crossing pairs indices and count*/
			int *pair1, int *pair2, int numPair1, int numPair2, int *crosspairings, int numCross,
			/* information of matched pairs, match contains score and number of total matched pairs
			   numPrematch is the matched nested pairs, matchedpFin(s) are matched pairs' indices*/ 
			MATCH *match, int numPrematch, int *matchedpFin1, int *matchedpFin2,
			/* information of the cleavage sites*/
			int *Cleavages, int numCleavage){
	
	if(numPrematch <= 0){
		return;
	}
	/* the arrays that hold the result to be printed*/
	char *OutMotif, *OutSymbol, *OutTarget;
	/* the location of the nucleotides after alignment (in OutMotif)*/
	
	int numLayers;
	int OutTag = 0;
	
	int i, j, k, l;
	int index1Motif, index2Motif, index1Target, index2Target;
	int numSegments = 0;
	char EdgeHash[4] = {'N', 'W', 'H', 'S'};
	char *seq_b1, *seq_b2;
	seq_b1 = (char *) calloc(seqLeng1, sizeof(char));
	seq_b2 = (char *) calloc(seqLeng2, sizeof(char));
	for(i=0; i<seqLeng1; ++i){
		seq_b1[i] = Char2int(seq1[i]);
	}
	for(i=0; i<seqLeng2; ++i){
		seq_b2[i] = Char2int(seq2[i]);
	}
	/* allocate memory for OutMotif, OutSymbol and OutTarget*/
	OutMotif = (char *) calloc(2 * (seqLeng1 + seqLeng2 + 1), sizeof(char));
	OutSymbol = (char *) calloc(2 * (seqLeng1 + seqLeng2 + 1), sizeof(char));
	OutTarget = (char *) calloc(2 * (seqLeng1 + seqLeng2 + 1), sizeof(char));
	/* check if there are some base-pair matched*/
	/* allocate memory for segMotif and segTarget*/
	int *segMotif;			/* indicating segment of motif*/
	int *segTarget;			/* indicating segments of target*/
	segMotif = (int *) calloc(seqLeng1, sizeof(int));
	segTarget = (int *) calloc(seqLeng2, sizeof(int));
	/* cuts the sequence into segments based on the matched basepair's end points*/
	for(i=0; i<match->numbasepair; ++i){
		segMotif[matchedpFin1[6 * i]] = 1;
		segMotif[matchedpFin1[6 * i + 3]] = 1;
		segTarget[matchedpFin2[6 * i]] = 1;
		segTarget[matchedpFin2[6 * i + 3]] = 1;
	}

	/* gets how many segments are there is the two sequences*/
	for(i=0; i<seqLeng1; ++i){
		if(segMotif[i] == 1){
			++numSegments;
		}
	}
	
	/* finds the locations of the endpoints*/
	int *intervalMotif, *intervalTarget;
	intervalMotif = (int *) calloc(numSegments, sizeof(int));
	intervalTarget = (int *) calloc(numSegments, sizeof(int));
	j=0;
	for(i=0; i<seqLeng1; ++i){
		if(segMotif[i] == 1){
			intervalMotif[j] = i;
			++j;
		}
	}
	j=0;
	for(i=0; i<seqLeng2; ++i){
		if(segTarget[i] == 1){
			intervalTarget[j] = i;
			++j;
		}
	}

	/* check whether the pairing start from the first nucleotide*/
	if(intervalMotif[0] > 0 || intervalTarget[0] > 0){
		OutTag = AlignIntervals(-1, intervalMotif[0], -1, intervalTarget[0], OutTag, seq1, seqLeng1, seq2, seqLeng2, OutMotif, OutSymbol, OutTarget);
	}
	/* for each segment*/
	for(i=0; i<numSegments-1; ++i){
		index1Motif = intervalMotif[i];
		index2Motif = intervalMotif[i+1];
		index1Target = intervalTarget[i];
		index2Target = intervalTarget[i+1];
		/* if there is no sequence inside this interval*/
		if(index2Motif - index1Motif == 1 && index2Target - index1Target == 1){
			/* directly add the nucleotide specified by index1Motif and index1Target to the output array*/
			OutMotif[OutTag] = seq1[index1Motif];
			OutTarget[OutTag] = seq2[index1Target];
			OutSymbol[OutTag] = '|';
			++OutTag;
		}else{
			/* add the former endpoint pair first*/
			OutMotif[OutTag] = seq1[index1Motif];
			OutTarget[OutTag] = seq2[index1Target];
			OutSymbol[OutTag] = '|';
			++OutTag;
			OutTag = AlignIntervals(index1Motif, index2Motif, index1Target, index2Target, OutTag, 
				seq1, seqLeng1, seq2, seqLeng2, OutMotif, OutSymbol, OutTarget);
		}
	}
	/* adds the last endpoint*/
	OutMotif[OutTag] = seq1[index2Motif];
	OutTarget[OutTag] = seq2[index2Target];
	OutSymbol[OutTag] = '|';
	++OutTag;
	/* check whether the endpoint is the last nucleotide in the sequence*/
	if(index2Motif+1 < seqLeng1 || index2Target+1 < seqLeng2){
		OutTag = AlignIntervals(index2Motif, seqLeng1, index2Target, seqLeng2, OutTag, 
			seq1, seqLeng1, seq2, seqLeng2, OutMotif, OutSymbol, OutTarget);
	}
	
	/* adds the basepairs*/
	/* builds the OutputIndex*/
	int *OutputIndex;
	OutputIndex = (int *) calloc(seqLeng1, sizeof(int));
	j=0;
	for(i=0; i<OutTag; ++i){
		if(OutMotif[i] == '-'){
			continue;
		}else{
			OutputIndex[j] = i;
			++j;
		}
	}
	char **OutLayers, **OutIsos;
	char **OutEdges1, **OutEdges2;
	/* allocate memory for OutLayers, maximum 10 layers*/
	OutLayers = (char **) calloc(10, sizeof(char *));
	OutEdges1 = (char **) calloc(10, sizeof(char *));
	OutEdges2 = (char **) calloc(10, sizeof(char *));
	OutIsos = (char **) calloc(10, sizeof(char *));
	for(i=0; i<10; ++i){
		OutLayers[i] = (char *) calloc(OutTag, sizeof(char));
		OutEdges1[i] = (char *) calloc(OutTag, sizeof(char));
		OutEdges2[i] = (char *) calloc(OutTag, sizeof(char));
		OutIsos[i] = (char *) calloc(OutTag, sizeof(char));
	}
	/* initialize the values*/
	for(i=0; i<10; ++i){
		/* sets the Output contents to '.'*/
		for(j=0; j<OutTag; ++j){
			OutLayers[i][j] = '.';
			OutEdges1[i][j] = '.';
			OutEdges2[i][j] = '.';
			OutIsos[i][j] = '.';
		}
	}
	/* filling the OutLayers with after check matched basepairs*/
	numLayers = AssignPairs(seq_b1, seq_b2, matchedpFin1, matchedpFin2, numPrematch, 
			OutputIndex, OutLayers, OutEdges1, OutEdges2, OutIsos, '(', ')', 0);
	if(numPrematch < match->numbasepair){	
		numLayers = AssignPairs(seq_b1, seq_b2, &matchedpFin1[6 * numPrematch], &matchedpFin2[6 * numPrematch], match->numbasepair - numPrematch, 
			OutputIndex, OutLayers, OutEdges1, OutEdges2, OutIsos, '<', '>', numLayers);
	}
	/* change the OutSymbols*/
	for(i=0; i<match->numbasepair; ++i){
		if(!IsSameIsoGroup(seq_b1, seq_b2, &matchedpFin1[6 * i], &matchedpFin2[6 * i])){
			/* meas they are not isosteric*/
			OutSymbol[OutputIndex[matchedpFin1[6 * i]]] = '+';
			OutSymbol[OutputIndex[matchedpFin1[6 * i + 3]]] = '+';
		}
	}
	for(i=0; i<match->numbasepair; ++i){
		if(IsSameIsoGroup(seq_b1, seq_b2, &matchedpFin1[6 * i], &matchedpFin2[6 * i])){
			if(OutSymbol[OutputIndex[matchedpFin1[6 * i]]] == '+' || OutSymbol[OutputIndex[matchedpFin1[6 * i + 3]]] == '+'){
				OutSymbol[OutputIndex[matchedpFin1[6 * i]]] = '+';
				OutSymbol[OutputIndex[matchedpFin1[6 * i + 3]]] = '+';
			}else{
				OutSymbol[OutputIndex[matchedpFin1[6 * i]]] = '*';
				OutSymbol[OutputIndex[matchedpFin1[6 * i + 3]]] = '*';
			}
		}
	}
	/* add the cleavage information into*/
	char *cOutMotif = (char *) calloc(OutTag + numCleavage, sizeof(char));
	char *cOutTarget = (char *) calloc(OutTag + numCleavage, sizeof(char));
	char *cOutSymbol = (char *) calloc(OutTag + numCleavage, sizeof(char));
	char **cOutIsos = (char **) calloc(10, sizeof(char *));
	char **cOutEdges1 = (char **) calloc(10, sizeof(char *));
	char **cOutEdges2 = (char **) calloc(10, sizeof(char *));
	char **cOutLayers = (char **) calloc(10, sizeof(char *));
	for(i=0; i<numLayers; ++i){
		cOutIsos[i] = (char *) calloc(OutTag + numCleavage, sizeof(char));
		cOutEdges1[i] = (char *) calloc(OutTag + numCleavage, sizeof(char));
		cOutEdges2[i] = (char *) calloc(OutTag + numCleavage, sizeof(char));
		cOutLayers[i] = (char *) calloc(OutTag + numCleavage, sizeof(char));
	}
	int cTag = 0;
	for(i=0; i<OutTag; ++i){
		/* check for whether there is a cleavage site*/
		for(j=0; j<numCleavage; ++j){
			if(i == OutputIndex[Cleavages[j]]){
				/* add a space for every output*/
				cOutMotif[cTag] = '.';
				cOutTarget[cTag] = '.';
				cOutSymbol[cTag] = '.';
				for(k=0; k<numLayers; ++k){
					cOutIsos[k][cTag] = '.';
					cOutEdges1[k][cTag] = '.';
					cOutEdges2[k][cTag] = '.';
					cOutLayers[k][cTag] = '.';
				}
				++cTag;
				break;
			}
		}
		cOutMotif[cTag] = OutMotif[i];
		cOutTarget[cTag] = OutTarget[i];
		cOutSymbol[cTag] = OutSymbol[i];
		for(k=0; k<numLayers; ++k){
			cOutIsos[k][cTag] = OutIsos[k][i];
			cOutEdges1[k][cTag] = OutEdges1[k][i];
			cOutEdges2[k][cTag] = OutEdges2[k][i];
			cOutLayers[k][cTag] = OutLayers[k][i];
		}
		++cTag;
	}

	/* Output the result*/
	printf("\nAligning:\nQuery:	%s\nTarget:	%s\n\n", seqname1, seqname2);
	k = 0;
	while(k < OutTag+numCleavage){
		int outlength = ((OutTag + numCleavage - k)>70) ? (k + 70) : (OutTag + numCleavage);
		/* reverse printing the OutLayers*/
		for(i=numLayers-1; i>=0; --i){
			printf("iso	");
			for(j=k; j<outlength; ++j){
				printf("%c", cOutIsos[i][j]);
			}
			printf("\n");
			printf("edge	");
			for(j=k; j<outlength; ++j){
				printf("%c", cOutEdges1[i][j]);
			}
			printf("\n");
			printf("struc	");
			for(j=k; j<outlength; ++j){
				printf("%c", cOutLayers[i][j]);
			}
			printf("\n\n");
		}
		/* prints the sequences*/
		printf("	");
		for(i=k; i<outlength; ++i){
			printf("%c", cOutMotif[i]);
		}
		printf("\n");
		printf("	");
		for(i=k; i<outlength; ++i){
			printf("%c", cOutSymbol[i]);
		}
		printf("\n");
		printf("	");
		for(i=k; i<outlength; ++i){
			printf("%c", cOutTarget[i]);
		}
		printf("\n");
		printf("\n");
		/* prints the OutLayers*/
		for(i=0; i<numLayers; ++i){
			printf("struc	");
			for(j=k; j<outlength; ++j){
				printf("%c", cOutLayers[i][j]);
			}
			printf("\n");
			printf("edge	");
			for(j=k; j<outlength; ++j){
				printf("%c", cOutEdges2[i][j]);
			}
			printf("\n");
			printf("iso	");
			for(j=k; j<outlength; ++j){
				printf("%c", cOutIsos[i][j]);
			}
			printf("\n\n");
		}
		k += 70;
	}
	free((void *) seq_b1);
	free((void *) seq_b2);
	free((void *) OutMotif);
	free((void *) OutSymbol);
	free((void *) OutTarget);
	free((void *) cOutMotif);
	free((void *) cOutSymbol);
	free((void *) cOutTarget);
	free((void *) segMotif);
	free((void *) segTarget);
	free((void *) OutputIndex);
	free((void *) intervalMotif);
	free((void *) intervalTarget);
	for(i=0; i<10; ++i){
		free((void *) OutLayers[i]);
		free((void *) OutEdges1[i]);
		free((void *) OutEdges2[i]);
		free((void *) OutIsos[i]);
		free((void *) cOutLayers[i]);
		free((void *) cOutEdges1[i]);
		free((void *) cOutEdges2[i]);
		free((void *) cOutIsos[i]);
	}
	free((void **) OutLayers);
	free((void **) OutEdges1);
	free((void **) OutEdges2);
	free((void **) OutIsos);
	free((void **) cOutLayers);
	free((void **) cOutEdges1);
	free((void **) cOutEdges2);
	free((void **) cOutIsos);
	return;	
}
