#!/usr/bin/perl -w
use strict;
use Getopt::Long;
use Cwd 'abs_path';

use lib '/home/cuncong/RNAMotifScan_Ver_5_3';
use RNAMotifScan;
Getopt::Long::Configure ("bundling");

my $version = '5_3';

my @existingID;
foreach(<*.processID~>){
	$_ =~ /(\d+)\.processID~/;
	push @existingID, $1; 
}

my $processID;
my $conflictTag = 0;
while($conflictTag == 0){
	$conflictTag = 1;
	$processID = int(rand(10000));
	foreach(@existingID){
		if($processID == $_){
			$conflictTag = 0;
			last;
		}
	}
}
system "touch $processID.processID~";

my $queryFile = "NOTAFILE";	# default the input query file name
my $targetID = "NOTANID";	# default the PDB ID
my $outputFile = "NOTAFILE";	# default the output filename
my $pdbList = "NOTALIST";	# default the list to be null
my $reduced = 1;		# default to reduced display mode
my $aligned = 0;		# default not to print alignment
my $wholeScan = 0;		# default not to Scan whole PDB
my $cutoff = 0.01;		# default cutoff score to 0
my $antVersion = "MCAnnotate";	# default annotating program

sub printHelp(){
	print "Usage:	perl RNAMotifScan.pl -q QUERY_FILE [-t xxxx] [-l LIST_FILE] [-o OUTPUT_FILE] [-r] [-a] [-c]\n";
	print "q:	query file path (the query RNA motif to be searched)\n";
	print "t:	target PDB id (search one PDB file only, the files need to be in ./RNAinPDB, only 4-letter PDB ID is required)\n";
	print "v:	target PDB annotation program version, MC-Annotate or RNAVIEW (default MC-Annotate)\n";
	print "l:	input PDB ID list for scan (input the list of PDB file you want to search, the files need to be in ./RNAinPDB)\n";
	print "o:	output file path (the path of the file to be used to store result, default to STDOUT)\n";
	print "r:	reduced result output mode (without alignment, default)\n";
	print "a:	alignment result output mode (with alignment)\n";
	print "c:	false positive rate cutoff (program only outputs hits smaller than this threshold, default 0.01)\n";
	exit;
}


GetOptions(	"q=s" => \$queryFile,
		"t=s" => \$targetID,
		"v=s" => \$antVersion,
		"o=s" => \$outputFile,
		"l=s" => \$pdbList,
		"r" => \$reduced,
		"a" => \$aligned,
		"h" => \$wholeScan,
		"c=f" => \$cutoff,
		"version" => \&printVersion,
		"help" => \&printHelp,
);

if($queryFile ne 'NOTAFILE'){
	$queryFile = abs_path($queryFile);
}
if($outputFile ne 'NOTAFILE'){
	$outputFile = abs_path($outputFile);
}
if($pdbList ne 'NOTALIST'){
	$pdbList = abs_path($pdbList);
}


if(!(-e "./WorkSpace")){
	system "mkdir ./WorkSpace";
}

if($queryFile eq "NOTAFILE"){
	die "No query specified, type --help for usage instruction.\n";
}

if($targetID eq "NOTANID" && $pdbList eq "NOTALIST"){
	die "No target specified, type --help for usage instruction.\n";
}

my @toProcess;
if($targetID ne "NOTANID"){
	push @toProcess, lc($targetID);
}
if($pdbList ne "NOTALIST"){
	open my $LIST, "<$pdbList" or die "Cannot open PDB list: $!\n";
	foreach(<$LIST>){
		chomp;
		push @toProcess, lc($_);
	}
}

if(-e "./WorkSpace/$processID"){
	system "rm -r ./WorkSpace/$processID";	
}
mkdir "./WorkSpace/$processID" or die "Cannot make directory: $!\n";
RNAMotifScan::SetProcessID($processID);
RNAMotifScan::SetCutoff($cutoff);
RNAMotifScan::GetMotifInfo($queryFile);
my $numStrand = RNAMotifScan::ExNumStrand();
print "Simulating mean and deviation...\n";
RNAMotifScan::ComputeMeanDev($queryFile);
foreach(@toProcess){
	my $pdbID = $_;
	my $antDir = 'RNAinPDB_' . $antVersion;
	print "Processing $pdbID...\n";
	foreach(<./$antDir/*>){
		if($_ =~ /$pdbID/){
			my $filePath = abs_path($_);
			#print "	Cutting Structures...\n";
			if($numStrand == 1){
				RNAMotifScan::GenFragments_single($filePath);
			}elsif($numStrand == 2){
				RNAMotifScan::GenFragments_double($filePath);
			}
			#print "	Running Alignments...\n";
			RNAMotifScan::RunAlignment($queryFile);
			#print "	Analyzing Results...\n";
			RNAMotifScan::GetOptScore();
			RNAMotifScan::RemoveSegmentRedundancy();
		}
	}
}
print "Loading Alignment results...\n";
RNAMotifScan::SortShuffle();
#print "after sort shuffle\n";
#RNAMotifScan::ReadAlignment();
RNAMotifScan::SetPrintMode($aligned);
RNAMotifScan::RedoAlignment($queryFile);
#print "after redoalignment\n";
RNAMotifScan::PrintResult($outputFile);
#RNAMotifScan::PrintContent();
system "rm $processID.processID~";
system "rm -r ./WorkSpace/$processID";

