#!/usr/bin/perl -w
use strict;
use Cwd 'abs_path';

our $directory = "/home/cczhong/C_programs/RNAMotifScan_Ver_5_3";
$directory = abs_path($directory);

package RNAMotifScan;
require Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw(SetProcessID ExNumStrand GetMotifInfo GenFragments_single GenFragments_double GetOptScore ReadAlignment RemoveSegmentRedundancy ComputeMeanDev RedoAlignment PrintResult PrintContent);
our @EXPORT_OK = qw();
our $VERSION = 2.0;
our $processID = 0;
our $printMode = 0;			# defalut not to print the alignment 
our $windowSize = 0.2;			# default percentage of shift along the stack
our $nameMotif;				# name of the motif
our $lenMotif = 0;			# length of the motif sequence
our $numCrossing = 0;			# number of crossing base-pairs
our $numPairMotif = 0;			# number of base-pairs in the motif structure
our $cutoff = 0.1;			# pValue Cutoff
our $alignmentCount = 0;
our $numStrand;
our $mean;
our $squareDev;
our @shuffleScore;
our %originalAlignment;
our %rotatedAlignment;
our %optimalAlignment;
our %sortedAlignment;
our %storeAlignment;
our @sortedKeys;
#################################################
#	The function prototypes
#################################################

# set the process ID for the current job
sub SetProcessID($);

# set the printting mode
sub SetPrintMode($);

# export number of strands
sub ExNumStrand();

# generate fragments
sub GenFragments($);		# input must be absolute path, input is the PDB target file

# run alignment on structures cutted, cutted reversed, and cutted shuffled
sub RunAlignment($);		# input must be absolute path, input is the motif query file

# sort the shuffled structure's scores
sub SortShuffle();

# reads in the alignment results
sub ReadAlignment();

# get the optimal score from different strands permutation
sub GetOptScore();

# remove redundancy of the cutted structural segments
sub RemoveSegmentRedundancy();

# compute mean and deviation from cutted structures and cutted reversed structures
sub ComputeMeanDev($);

# redo the alignment
sub RedoAlignment($);

# output result
sub PrintResult($);

sub PrintContent();



#################################################
#	The actural function definition
#################################################

#################################################
#	Private functions
#################################################
# returns the maximum value
sub max(@){# gets the length of the sequence and number of base-pairs in the structure
	my $i;
	my $maxnum = $_[0];
	for($i = 0; $i < @_; ++ $i){
		if($_[$i] > $maxnum){
			$maxnum = $_[$i];
		}
	}
	return $maxnum;
}

# returns the minimum value
sub min(@){
	my $i;
	my $minnum = $_[0];
	for($i = 0; $i < @_; ++ $i){
		if($_[$i] < $minnum){
			$minnum = $_[$i];
		}
	}
	return $minnum;
}

# returns average of a set of numbers
sub center(@$){
	my $i;
	my $j;
	my $n;
	my $sum;
	my @numbers;
	my @temp;
	my $maxdeviation;
	my $numvariables = scalar(@_) - 1;
	for($i = 0; $i < $numvariables; ++ $i){
		push @numbers, shift;
	}
	my $interval = shift;
	my $preave = 0;
	my $newave = 100;
	while($preave != $newave && scalar(@numbers) > 1){
		$preave = $newave;
		$sum = 0;
		$n = 0;
		for($i = 0; $i < scalar(@numbers); ++ $i){
			$sum += $numbers[$i];
			++ $n;
		}
		$newave = $sum/$n;
		$j = 0;
		$maxdeviation = 0;
		for($i = 0; $i < scalar(@numbers); ++ $i){
			if(abs($numbers[$i] - $newave) > $interval){
				if(abs($numbers[$i] - $newave) > $maxdeviation){
					$maxdeviation = abs($numbers[$i] - $newave);
				}
			}
		}
		if($maxdeviation == 0){
			return $newave;
		}
		for($i = 0; $i < scalar(@numbers); ++ $i){
			if(abs($numbers[$i] - $newave) < $maxdeviation){
				$temp[$j++] = $numbers[$i];
			}
		}
		if($i == $j){
			return $newave;
		}
		@numbers = undef;
		for($i = 0; $i < scalar(@temp); ++ $i){
			$numbers[$i] = $temp[$i];
		}
		@temp = undef;
	}
	return $newave;
}

#################################################
#	public functions
#################################################
sub SetProcessID($){
	$processID = shift;
}

sub SetPrintMode($){
	$printMode = shift;
}

sub SetCutoff($){
	$cutoff = shift;
}

sub ExNumStrand(){
	return $numStrand;
}

#sub by_hash_value {$optimalAlignment{$a} <=> $optimalAlignment{$b}}
sub by_hash_value2 {$sortedAlignment{$a} <=> $sortedAlignment{$b}}

# gets the length of the sequence and number of base-pairs in the structure
sub GetMotifInfo($){
	my $file = shift;
	open my $motif, "<$file" or die "Cannot open file: $!\n";
	my $temp;
	$temp = <$motif>;
	chomp $temp;
	$temp =~ s/^>//g;
	$nameMotif = $temp;
	$temp = <$motif>;
	chomp $temp;
	$numStrand = 1;
	my $i;
	for($i = 0; $i < length($temp) - 1; ++ $i){
		if(substr($temp, $i, 1) eq '.' && substr($temp, $i + 1, 1) ne '.'){
			++ $numStrand;
		}
	}
	$temp =~ s/\.//g;
	$lenMotif = length($temp);
	$temp = <$motif>;
	$temp = <$motif>;
	my $j;
	chomp $temp;
	for($j = 0; $j < length($temp); ++ $j){
		my $d = substr($temp, $j, 1);
		if($d eq '('){
			++ $numPairMotif;
		}
	}
	$temp = <$motif>;
	while($temp = <$motif>){
		chomp $temp;
		if($temp =~ /^\@\@/){
			$temp = <$motif>;
			chomp $temp;
		}
		my $i;
		for($i = 0; $i < length($temp); ++ $i){
			my $c = substr($temp, $i, 1);
			if($c eq '(' || $c eq '<'){
				++ $numCrossing;
				++ $numPairMotif;
			}
		}
		$temp = <$motif>;
	}
	close $motif;
}

sub GenFragments_single($){
	my $fileTarget = shift;
	open Target, "<$fileTarget" or die "Cannot open file:$!\n";
	my $targetName;
	my $sequence;
	# gets the name and the sequence;
	$targetName = <Target>;
	chomp $targetName;
	$sequence = <Target>;
	chomp $sequence;
	<Target>;								# remove the '//'

	# gets the pairing information
	my @Pairings;
	my $i = 0;
	my $j = 0;
	my $k = 0;
	my $l = 0;
	while(<Target>){
		if($_ =~ /^\#	(\d+)\-(\d+) \: (\w)\-(\w) (\w)\/(\w) (\w+) (.+)\-(.+)/){
			#print "$1	$2	$3	$4	$5	$6	$7	$8	$9\n";
			$Pairings[$i][0] = $1;
			$Pairings[$i][1] = $2;
			$Pairings[$i][2] = $3;
			$Pairings[$i][3] = $4;
			$Pairings[$i][4] = $5;
			$Pairings[$i][5] = $6;
			$Pairings[$i][6] = $7;
			$Pairings[$i][7] = $8;
			$Pairings[$i][8] = $9;
			++ $i;
		}
	}
	# format the base-pairs
	for($i = 0; $i < scalar(@Pairings); ++ $i){
		if($Pairings[$i][0] > $Pairings[$i][1]){
			my $temp;
			$temp = $Pairings[$i][0];
			$Pairings[$i][0] = $Pairings[$i][1];
			$Pairings[$i][1] = $temp;
			$temp = $Pairings[$i][2];
			$Pairings[$i][2] = $Pairings[$i][3];
			$Pairings[$i][3] = $temp;
			$temp = $Pairings[$i][4];
			$Pairings[$i][4] = $Pairings[$i][5];
			$Pairings[$i][5] = $temp;
			$temp = $Pairings[$i][7];
			$Pairings[$i][7] = $Pairings[$i][8];
			$Pairings[$i][8] = $temp;
		}
	}
	# sort the formatted base-pairs
	for($i = 0; $i < scalar(@Pairings); ++ $i){
		for($j = $i + 1; $j < scalar(@Pairings); ++ $j){
			if($Pairings[$i][0] > $Pairings[$j][0] || ($Pairings[$i][0] == $Pairings[$j][0] && $Pairings[$i][1] > $Pairings[$j][1])){
				my @temp;				
				for($k = 0; $k < 9; ++ $k){
					$temp[$k] = $Pairings[$i][$k];
				}
				for($k = 0; $k < 9; ++ $k){
					$Pairings[$i][$k] = $Pairings[$j][$k];
				}
				for($k = 0; $k < 9; ++ $k){
					$Pairings[$j][$k] = $temp[$k];
				}
			}
		}
	} 
	#for($i = 0; $i <  scalar(@Pairings); ++ $i){
	#	for($j = 0; $j < 9; ++ $j){
	#		print "$Pairings[$i][$j]	";
	#	}
	#	print "\n";
	#}
	my $lenCut = int(1.5 * $lenMotif);
	my @pickedPairings;
	my $pickedTag = 0;
	my %pickedPSHash;
	# get the feasible segments from the structure
	for($i = 0; $i < length($sequence) - $lenCut + 1; $i += 1){
		my $leftIndex = $i;
		my $rightIndex = $i + $lenCut - 1;
		# remove those pairs that are too far away from the center
		my $takenTag = 0;
		$k = 0;
		for($j = 0; $j < scalar(@Pairings); ++ $j){
			if($Pairings[$j][0] >= $leftIndex && $Pairings[$j][1] <= $rightIndex){
				for($l = 0; $l < 9; ++ $l){
					$pickedPairings[$pickedTag][$k][$l] = $Pairings[$j][$l];
				}
				++ $k;
				$takenTag = 1;
			}
		}
		if($takenTag == 1){
			$pickedPSHash{$pickedTag} = $i;
			++ $pickedTag;
		}
	}


	# remove redundant structures
	my %redundants;
	for($i = 0; $i < @pickedPairings - 1; ++ $i){
		my $minNumPair = min(scalar(@{$pickedPairings[$i]}), scalar(@{$pickedPairings[$i + 1]}));
		for($k = 0; $k <= abs(scalar(@{$pickedPairings[$i]}) - scalar(@{$pickedPairings[$i + 1]})); ++ $k){
			if(scalar(@{$pickedPairings[$i]}) >= scalar(@{$pickedPairings[$i + 1]})){
				for($j = 0; $j < $minNumPair; ++ $j){
					if($pickedPairings[$i][$j + $k][0] != $pickedPairings[$i+1][$j][0] || 
					$pickedPairings[$i][$j + $k][1] != $pickedPairings[$i+1][$j][1]){
					last;
					}
				}
			}else{
				for($j = 0; $j < $minNumPair; ++ $j){
					if($pickedPairings[$i][$j][0] != $pickedPairings[$i+1][$j + $k][0] || 
						$pickedPairings[$i][$j][1] != $pickedPairings[$i+1][$j + $k][1]){
						last;
					}
				}
			}
			if($j == $minNumPair){
				my $toRemove = ((scalar(@{$pickedPairings[$i]}) > scalar(@{$pickedPairings[$i + 1]})) ? ($i + 1) : ($i));
				$redundants{$toRemove} = 1;
			}
		}
		# if one segment is complete subset of another
	}

	my %nrPSHash;
	my @nrPairings;
	my $nrPairingsTag = 0;
	for($i = 0; $i < @pickedPairings; ++ $i){
		if(!(exists $redundants{$i})){
			for($j = 0; $j < @{$pickedPairings[$i]}; ++ $j){
				for($k = 0; $k < 9; ++ $k){
					$nrPairings[$nrPairingsTag][$j][$k] = $pickedPairings[$i][$j][$k];
				}
			}
			$nrPSHash{$nrPairingsTag} = $pickedPSHash{$i};
			++ $nrPairingsTag;
		}
	}
	if(!(-e "$directory/WorkSpace/$processID/CuttedStructs")){
		mkdir "$directory/WorkSpace/$processID/CuttedStructs" or die "Cannot make directory: $!\n";
	}
	
	chdir "$directory/WorkSpace/$processID/CuttedStructs" or die "Cannot change into directory: $!\n";
	my $minLeft;
	my $maxLeft;
	my $minRight;
	my $maxRight;
	my $naminLeft;
	my $namaxLeft;
	my $naminRight;
	my $namaxRight;
	$targetName =~ s/^>//g;
	for($i = 0; $i < scalar(@nrPairings); ++ $i){
		if(scalar(@{$nrPairings[$i]}) < 0.5 * $numPairMotif){
			next;
		}
		my @leftIndex;
		my @rightIndex;
		for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
			push @leftIndex, $nrPairings[$i][$j][0];
			push @rightIndex, $nrPairings[$i][$j][1];
		}
		$minLeft = min(@leftIndex);
		$maxRight = max(@rightIndex);
		$maxLeft = max(@leftIndex);
		$minRight = min(@rightIndex);
		for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
			if($nrPairings[$i][$j][0] == $minLeft){
				$naminLeft = $nrPairings[$i][$j][7];
			}
			if($nrPairings[$i][$j][0] == $maxLeft){
				$namaxLeft = $nrPairings[$i][$j][7];
			}
			if($nrPairings[$i][$j][1] == $minRight){
				$naminRight = $nrPairings[$i][$j][8];
			}
			if($nrPairings[$i][$j][1] == $maxRight){
				$namaxRight = $nrPairings[$i][$j][8];
			}
			#print "$nrPairings[$i][$j][0]	$nrPairings[$i][$j][1]	$nrPairings[$i][$j][2]	$nrPairings[$i][$j][3]\n";
		}
		my $outFilename;
		my $outSeq;

		# print the suffled version
		#print "shuffled\n";
		$outFilename = 'shuffled_' . $targetName . '_' . $minLeft . '_' . $maxLeft . '_' . $minRight . '_' . $maxRight . '.mca';
		#$outSeq = substr($sequence, $minLeft, $maxLeft - $minLeft + 1) . substr($sequence, $minRight, $maxRight - $minRight + 1);
		$outSeq = substr($sequence, $minLeft, $maxRight - $minLeft + 1);
		my @shuffledArray;
		my $m;
		my $n;
		for($m = 0; $m < length($outSeq); ++ $m){
			$shuffledArray[$m] = $m;
		}
		my @currentStack;
		for($m = 0; $m < scalar(@{$nrPairings[$i]}); ++ $m){
			for($n = 0; $n < 9; ++ $n){
				$currentStack[$m][$n] = $nrPairings[$i][$m][$n];
			}
		}
		for($j = 0; $j < scalar(@currentStack); ++ $j){
			$currentStack[$j][0] -= $minLeft;
			$currentStack[$j][1] -= $minLeft;
			#$currentStack[$m][1] -= $minRight - $maxLeft + $minLeft - 1;
		}
		my $a = scalar(@currentStack);
		if($a <= 2){
			next;
		}
		for($m = 0; $m < scalar(@currentStack)/3 + 1; ++ $m){
			my $ranNum1 = int(rand(scalar(@currentStack)));
			my $ranNum2 = int(rand(scalar(@currentStack)));
			my $countWhile = 0;
			while($ranNum1 == $ranNum2 && $countWhile < 20){
				++ $countWhile;
				$ranNum2 = int(rand(scalar(@currentStack) - 1));
				if($ranNum2 < $ranNum1){
					;
				}else{
					++ $ranNum2;
				}
				if($currentStack[$ranNum1][0] == $currentStack[$ranNum2][1] || $currentStack[$ranNum1][1] == $currentStack[$ranNum2][0]){
					$ranNum2 = $ranNum1;
				}
			}
			# Swap the two base-pairs
			my @temp;
			for($n = 2; $n < 9; ++ $n){
				$temp[$n] = $currentStack[$ranNum1][$n];
			}
			for($n = 2; $n < 9; ++ $n){
				$currentStack[$ranNum1][$n] = $currentStack[$ranNum2][$n];
			}
			for($n = 2; $n < 9; ++ $n){
				$currentStack[$ranNum2][$n] = $temp[$n];
			}
			my $tempS1; my $tempS2;
			$tempS1 = $shuffledArray[$currentStack[$ranNum1][0]];
			$tempS2 = $shuffledArray[$currentStack[$ranNum1][1]];
			$shuffledArray[$currentStack[$ranNum1][0]] = $shuffledArray[$currentStack[$ranNum2][0]];
			$shuffledArray[$currentStack[$ranNum1][1]] = $shuffledArray[$currentStack[$ranNum2][1]];
			$shuffledArray[$currentStack[$ranNum2][0]] = $tempS1;
			$shuffledArray[$currentStack[$ranNum2][1]] = $tempS2;
			for($n = 0; $n < scalar(@currentStack); ++ $n){
				if($n != $ranNum1 && $n != $ranNum2){
					if($currentStack[$n][0] == $currentStack[$ranNum1][0]){
						$currentStack[$n][0] = $currentStack[$ranNum2][0];
					}elsif($currentStack[$n][0] == $currentStack[$ranNum2][0]){
						$currentStack[$n][0] = $currentStack[$ranNum1][0];
					}elsif($currentStack[$n][0] == $currentStack[$ranNum1][1]){
						$currentStack[$n][0] = $currentStack[$ranNum2][1];
					}elsif($currentStack[$n][0] == $currentStack[$ranNum2][1]){
						$currentStack[$n][0] = $currentStack[$ranNum1][1];
					}
					if($currentStack[$n][1] == $currentStack[$ranNum1][1]){
						$currentStack[$n][1] = $currentStack[$ranNum2][1];
					}elsif($currentStack[$n][1] == $currentStack[$ranNum2][1]){
						$currentStack[$n][1] = $currentStack[$ranNum1][1];
					}elsif($currentStack[$n][1] == $currentStack[$ranNum1][0]){
						$currentStack[$n][1] = $currentStack[$ranNum2][0];
					}elsif($currentStack[$n][1] == $currentStack[$ranNum2][0]){
						$currentStack[$n][1] = $currentStack[$ranNum1][0];
					}
				}
			}
		}
		# move single nucleotide
		for($m = 0; $m < scalar(@currentStack); ++ $m){
			my $clleft = -1; my $crleft = length($outSeq); my $clright = -1; my $crright = length($outSeq);
			for($n = 0; $n < scalar(@currentStack); ++ $n){
				if($currentStack[$n][0] < $currentStack[$m][0] && $currentStack[$n][0] > $clleft){
					$clleft = $currentStack[$n][0];
				}
				if($currentStack[$n][0] > $currentStack[$m][0] && $currentStack[$n][0] < $crleft){
					$crleft = $currentStack[$n][0];
				}
				if($currentStack[$n][0] < $currentStack[$m][1] && $currentStack[$n][0] > $clright){
					$clright = $currentStack[$n][0];
				}
				if($currentStack[$n][0] > $currentStack[$m][1] && $currentStack[$n][0] < $crright){
					$crright = $currentStack[$n][0];
				}
				
				if($currentStack[$n][1] < $currentStack[$m][0] && $currentStack[$n][1] > $clleft){
					$clleft = $currentStack[$n][1];
				}
				if($currentStack[$n][1] > $currentStack[$m][0] && $currentStack[$n][1] < $crleft){
					$crleft = $currentStack[$n][1];
				}
				if($currentStack[$n][1] < $currentStack[$m][1] && $currentStack[$n][1] > $clright){
					$clright = $currentStack[$n][1];
				}
				if($currentStack[$n][1] > $currentStack[$m][1] && $currentStack[$n][1] < $crright){
					$crright = $currentStack[$n][1];
				}
			}
			my $newleft = $clleft + int(rand($crleft - $clleft - 1)) + 1;
			my $newright = $clright + int(rand($crright - $clright - 1)) + 1;
			my $left = $currentStack[$m][0];
			my $right = $currentStack[$m][1];
			if($left == $newleft && $right == $newright){
				next;
			}
			for($n = 0; $n < scalar(@currentStack); ++ $n){
				if($currentStack[$n][0] == $left){
					$currentStack[$n][0] = $newleft;
				}elsif($currentStack[$n][0] == $newleft){
					$currentStack[$n][0] = $left;
				}elsif($currentStack[$n][0] == $right){
					$currentStack[$n][0] = $newright;
				}elsif($currentStack[$n][0] == $newright){
					$currentStack[$n][0] = $right;
				}

				if($currentStack[$n][1] == $left){
					$currentStack[$n][1] = $newleft;
				}elsif($currentStack[$n][1] == $newleft){
					$currentStack[$n][1] = $left;
				}elsif($currentStack[$n][1] == $right){
					$currentStack[$n][1] = $newright;
				}elsif($currentStack[$n][1] == $newright){
					$currentStack[$n][1] = $right;
				}
			}
			my $temp;
			$temp = $shuffledArray[$left];
			$shuffledArray[$left] = $shuffledArray[$newleft];
			$shuffledArray[$newleft] = $temp;
			$temp = $shuffledArray[$right];
			$shuffledArray[$right] = $shuffledArray[$newright];
			$shuffledArray[$newright] = $temp;
			
		}
		my @presentArray;
		for($m = 0; $m < length($outSeq); ++ $m){
			$presentArray[$m] = 0;
		}
		for($m = 0; $m < scalar(@currentStack); ++ $m){
			$presentArray[$currentStack[$m][0]] = 1;
			$presentArray[$currentStack[$m][1]] = 1;
		}
		my $upTag = 0;
		my %upHash;
		for($m = 0; $m < length($outSeq); ++ $m){
			if($presentArray[$m] == 0){
				$upHash{$upTag} = $m;
				++ $upTag;
			}
		}
		if($upTag > 1){
			for($m = 0; $m < $upTag/3 + 1; ++ $m){
				my $ranNum1 = int(rand($upTag));
				my $ranNum2 = int(rand($upTag));
				my $countWhile = 0;
				while($ranNum1 == $ranNum2 && $countWhile < 20){
					++ $countWhile;
					$ranNum2 = int(rand($upTag));
				}
				my $temp;
				$temp = $shuffledArray[$upHash{$ranNum1}];
				$shuffledArray[$upHash{$ranNum1}] = $shuffledArray[$upHash{$ranNum2}];
				$shuffledArray[$upHash{$ranNum2}] = $temp;
			}
		}
		my $newSeq = undef;
		my @sequence;
		@sequence = split //, $outSeq;
		for($m = 0; $m < length($outSeq); ++ $m){
			$newSeq .= $sequence[$shuffledArray[$m]];
		}
		open OUT, ">$outFilename" or die "Cannot create file: $!\n";
		print OUT ">shuffled\_$targetName\_$minLeft\_$maxLeft\_$minRight\_$maxRight\_\_$naminLeft\_$namaxLeft\_$naminRight\_$namaxRight\n";
		print OUT "$newSeq\n";
		print OUT "//\n";
		for($j = 0; $j < scalar(@currentStack); ++ $j){
			#print "$currentStack[$j][0]	$currentStack[$j][1]	$currentStack[$j][2]	$currentStack[$j][3]\n";
			my $left = $currentStack[$j][0];
			my $right = $currentStack[$j][1];
			if(substr($newSeq, $left, 1) ne $currentStack[$j][2] || 
				substr($newSeq, $right, 1) ne $currentStack[$j][3]){
				next;
			}
			if($left < $right){
				print OUT "#	$left-$right : $currentStack[$j][2]-$currentStack[$j][3] ";
				print OUT "$currentStack[$j][4]\/$currentStack[$j][5] $currentStack[$j][6]\n";
			}else{
				print OUT "#	$right-$left : $currentStack[$j][3]-$currentStack[$j][2] ";
				print OUT "$currentStack[$j][5]\/$currentStack[$j][4] $currentStack[$j][6]\n";
			}	
		}
		close OUT;

		# print the cutted, not reversed version
		#print "original\n";
		$outFilename = $targetName . '_' . $minLeft . '_' . $maxLeft . '_' . $minRight . '_' . $maxRight . '.mca';
		$outSeq = substr($sequence, $minLeft, $maxRight - $minLeft + 1);
		open OUT, ">$outFilename" or die "Cannot create file: $!\n";
		print OUT ">$targetName\_$minLeft\_$maxLeft\_$minRight\_$maxRight\_\_$naminLeft\_$namaxLeft\_$naminRight\_$namaxRight\n";
		print OUT "$outSeq\n";
		print OUT "//\n";
		for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
			#print "$nrPairings[$i][$j][0]	$nrPairings[$i][$j][1]\n";
			$nrPairings[$i][$j][0] -= $minLeft;
			$nrPairings[$i][$j][1] -= $minLeft;
			#print "$minLeft\n";
			if(substr($outSeq, $nrPairings[$i][$j][0], 1) ne $nrPairings[$i][$j][2] || 
				substr($outSeq, $nrPairings[$i][$j][1], 1) ne $nrPairings[$i][$j][3]){
			#	print "Indexing Error! two sides\n";
			}
			print OUT "#	$nrPairings[$i][$j][0]-$nrPairings[$i][$j][1] : $nrPairings[$i][$j][2]-$nrPairings[$i][$j][3] ";
			print OUT "$nrPairings[$i][$j][4]\/$nrPairings[$i][$j][5] $nrPairings[$i][$j][6]\n";	
		}
		close OUT;
	}	
	close Target;
	chdir "$directory" or die "Cannot change directory: $!\n";
}

sub GenFragments_double($){
	my $fileTarget = shift;
	open Target, "<$fileTarget" or die "Cannot open file:$!\n";
	my $targetName;
	my $sequence;
	# gets the name and the sequence;
	$targetName = <Target>;
	chomp $targetName;
	$sequence = <Target>;
	chomp $sequence;
	<Target>;								# remove the '//'

	# gets the pairing information
	my @Pairings;
	my $i = 0;
	my $j = 0;
	my $k = 0;
	my $l = 0;
	while(<Target>){
		if($_ =~ /^\#	(\d+)\-(\d+) \: (\w)\-(\w) (\w)\/(\w) (\w+) (.+)\-(.+)/){
			#print "$1	$2	$3	$4	$5	$6	$7	$8	$9\n";
			$Pairings[$i][0] = $1;
			$Pairings[$i][1] = $2;
			$Pairings[$i][2] = $3;
			$Pairings[$i][3] = $4;
			$Pairings[$i][4] = $5;
			$Pairings[$i][5] = $6;
			$Pairings[$i][6] = $7;
			$Pairings[$i][7] = $8;
			$Pairings[$i][8] = $9;
			++ $i;
		}
	}
	# format the base-pairs
	for($i = 0; $i < scalar(@Pairings); ++ $i){
		if($Pairings[$i][0] > $Pairings[$i][1]){
			my $temp;
			$temp = $Pairings[$i][0];
			$Pairings[$i][0] = $Pairings[$i][1];
			$Pairings[$i][1] = $temp;
			$temp = $Pairings[$i][2];
			$Pairings[$i][2] = $Pairings[$i][3];
			$Pairings[$i][3] = $temp;
			$temp = $Pairings[$i][4];
			$Pairings[$i][4] = $Pairings[$i][5];
			$Pairings[$i][5] = $temp;
			$temp = $Pairings[$i][7];
			$Pairings[$i][7] = $Pairings[$i][8];
			$Pairings[$i][8] = $temp;
		}
	}
	# sort the formatted base-pairs
	for($i = 0; $i < scalar(@Pairings); ++ $i){
		for($j = $i + 1; $j < scalar(@Pairings); ++ $j){
			if($Pairings[$i][0] > $Pairings[$j][0] || ($Pairings[$i][0] == $Pairings[$j][0] && $Pairings[$i][1] > $Pairings[$j][1])){
				my @temp;				
				for($k = 0; $k < 9; ++ $k){
					$temp[$k] = $Pairings[$i][$k];
				}
				for($k = 0; $k < 9; ++ $k){
					$Pairings[$i][$k] = $Pairings[$j][$k];
				}
				for($k = 0; $k < 9; ++ $k){
					$Pairings[$j][$k] = $temp[$k];
				}
			}
		}
	} 
	#my $step = int ($numPairMotif * $windowSize);
	#if($step < 1){
		my $step = 1;
	#}
	my $lenCut = int($numPairMotif * 2); 
	my @pickedPairings;
	my $pickedTag = 0;
	my %pickedPSHash;
	# get the feasible segments from the structure
	for($i = 0; $i < scalar(@Pairings) - $lenCut; $i += $step){
		my @leftPairings;
		my @rightPairings;
		my $centerLeft;
		my $centerRight;
		my $takenTag = 0;
		for($j = $i; $j < $i + $lenCut; ++ $j){
			push @leftPairings, $Pairings[$j][0];
			push @rightPairings, $Pairings[$j][1];
		}
		$centerLeft = center(@leftPairings, $lenMotif);
		$centerRight = center(@rightPairings, $lenMotif);
		# remove those pairs that are too far away from the center
		$k = 0;
		for($j = $i; $j < $i + $lenCut; ++ $j){
			if(abs($Pairings[$j][0] - $centerLeft) < $lenMotif && abs($Pairings[$j][1] - $centerRight) < $lenMotif){
				for($l = 0; $l < 9; ++ $l){
					$pickedPairings[$pickedTag][$k][$l] = $Pairings[$j][$l];
				}
				++ $k;
				$takenTag = 1;
			}
		}
		if($takenTag == 1){
			$pickedPSHash{$pickedTag} = $i;
			++ $pickedTag;
		}
	}


	# remove redundant structures
	my %redundants;
	for($i = 0; $i < @pickedPairings - 1; ++ $i){
		my $minNumPair = min(scalar(@{$pickedPairings[$i]}), scalar(@{$pickedPairings[$i + 1]}));
		for($k = 0; $k <= abs(scalar(@{$pickedPairings[$i]}) - scalar(@{$pickedPairings[$i + 1]})); ++ $k){
			if(scalar(@{$pickedPairings[$i]}) >= scalar(@{$pickedPairings[$i + 1]})){
				for($j = 0; $j < $minNumPair; ++ $j){
					if($pickedPairings[$i][$j + $k][0] != $pickedPairings[$i+1][$j][0] || 
					$pickedPairings[$i][$j + $k][1] != $pickedPairings[$i+1][$j][1]){
					last;
					}
				}
			}else{
				for($j = 0; $j < $minNumPair; ++ $j){
					if($pickedPairings[$i][$j][0] != $pickedPairings[$i+1][$j + $k][0] || 
						$pickedPairings[$i][$j][1] != $pickedPairings[$i+1][$j + $k][1]){
						last;
					}
				}
			}
			if($j == $minNumPair){
				my $toRemove = ((scalar(@{$pickedPairings[$i]}) > scalar(@{$pickedPairings[$i + 1]})) ? ($i + 1) : ($i));
				$redundants{$toRemove} = 1;
			}
		}
		# if one segment is complete subset of another
	}

	my %nrPSHash;
	my @nrPairings;
	my $nrPairingsTag = 0;
	for($i = 0; $i < @pickedPairings; ++ $i){
		if(!(exists $redundants{$i})){
			for($j = 0; $j < @{$pickedPairings[$i]}; ++ $j){
				for($k = 0; $k < 9; ++ $k){
					$nrPairings[$nrPairingsTag][$j][$k] = $pickedPairings[$i][$j][$k];
				}
			}
			$nrPSHash{$nrPairingsTag} = $pickedPSHash{$i};
			++ $nrPairingsTag;
		}
	}
	if(!(-e "$directory/WorkSpace/$processID/CuttedStructs")){
		mkdir "$directory/WorkSpace/$processID/CuttedStructs" or die "Cannot make directory: $!\n";
	}
	
	chdir "$directory/WorkSpace/$processID/CuttedStructs" or die "Cannot change into directory: $!\n";
	my $minLeft;
	my $maxLeft;
	my $minRight;
	my $maxRight;
	my $naminLeft;
	my $namaxLeft;
	my $naminRight;
	my $namaxRight;
	$targetName =~ s/^>//g;
	for($i = 0; $i < scalar(@nrPairings); ++ $i){
		if(scalar(@{$nrPairings[$i]}) < 0.5 * $numPairMotif){
			next;
		}
		my @leftIndex;
		my @rightIndex;
		for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
			push @leftIndex, $nrPairings[$i][$j][0];
			push @rightIndex, $nrPairings[$i][$j][1];
		}
		$minLeft = min(@leftIndex);
		$maxLeft = max(@leftIndex);
		$minRight = min(@rightIndex);
		$maxRight = max(@rightIndex);
		for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
			if($nrPairings[$i][$j][0] == $minLeft){
				$naminLeft = $nrPairings[$i][$j][7];
			}
			if($nrPairings[$i][$j][0] == $maxLeft){
				$namaxLeft = $nrPairings[$i][$j][7];
			}
			if($nrPairings[$i][$j][1] == $minRight){
				$naminRight = $nrPairings[$i][$j][8];
			}
			if($nrPairings[$i][$j][1] == $maxRight){
				$namaxRight = $nrPairings[$i][$j][8];
			}
		}
		my $outFilename;
		my $outSeq;

		# print the suffled version
		$outFilename = 'shuffled_' . $targetName . '_' . $minLeft . '_' . $maxLeft . '_' . $minRight . '_' . $maxRight . '.mca';
		$outSeq = substr($sequence, $minLeft, $maxLeft - $minLeft + 1) . substr($sequence, $minRight, $maxRight - $minRight + 1);
		my @shuffledArray;
		my $m;
		my $n;
		for($m = 0; $m < length($outSeq); ++ $m){
			$shuffledArray[$m] = $m;
		}
		my @currentStack;
		for($m = 0; $m < scalar(@{$nrPairings[$i]}); ++ $m){
			for($n = 0; $n < 9; ++ $n){
				$currentStack[$m][$n] = $nrPairings[$i][$m][$n];
			}
		}
		for($m = 0; $m < scalar(@{$nrPairings[$i]}); ++ $m){
			$currentStack[$m][0] -= $minLeft;
			$currentStack[$m][1] -= $minRight - $maxLeft + $minLeft - 1;
		}
		for($m = 0; $m < scalar(@currentStack)/3 + 1; ++ $m){
			my $ranNum1 = int(rand(scalar(@currentStack)));
			my $ranNum2 = int(rand(scalar(@currentStack)));
			my $a = scalar(@currentStack);
			my $countWhile = 0;
			while($ranNum1 == $ranNum2 && $a != 1 && $countWhile < 20){
				++ $countWhile;
				$ranNum2 = int(rand(scalar(@currentStack) - 1));
				if($ranNum2 < $ranNum1){
					;
				}else{
					++ $ranNum2;
				}
				if($currentStack[$ranNum1][0] == $currentStack[$ranNum2][1] || $currentStack[$ranNum1][1] == $currentStack[$ranNum2][0]){
					$ranNum2 = $ranNum1;
				}
			}
			my @temp;
			for($n = 2; $n < 9; ++ $n){
				$temp[$n] = $currentStack[$ranNum1][$n];
			}
			for($n = 2; $n < 9; ++ $n){
				$currentStack[$ranNum1][$n] = $currentStack[$ranNum2][$n];
			}
			for($n = 2; $n < 9; ++ $n){
				$currentStack[$ranNum2][$n] = $temp[$n];
			}
			my $tempS1; my $tempS2;
			$tempS1 = $shuffledArray[$currentStack[$ranNum1][0]];
			$tempS2 = $shuffledArray[$currentStack[$ranNum1][1]];
			$shuffledArray[$currentStack[$ranNum1][0]] = $shuffledArray[$currentStack[$ranNum2][0]];
			$shuffledArray[$currentStack[$ranNum1][1]] = $shuffledArray[$currentStack[$ranNum2][1]];
			$shuffledArray[$currentStack[$ranNum2][0]] = $tempS1;
			$shuffledArray[$currentStack[$ranNum2][1]] = $tempS2;
			for($n = 0; $n < scalar(@currentStack); ++ $n){
				if($n != $ranNum1 && $n != $ranNum2){
					if($currentStack[$n][0] == $currentStack[$ranNum1][0]){
						$currentStack[$n][0] = $currentStack[$ranNum2][0];
					}elsif($currentStack[$n][0] == $currentStack[$ranNum2][0]){
						$currentStack[$n][0] = $currentStack[$ranNum1][0];
					}elsif($currentStack[$n][0] == $currentStack[$ranNum1][1]){
						$currentStack[$n][0] = $currentStack[$ranNum2][1];
					}elsif($currentStack[$n][0] == $currentStack[$ranNum2][1]){
						$currentStack[$n][0] = $currentStack[$ranNum1][1];
					}
					if($currentStack[$n][1] == $currentStack[$ranNum1][1]){
						$currentStack[$n][1] = $currentStack[$ranNum2][1];
					}elsif($currentStack[$n][1] == $currentStack[$ranNum2][1]){
						$currentStack[$n][1] = $currentStack[$ranNum1][1];
					}elsif($currentStack[$n][1] == $currentStack[$ranNum1][0]){
						$currentStack[$n][1] = $currentStack[$ranNum2][0];
					}elsif($currentStack[$n][1] == $currentStack[$ranNum2][0]){
						$currentStack[$n][1] = $currentStack[$ranNum1][0];
					}
				}
			}
		}
		for($m = 0; $m < scalar(@currentStack); ++ $m){
			my $clleft = -1; my $crleft = length($outSeq); my $clright = -1; my $crright = length($outSeq);
			for($n = 0; $n < scalar(@currentStack); ++ $n){
				if($currentStack[$n][0] < $currentStack[$m][0] && $currentStack[$n][0] > $clleft){
					$clleft = $currentStack[$n][0];
				}
				if($currentStack[$n][0] > $currentStack[$m][0] && $currentStack[$n][0] < $crleft){
					$crleft = $currentStack[$n][0];
				}
				if($currentStack[$n][0] < $currentStack[$m][1] && $currentStack[$n][0] > $clright){
					$clright = $currentStack[$n][0];
				}
				if($currentStack[$n][0] > $currentStack[$m][1] && $currentStack[$n][0] < $crright){
					$crright = $currentStack[$n][0];
				}
				
				if($currentStack[$n][1] < $currentStack[$m][0] && $currentStack[$n][1] > $clleft){
					$clleft = $currentStack[$n][1];
				}
				if($currentStack[$n][1] > $currentStack[$m][0] && $currentStack[$n][1] < $crleft){
					$crleft = $currentStack[$n][1];
				}
				if($currentStack[$n][1] < $currentStack[$m][1] && $currentStack[$n][1] > $clright){
					$clright = $currentStack[$n][1];
				}
				if($currentStack[$n][1] > $currentStack[$m][1] && $currentStack[$n][1] < $crright){
					$crright = $currentStack[$n][1];
				}
			}
			my $newleft = $clleft + int(rand($crleft - $clleft - 1)) + 1;
			my $newright = $clright + int(rand($crright - $clright - 1)) + 1;
			my $left = $currentStack[$m][0];
			my $right = $currentStack[$m][1];
			if($left == $newleft && $right == $newright){
				next;
			}
			for($n = 0; $n < scalar(@currentStack); ++ $n){
				if($currentStack[$n][0] == $left){
					$currentStack[$n][0] = $newleft;
				}elsif($currentStack[$n][0] == $newleft){
					$currentStack[$n][0] = $left;
				}elsif($currentStack[$n][0] == $right){
					$currentStack[$n][0] = $newright;
				}elsif($currentStack[$n][0] == $newright){
					$currentStack[$n][0] = $right;
				}

				if($currentStack[$n][1] == $left){
					$currentStack[$n][1] = $newleft;
				}elsif($currentStack[$n][1] == $newleft){
					$currentStack[$n][1] = $left;
				}elsif($currentStack[$n][1] == $right){
					$currentStack[$n][1] = $newright;
				}elsif($currentStack[$n][1] == $newright){
					$currentStack[$n][1] = $right;
				}
			}
			my $temp;
			$temp = $shuffledArray[$left];
			$shuffledArray[$left] = $shuffledArray[$newleft];
			$shuffledArray[$newleft] = $temp;
			$temp = $shuffledArray[$right];
			$shuffledArray[$right] = $shuffledArray[$newright];
			$shuffledArray[$newright] = $temp;
			
		}
		my @presentArray;
		for($m = 0; $m < length($outSeq); ++ $m){
			$presentArray[$m] = 0;
		}
		for($m = 0; $m < scalar(@currentStack); ++ $m){
			$presentArray[$currentStack[$m][0]] = 1;
			$presentArray[$currentStack[$m][1]] = 1;
		}
		my $upTag = 0;
		my %upHash;
		for($m = 0; $m < length($outSeq); ++ $m){
			if($presentArray[$m] == 0){
				$upHash{$upTag} = $m;
				++ $upTag;
			}
		}
		if($upTag > 1){
			for($m = 0; $m < $upTag/3 + 1; ++ $m){
				my $ranNum1 = int(rand($upTag));
				my $ranNum2 = int(rand($upTag));
				my $countWhile = 0;
				while($ranNum1 == $ranNum2 && $countWhile < 20){
					++ $countWhile;
					$ranNum2 = int(rand($upTag));
				}
				my $temp;
				$temp = $shuffledArray[$upHash{$ranNum1}];
				$shuffledArray[$upHash{$ranNum1}] = $shuffledArray[$upHash{$ranNum2}];
				$shuffledArray[$upHash{$ranNum2}] = $temp;
			}
		}
		my $newSeq = undef;
		my @sequence;
		@sequence = split //, $outSeq;
		for($m = 0; $m < length($outSeq); ++ $m){
			$newSeq .= "$sequence[$shuffledArray[$m]]";
		}
		open OUT, ">$outFilename" or die "Cannot create file: $!\n";
		print OUT ">shuffled\_$targetName\_$minLeft\_$maxLeft\_$minRight\_$maxRight\_\_$naminLeft\_$namaxLeft\_$naminRight\_$namaxRight\n";
		my $temp = substr($newSeq, 0, $maxLeft - $minLeft + 1);
		print OUT "$temp";
		print OUT "...";
		$temp = substr($newSeq, $maxLeft - $minLeft + 1);
		print OUT "$temp\n";
		print OUT "//\n";
		for($j = 0; $j < scalar(@currentStack); ++ $j){
			my $left = $currentStack[$j][0];
			my $right = $currentStack[$j][1];
			if(substr($newSeq, $left, 1) ne $currentStack[$j][2] || 
				substr($newSeq, $right, 1) ne $currentStack[$j][3]){
				#print "$left	$right\n";
				#print "Indexing Error! two sides\n";
				next;
			}
			if($left < $right){
				print OUT "#	$left-$right : $currentStack[$j][2]-$currentStack[$j][3] ";
				print OUT "$currentStack[$j][4]\/$currentStack[$j][5] $currentStack[$j][6]\n";
			}else{
				print OUT "#	$right-$left : $currentStack[$j][3]-$currentStack[$j][2] ";
				print OUT "$currentStack[$j][5]\/$currentStack[$j][4] $currentStack[$j][6]\n";
			}	
		}
		for($k = $nrPSHash{$i}; $k < $nrPSHash{$i} + $lenCut; ++ $k){
			my $matchTag = 0;
			for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
				if($Pairings[$k][7] eq $currentStack[$j][7] && $Pairings[$k][8] eq $currentStack[$j][8]){
					$matchTag = 1;
					last;
				}
			}
			if($matchTag == 0){
				if($Pairings[$k][0] >= $minLeft && $Pairings[$k][0] <= $maxLeft &&
					$Pairings[$k][1] >= $minLeft && $Pairings[$k][1] <= $maxLeft){
					my $idx1 = $Pairings[$k][0] - $minLeft;
					my $idx2 = $Pairings[$k][1] - $minLeft;
					my $n;
					my $left;
					my $right;
					for($n = 0; $n < scalar(@shuffledArray); ++ $n){
						if($shuffledArray[$n] == $idx1){
							$left = $n;
						}
						if($shuffledArray[$n] == $idx2){
							$right = $n;
						}
					}
					if(substr($newSeq, $left, 1) ne $Pairings[$k][2] ||
						substr($newSeq, $right, 1) ne $Pairings[$k][3]){
						#print "Indexing Error! left\n";
						next;
					}
					if($left < $right){
						print OUT "#	$left-$right : $Pairings[$k][2]-$Pairings[$k][3] ";
						print OUT "$Pairings[$k][4]\/$Pairings[$k][5] $Pairings[$k][6]\n";
					}else{
						print OUT "#	$right-$left : $Pairings[$k][3]-$Pairings[$k][2] ";
						print OUT "$Pairings[$k][5]\/$Pairings[$k][4] $Pairings[$k][6]\n";
					}
				}elsif($Pairings[$k][0] >= $minRight && $Pairings[$k][0] <= $maxRight &&
					$Pairings[$k][1] >= $minRight && $Pairings[$k][1] <= $maxRight){
					my $idx1 = $Pairings[$k][0] - ($minRight - $maxLeft + $minLeft - 1);
					my $idx2 = $Pairings[$k][1] - ($minRight - $maxLeft + $minLeft - 1);
					my $n;
					my $left;
					my $right;
					for($n = 0; $n < scalar(@shuffledArray); ++ $n){
						if($shuffledArray[$n] == $idx1){
							$left = $n;
						}
						if($shuffledArray[$n] == $idx2){
							$right = $n;
						}
					}
					if(substr($newSeq, $left, 1) ne $Pairings[$k][2] ||
						substr($newSeq, $right, 1) ne $Pairings[$k][3]){
						#print "Indexing Error! right\n";
						next;
					}
					if($left < $right){
						print OUT "#	$left-$right : $Pairings[$k][2]-$Pairings[$k][3] ";
						print OUT "$Pairings[$k][4]\/$Pairings[$k][5] $Pairings[$k][6]\n";
					}else{ 
						print OUT "#	$right-$left : $Pairings[$k][3]-$Pairings[$k][2] ";
						print OUT "$Pairings[$k][5]\/$Pairings[$k][4] $Pairings[$k][6]\n";
					}
				}
			}
		}
		close OUT;

		# print the cutted, not reversed version
		$outFilename = $targetName . '_' . $minLeft . '_' . $maxLeft . '_' . $minRight . '_' . $maxRight . '.mca';
		$outSeq = substr($sequence, $minLeft, $maxLeft - $minLeft + 1) . '...' . substr($sequence, $minRight, $maxRight - $minRight + 1);
		open OUT, ">$outFilename" or die "Cannot create file: $!\n";
		print OUT ">$targetName\_$minLeft\_$maxLeft\_$minRight\_$maxRight\_\_$naminLeft\_$namaxLeft\_$naminRight\_$namaxRight\n";
		print OUT "$outSeq\n";
		print OUT "//\n";
		$outSeq =~ s/\.\.\.//g;
		for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
			$nrPairings[$i][$j][0] -= $minLeft;
			$nrPairings[$i][$j][1] -= $minRight - $maxLeft + $minLeft - 1;
			if(substr($outSeq, $nrPairings[$i][$j][0], 1) ne $nrPairings[$i][$j][2] || 
				substr($outSeq, $nrPairings[$i][$j][1], 1) ne $nrPairings[$i][$j][3]){
				#print "Indexing Error! two sides\n";
				next;
			}
			print OUT "#	$nrPairings[$i][$j][0]-$nrPairings[$i][$j][1] : $nrPairings[$i][$j][2]-$nrPairings[$i][$j][3] ";
			print OUT "$nrPairings[$i][$j][4]\/$nrPairings[$i][$j][5] $nrPairings[$i][$j][6]\n";	
		}
		for($k = $nrPSHash{$i}; $k < $nrPSHash{$i} + $lenCut; ++ $k){
			my $matchTag = 0;
			for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
				if($Pairings[$k][7] eq $nrPairings[$i][$j][7] && $Pairings[$k][8] eq $nrPairings[$i][$j][8]){
					$matchTag = 1;
					last;
				}
			}
			if($matchTag == 0){
				if($Pairings[$k][0] >= $minLeft && $Pairings[$k][0] <= $maxLeft &&
					$Pairings[$k][1] >= $minLeft && $Pairings[$k][1] <= $maxLeft){
					my $idx1 = $Pairings[$k][0] - $minLeft;
					my $idx2 = $Pairings[$k][1] - $minLeft;
					if(substr($outSeq, $idx1, 1) ne $Pairings[$k][2] ||
						substr($outSeq, $idx2, 1) ne $Pairings[$k][3]){
						#print "Indexing Error! left\n";
						#print "$minLeft	$maxLeft	$minRight	$maxRight\n";
						#print "$idx1	$idx2	$Pairings[$k][2]	$Pairings[$k][3]\n";
						next;
					}
					print OUT "#	$idx1-$idx2 : $Pairings[$k][2]-$Pairings[$k][3] ";
					print OUT "$Pairings[$k][4]\/$Pairings[$k][5] $Pairings[$k][6]\n";
				}elsif($Pairings[$k][0] >= $minRight && $Pairings[$k][0] <= $maxRight &&
					$Pairings[$k][1] >= $minRight && $Pairings[$k][1] <= $maxRight){
					my $idx1 = $Pairings[$k][0] - ($minRight - $maxLeft + $minLeft - 1);
					my $idx2 = $Pairings[$k][1] - ($minRight - $maxLeft + $minLeft - 1);
					if(substr($outSeq, $idx1, 1) ne $Pairings[$k][2] ||
						substr($outSeq, $idx2, 1) ne $Pairings[$k][3]){
						#print "Indexing Error! right\n";
						#print "$minLeft	$maxLeft	$minRight	$maxRight\n";
						#print "$idx1	$idx2	$Pairings[$k][2]	$Pairings[$k][3]\n";
						next;
					}
					print OUT "#	$idx1-$idx2 : $Pairings[$k][2]-$Pairings[$k][3] ";
					print OUT "$Pairings[$k][4]\/$Pairings[$k][5] $Pairings[$k][6]\n";
				}
			}
		}
		close OUT;

		# print the reverse version
		$outFilename = 'rotate_' . $targetName . '_' . $minRight . '_' . $maxRight . '_' . $minLeft . '_' . $maxLeft . '.mca';
		$outSeq = substr($sequence, $minRight, $maxRight - $minRight + 1) . '...' . substr($sequence, $minLeft, $maxLeft - $minLeft + 1);
		open OUT, ">$outFilename" or die "Cannot create file: $!\n";
		print OUT ">rotate\_$targetName\_$minRight\_$maxRight\_$minLeft\_$maxLeft\_\_$naminRight\_$namaxRight\_$naminLeft\_$namaxLeft\n";
		print OUT "$outSeq\n";
		print OUT "//\n";
		$outSeq =~ s/\.\.\.//g;
		for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
			$nrPairings[$i][$j][0] += $maxRight - $minRight + 1;
			$nrPairings[$i][$j][1] -= $maxLeft - $minLeft + 1;
			if(substr($outSeq, $nrPairings[$i][$j][0], 1) ne $nrPairings[$i][$j][2] || 
				substr($outSeq, $nrPairings[$i][$j][1], 1) ne $nrPairings[$i][$j][3]){
				#print "Indexing Error! rotate two sides\n";
				next;
			}
			print OUT "#	$nrPairings[$i][$j][1]-$nrPairings[$i][$j][0] : $nrPairings[$i][$j][3]-$nrPairings[$i][$j][2] ";
			print OUT "$nrPairings[$i][$j][5]\/$nrPairings[$i][$j][4] $nrPairings[$i][$j][6]\n";	
		}
		for($k = $nrPSHash{$i}; $k < $nrPSHash{$i} + $lenCut; ++ $k){
			my $matchTag = 0;
			for($j = 0; $j < scalar(@{$nrPairings[$i]}); ++ $j){
				if($Pairings[$k][7] eq $nrPairings[$i][$j][7] && $Pairings[$k][8] eq $nrPairings[$i][$j][8]){
					$matchTag = 1;
					last;
				}
			}
			if($matchTag == 0){
				if($Pairings[$k][0] >= $minLeft && $Pairings[$k][0] <= $maxLeft &&
					$Pairings[$k][1] >= $minLeft && $Pairings[$k][1] <= $maxLeft){
					my $idx1 = $Pairings[$k][0] - ($minLeft - $maxRight + $minRight - 1);
					my $idx2 = $Pairings[$k][1] - ($minLeft - $maxRight + $minRight - 1);
					if(substr($outSeq, $idx1, 1) ne $Pairings[$k][2] ||
						substr($outSeq, $idx2, 1) ne $Pairings[$k][3]){
						#print "Indexing Error! rotate left\n";
						#print "$minLeft	$maxLeft	$minRight	$maxRight\n";
						#print "$Pairings[$k][0]	$Pairings[$k][1]	$Pairings[$k][2]	$Pairings[$k][3]\n";
						next;
					}

					print OUT "#	$idx1-$idx2 : $Pairings[$k][2]-$Pairings[$k][3] ";
					print OUT "$Pairings[$k][4]\/$Pairings[$k][5] $Pairings[$k][6]\n";
				}elsif($Pairings[$k][0] >= $minRight && $Pairings[$k][0] <= $maxRight &&
					$Pairings[$k][1] >= $minRight && $Pairings[$k][1] <= $maxRight){
					my $idx1 = $Pairings[$k][0] - $minRight;
					my $idx2 = $Pairings[$k][1] - $minRight;
					if(substr($outSeq, $idx1, 1) ne $Pairings[$k][2] ||
						substr($outSeq, $idx2, 1) ne $Pairings[$k][3]){
						#print "Indexing Error! rotate right\n";
						#print "$minLeft	$maxLeft	$minRight	$maxRight\n";
						#print "$idx1	$idx2	$Pairings[$k][2]	$Pairings[$k][3]\n";
						next;
					}
					print OUT "#	$idx1-$idx2 : $Pairings[$k][2]-$Pairings[$k][3] ";
					print OUT "$Pairings[$k][4]\/$Pairings[$k][5] $Pairings[$k][6]\n";
				}
			}
		}
		close OUT;
	}	
	close Target;
	chdir "$directory" or die "Cannot change directory: $!\n";
}

sub RunAlignment($){
	# added
	undef %rotatedAlignment;
	undef %originalAlignment;
	my $motifFile = shift;
#	chdir "./WorkSpace/$processID/" or die "Cannot change directory: $!\n";
	if(!(-e "$directory/WorkSpace/$processID/HighScoreHits")){
		mkdir "$directory/WorkSpace/$processID/HighScoreHits" or die "Cannot make directory";
	}
	my @filelist;
	foreach(<$directory/WorkSpace/$processID/CuttedStructs/*>){
		push @filelist, $_;
	}
	foreach(@filelist){
		++ $alignmentCount;
		#print "$_\n";
		if(/rotate/){
			#print "do alignment\n";
			system "$directory/RNAMotifAlign -i $motifFile -t $_ >$directory/WorkSpace/$processID/temp.align";
			open my $tempAlign, "<$directory/WorkSpace/$processID/temp.align" or die "Cannot open file: $!\n";
			my $singleAlign = undef;
			if(!eof($tempAlign)){
					$singleAlign = <$tempAlign>;
			}
			if(defined($singleAlign) && $singleAlign =~ /^program successed:\s+\S+\s+(\S+)\s+score:	(\S+)/){
				if($2 > $mean){
					# added
					my $pValue = $squareDev / (($2 - $mean) * ($2 - $mean));
					#if($pValue <= $cutoff){
						$rotatedAlignment{$1} = $pValue;
						system "mv $_ $directory/WorkSpace/$processID/HighScoreHits";
					#}
				}
			}
			close $tempAlign;
		}	elsif	(/shuffled/)	{
			#print "do alignment\n";
			system "$directory/RNAMotifAlign -i $motifFile -t $_ >>$directory/WorkSpace/$processID/shuffled.align";
		}	else	{
			#print "do alignment\n";
			system "$directory/RNAMotifAlign -i $motifFile -t $_ >$directory/WorkSpace/$processID/temp.align";
			open my $tempAlign, "<$directory/WorkSpace/$processID/temp.align" or die "Cannot open file: $!\n";
			my $singleAlign = undef;
			if(!eof($tempAlign)){
				$singleAlign = <$tempAlign>;
			}
			if(defined($singleAlign) && $singleAlign =~ /^program successed:\s+\S+\s+(\S+)\s+score:	(\S+)/){
				if($2 > $mean){
					# added
					my $pValue = $squareDev / (($2 - $mean) * ($2 - $mean));
					#if($pValue <= $cutoff){
						$originalAlignment{$1} = $pValue;
						system "mv $_ $directory/WorkSpace/$processID/HighScoreHits";
					#}
				}
			}
			close $tempAlign;
		}
	}

	#foreach(keys %originalAlignment)	{
	#	print "!	$_\n";
	#}
	#system "rm -r $directory/WorkSpace/$processID/CuttedStructs";
#	chdir "../../" or die "Cannot change directory: $!\n";
}

sub SortShuffle(){
	open SHUFFLE, "<$directory/WorkSpace/$processID/shuffled.align" or die "Cannot open file: $!\n";
	my @tempScore;
	while(<SHUFFLE>){
		if(/^program successed:\s+\S+\s+(\S+)\s+score:	(\S+)/){
			push @tempScore, $2;
		}
	}
	sub by_number {$b <=> $a}
	@shuffleScore = sort by_number @tempScore;
}

sub GetOptScore(){
	# added
	#print "Called\n";
	undef %optimalAlignment;
	foreach(keys %originalAlignment){
		#print "$_	$originalAlignment{$_}\n";
		my $oriKey = $_;
		my @components = split /\_/, $oriKey;
		my $rotKey = 'rotate_' . $components[0] . '_' . $components[1] . 
				'_' . $components[4] . '_' . $components[5] . '_' . $components[2] . '_' . $components[3]. 
				'__' . $components[9] . '_' . $components[10] . '_' . $components[7] . '_' . $components[8];
		my $oriPValue = $originalAlignment{$oriKey};
		if(exists $rotatedAlignment{$rotKey}){
			my $rotPValue = $rotatedAlignment{$rotKey};
			if($oriPValue <= $rotPValue){
				$optimalAlignment{$oriKey} = $oriPValue;
				delete $rotatedAlignment{$rotKey};
			}
		}else{
			$optimalAlignment{$oriKey} = $oriPValue;
		}
	}
	foreach(keys %rotatedAlignment){
		$optimalAlignment{$_} = $rotatedAlignment{$_};
		#print "$optimalAlignment{$_}\n";
	}
}

sub RemoveSegmentRedundancy(){
	my @segmentLocation = keys %optimalAlignment;
	my $pdbIDF; my $chainF; my $pdbIDS; my $chainS; my $startF1; my $endF1; my $startF2; my $endF2; my $startS1; my $endS1; my $startS2; my $endS2;
	foreach(@segmentLocation){
		if(exists $optimalAlignment{$_}){
			my $optKey = $_;
			my @toDelete;
			my @component = split /\_/, $_;
			if(/^rotate/){
				$pdbIDF = $component[1];
				$chainF = $component[2];
				$startF1 = $component[5];
				$endF1 = $component[6];
				$startF2 = $component[3];
				$endF2 = $component[4];
			}else{
				$pdbIDF = $component[0];
				$chainF = $component[1];
				$startF1 = $component[2];
				$endF1 = $component[3];
				$startF2 = $component[4];
				$endF2 = $component[5];
			}
			foreach(keys %optimalAlignment){
				if($optKey eq $_){
					next;
				}
				my $checkKey = $_;
				my @component2 = split /\_/, $checkKey;
				if(/^rotate/){
					$pdbIDS = $component2[1];
					$chainS = $component2[2];
					$startS1 = $component2[5];
					$endS1 = $component2[6];
					$startS2 = $component2[3];
					$endS2 = $component2[4];
				}else{
					$pdbIDS = $component2[0];
					$chainS = $component2[1];
					$startS1 = $component2[2];
					$endS1 = $component2[3];
					$startS2 = $component2[4];
					$endS2 = $component2[5];
				}
				if(	($pdbIDF eq $pdbIDS && $chainF eq $chainS) && 
					($startF1 <= $endS1 && $startF1 >= $startS1 && $endS1 - $startF1 >= 0.3 * ($endS1 - $startS1) ||
					 $endF1 <= $endS1 && $endF1 >= $startS1 && $endF1 - $startS1 >= 0.3 * ($endS1 - $startS1)) || 
					($startF2 <= $endS2 && $startF2 >= $startS2 && $endS2 - $startF2 >= 0.3 * ($endS2 - $startS2) ||
					 $endF2 <= $endS2 && $endF2 >= $startS2 && $endF2 - $startS2 >= 0.3 * ($endS2 - $startS2))){

					if($optimalAlignment{$optKey} <= $optimalAlignment{$checkKey}){
						push @toDelete, $checkKey;
						next;
					}else{
						push @toDelete, $optKey;
						$optKey = $checkKey;
						$pdbIDF = $pdbIDS; $chainF = $chainS;
						$startF1 = $startS1; $endF1 = $endS1; $startF2 = $startS2; $endF2 = $endS2;
					}
				}
			}
			foreach(@toDelete){
				#print "$_\n";
				delete $optimalAlignment{$_};
			}
		}
	}
	foreach(keys %optimalAlignment){
		#if($optimalAlignment{$_} <= $cutoff){
			$sortedAlignment{$_} = $optimalAlignment{$_};
		#}
		#print "$_\n";
	}
}
	
sub ComputeMeanDev($){
	my $motifFile = shift;
	my $file = "$directory/RNA_Pairs.txt";
	my $i; my $j; my $k; my $l;
	my %ntHash = (
		"0" => "A",
		"1" => "U",
		"2" => "C",
		"3" => "G"	
	);
	
	open my $pairs, "<$file" or die "Not base-pair information for background mean and deviation computation: missing file 'RNA_Pairs.txt'\n";
	my $pairIndex = 1;
	my %pairHash;
	while(<$pairs>){
		chomp;
		if($_ =~ /^#/){
			$pairHash{$pairIndex} = $_;
			++ $pairIndex;
		}
	}
	chdir "$directory/WorkSpace/$processID";
	if(-e "$directory/WorkSpace/$processID/Simulation"){
		system "$directory/WorkSpace/$processID/Simulation";
	}
	system "mkdir $directory/WorkSpace/$processID/Simulation";
	chdir "$directory/WorkSpace/$processID/Simulation" or die "Cannot change directory: $!\n";
	for($j = 0; $j < 5000; ++ $j){
		my %pairHashNest;
		my %pairHashCross;
		my $simSeqlength = 2 * $lenMotif - int(rand($lenMotif)) - 1;
		my $simNumPairs = $numPairMotif + int(rand(int($simSeqlength / 2) - $numPairMotif + 1));
		my $simNumCross = int(rand(2 * $numCrossing));
		$simSeqlength = $simSeqlength > 2 * $simNumPairs ? $simSeqlength : 2 * $simNumPairs;
		#print "$simSeqlength	$simNumPairs	$simNumCross\n";
		my @emptyIndicator;
		for($i = 0; $i < $simSeqlength; ++ $i){
			$emptyIndicator[$i] = 0;
		}
		for($i = 0; $i < $simSeqlength - (2 * $simNumPairs); ++ $i){
			my $nopair = int(rand($simSeqlength));
			while($emptyIndicator[$nopair] != 0){
				$nopair = int(rand($simSeqlength));
			}
			$emptyIndicator[$nopair] = 1;
		}
	
		my @sequence;
		my @symbol;
		my @edge;
	
		for($i = 0; $i < scalar(@emptyIndicator); ++ $i){
			if($emptyIndicator[$i] == 1){
				$sequence[$i] = $ntHash{int(rand(4))};
				$symbol[$i] = '.';
				$edge[$i] = '.';
			}
		}
		my $leftPointer;
		my $rightPointer;
	
		for($i = 0; $i < scalar(@emptyIndicator); ++ $i){
			if($emptyIndicator[$i] == 0){
				$leftPointer = $i;
				last;
			}
		}
	
		for($i = scalar(@emptyIndicator) - 1; $i >= 0; -- $i){
			if($emptyIndicator[$i] == 0){
				$rightPointer = $i;
				last;
			}
		}
		for($i = 0; $i < $simNumPairs; ++ $i){
			my $pickedPair = int(rand($pairIndex)) + 1;
			if(exists $pairHash{$pickedPair} && $pairHash{$pickedPair} =~ /^#	\d+-\d+ : (\w)-(\w) (\w)\/(\w) (\S+) \S+$/){
				$sequence[$leftPointer] = $1;
				$sequence[$rightPointer] = $2;
				if($5 eq 'cis'){
					$edge[$leftPointer] = $3;
					$edge[$rightPointer] = $4;
				}else{
					$edge[$leftPointer] = lc($3);
					$edge[$rightPointer] = lc($4);
				}
				$symbol[$leftPointer] = '(';
				$symbol[$rightPointer] = ')';
				$pairHash{$leftPointer} = $rightPointer;
				do{
					++ $leftPointer;
				}while($emptyIndicator[$leftPointer] != 0);
				do{
					-- $rightPointer;
				}while($emptyIndicator[$rightPointer] != 0); 
			}else{
				-- $i;
			}
		}
		my $outfilename = "sim.$j.mca";
		open my $tempOut, ">$outfilename" or die "Cannot create file: $!\n";
		print $tempOut ">$j\n";
		for($i = 0; $i < $simSeqlength; ++ $i){
			print $tempOut "$sequence[$i]";
		}
		print $tempOut "\n";
		print $tempOut "\/\/\n";
		for($i = 0; $i < $simSeqlength; ++ $i){
			print $tempOut "$symbol[$i]";
		}
		print $tempOut "\n";
		for($i = 0; $i < $simSeqlength; ++ $i){
			print $tempOut "$edge[$i]";
		}
		print $tempOut "\n";
		my @usedID;
		for($i = 0; $i < $simNumCross; ++ $i){
			for($k = 0; $k < $simSeqlength; ++ $k){
				$usedID[$i][$k] = 0; 
			}
		}
		my $crossCount = 0;
		for($i = 0; $i < $simNumCross; ++ $i){
			++ $crossCount;
			if($crossCount > 3 * $simNumCross){
				last;
			}
			my $pickedPair = int(rand($pairIndex)) + 1;
			if(exists $pairHash{$pickedPair} && $pairHash{$pickedPair} =~ /^#	\d+-\d+ : (\w)-(\w) (\w)\/(\w) (\S+) \S+$/){
				my $crossSymbol = '.' x $simSeqlength;
				my $crossEdge = '.' x $simSeqlength;
				my $firstIndex = -1;
				my $secondIndex = -1;
				for($k = 0; $k < $simSeqlength; ++ $k){
					if($firstIndex == -1 && $sequence[$k] eq $1 && $secondIndex != $k){
						$firstIndex = $k;
					}elsif($secondIndex ==  -1 && $sequence[$k] eq $2 && $firstIndex != $k){
						$secondIndex = $k;
					}
				}
				my $validIndexTag = 1;
				my $testTag = 0;
				if($firstIndex >= 0 && $secondIndex >= 0 && $firstIndex < $secondIndex){
					$testTag = 1;
					foreach(keys %pairHashNest){
						if($_ == $firstIndex && $pairHashNest{$_} == $secondIndex){
							$validIndexTag = 0;
							last;
						}
					}
					for($l = 0; $l < $simNumCross; ++ $l){
						if($usedID[$l][$firstIndex] == 1 && $usedID[$l][$secondIndex] == 1){
							$validIndexTag = 0;
							last;
						}
					}
				}
				if($validIndexTag == 1 && $testTag == 1){
					$usedID[$i][$firstIndex] = 1;
					$usedID[$i][$secondIndex] = 1;
					substr($crossSymbol, $firstIndex, 1) = '(';
					substr($crossSymbol, $secondIndex, 1) = ')';
					if($5 eq 'cis'){
						substr($crossEdge, $firstIndex, 1) = $3;
						substr($crossEdge, $secondIndex, 1) = $4;
					}else{
						substr($crossEdge, $firstIndex, 1) = lc($3);
						substr($crossEdge, $secondIndex, 1) = lc($4);
					}
					print $tempOut "$crossSymbol\n";
					print $tempOut "$crossEdge\n";
				}else{
					-- $i;
				}
			}else{
				-- $i;
			}
		}
	
		close $tempOut;
	}
	chdir "$directory/WorkSpace/$processID" or die "Cannot change directory: $!\n";
	foreach(<$directory/WorkSpace/$processID/Simulation/*>){
		system "$directory/RNAMotifAlign -i $motifFile -j $_ >>simulation.align";
	}
	open my $SIMScore, "<$directory/WorkSpace/$processID/simulation.align" or die "Cannot open simualtion score: $!\n";
	my @scores;
	my @originalScores;
	my $tempSum;
	foreach(<$SIMScore>){
		chomp;
		push @originalScores, $_;
	}
	close $SIMScore;
	for($k = 0; $k < scalar(@originalScores); ++ $k){
		if($originalScores[$k] =~ /^program successed:.*score:	(\S+)/){
			push @scores, $1;
		}
	}
	$tempSum = 0;
	foreach(@scores){
		$tempSum += $_;
	}
	$mean = $tempSum / scalar(@scores);
	$tempSum = 0;
	foreach(@scores){
		$tempSum += ($_ - $mean) * ($_ - $mean);
	}
	$squareDev = $tempSum / scalar(@scores);
	chdir "$directory" or die "Cannot change directory: $!\n";
	#print "$mean	$squareDev\n";
}


sub RedoAlignment($){
	my $motifFile = shift;
	chdir "$directory/WorkSpace/$processID/" or die "Cannot change directory: $!\n";
	if(-e "redo.align"){
		system "rm redo.align";
	}
	@sortedKeys = sort by_hash_value2 keys %sortedAlignment;
	foreach(@sortedKeys){
		my @components = split /\_\_/, $_;
		my $fileSegment = $components[0] . '.mca';
		system "$directory/RNAMotifAlign_align -i $motifFile -t $directory/WorkSpace/$processID/HighScoreHits/$fileSegment >>redo.align";		
	}
	# probably no high score hits found in a perticular PDB file
	my $hitTag = 1;
	open my $align, "<redo.align" or $hitTag = 0;
	if($hitTag == 0){
		print "No high score hits found in the target structure(s).\n";
		chdir "$directory" or die "Cannot change directory: $!\n";
		return;
	}
	my @alignmentContents;
	while(<$align>){
		chomp;
		push @alignmentContents, $_;
	}
	my $i;
	for($i = 0; $i < scalar(@alignmentContents); ++ $i){
		if($alignmentContents[$i] =~ /^Alignment Score/){
			my @temp;
			push @temp, $alignmentContents[$i];
			++ $i;
			while($i < scalar(@alignmentContents) && !($alignmentContents[$i] =~ /^Alignment Score/)){
				push @temp, $alignmentContents[$i];
				++ $i;
			}
			-- $i;
			$temp[5] =~ /Target:	(\S+)/;
			$storeAlignment{$1} = \@temp;
		}
	}
	chdir "$directory" or die "Cannot change directory: $!\n";
	return;
}

sub PrintResult($){
	my $outfile = shift;
	my $output;
	if($outfile eq 'NOTAFILE'){
		$output = *STDOUT;
	}else{
		open $output, ">$outfile" or die "Cannot create file: $!\n";
	}
	foreach(@sortedKeys){
		my $pdbID; my $chain; my $start1; my $end1; my $start2; my $end2;
		my @component = split /\_/, $_;
		if(/^rotate/){
			$pdbID = $component[1];
			$chain = $component[2];
			$start1 = $component[10];
			$end1 = $component[11];
			$start2 = $component[8];
			$end2 = $component[9];
		}else{
			$pdbID = $component[0];
			$chain = $component[1];
			$start1 = $component[7];
			$end1 = $component[8];
			$start2 = $component[9];
			$end2 = $component[10];
		}
		my $pValue = $sortedAlignment{$_};
		@{$storeAlignment{$_}}[0] =~ /^Alignment Score: (\S+)/;
		my $score = $1;
		my $n;
		for($n = 0; $n < scalar(@shuffleScore); ++ $n){
			if($shuffleScore[$n] < $score){
				last;
			}
		}
		my $fpr = $n / scalar(@shuffleScore);
		if($fpr > $cutoff){
			next;
		}
		$fpr = sprintf("%.5f", $fpr);
		$pValue = sprintf("%.5f", $pValue);
		if($printMode == 0){
			if(/^rotate/){
				print $output "$pdbID\_$chain	location: $start2-$end2/$start1-$end1	Score: $score	P-value: $pValue	FPR: $fpr\n";	
			}else{
				print $output "$pdbID\_$chain	location: $start1-$end1/$start2-$end2	Score: $score	P-value: $pValue	FPR: $fpr\n";	
			}
		}else{
			print $output "@{$storeAlignment{$_}}[3]\n@{$storeAlignment{$_}}[4]\n@{$storeAlignment{$_}}[5]\n";
			print $output "Score: $score	P-value: $pValue	FPR: $fpr\n";
			my $l;
			for($l = 6; $l < scalar(@{$storeAlignment{$_}}); ++ $l){
				print $output "${$storeAlignment{$_}}[$l]\n";
			}
		}
	}
	if($outfile ne 'NOTAFILE'){
		print "Result saved to $outfile.\n";
	}
	return;
}

sub PrintContent(){
	print "Number of strands:	$numStrand\n";
	#foreach(sort by_hash_value keys %optimalAlignment){
	#	print "$_	$optimalAlignment{$_}\n";
	#}
	#print "$mean	$squareDev\n";
}

1;
