#include "./inc/ReadInput.h"
#include "./inc/RetrieveScore.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/*
 *	Char2int
 *
 *	The function changes a nucleotide into a numeral
 *	doing this for sequence alignment llmax
 *	The accepted and return values are defined in 'nuc.h'
*/

char Char2int(char c){
	/* the external variables are defined in 'nuc.h'*/
	extern int totalNuc;
	extern char naName[17];
	int i, k;
	/* searching the representation through the array of na_name*/ 
	for(i = 0; i < totalNuc; i ++)	{
		if(c == naName[i]){
			k = i;
			break;
		}
	}
	/* if not found, arbitrarily return a value(G)*/
	if(i == totalNuc)	return(0);
	else			return(k);
}

int STK2Bpseq(int *pairs, char *seq_b, char *struc, char *edge, int length, char leftchar, char rightchar){
	int i;
	int stackId = 0;
	int numPair = 0;
	int *stack = (int *) calloc(length, sizeof(int));
	/* poping and pushing stacks, handling parenthesis*/
	for(i=0; i<length; ++i){
		if(struc[i] == leftchar){
			stack[stackId] = i;
			++stackId;
		}else if(struc[i] == rightchar){
			--stackId;
			pairs[6 * numPair] = stack[stackId];
			pairs[6 * numPair + 3] = i;
			/* check for edge type and orientation cis: upper case, trans: lower case*/
			switch(edge[stack[stackId]]){
				case 'W':
					pairs[6 * numPair + 1] = 1;
					pairs[6 * numPair + 2] = 0;
					break;
				case 'w':
					pairs[6 * numPair + 1] = 1;
					pairs[6 * numPair + 2] = 1;
					break;
				case 'H':
					pairs[6 * numPair + 1] = 2;
					pairs[6 * numPair + 2] = 0;
					break;
				case 'h':
					pairs[6 * numPair + 1] = 2;
					pairs[6 * numPair + 2] = 1;
					break;
				case 'S':
					pairs[6 * numPair + 1] = 3;
					pairs[6 * numPair + 2] = 0;
					break;
				case 's':
					pairs[6 * numPair + 1] = 3;
					pairs[6 * numPair + 2] = 1;
					break;
			}
			switch(edge[i]){
				case 'W':
					pairs[6 * numPair + 4] = 1;
					pairs[6 * numPair + 5] = 0;
					break;
				case 'w':
					pairs[6 * numPair + 4] = 1;
					pairs[6 * numPair + 5] = 1;
					break;
				case 'H':
					pairs[6 * numPair + 4] = 2;
					pairs[6 * numPair + 5] = 0;
					break;
				case 'h':
					pairs[6 * numPair + 4] = 2;
					pairs[6 * numPair + 5] = 1;
					break;
				case 'S':
					pairs[6 * numPair + 4] = 3;
					pairs[6 * numPair + 5] = 0;
					break;
				case 's':
					pairs[6 * numPair + 4] = 3;
					pairs[6 * numPair + 5] = 1;
					break;
			}
			/* check for edge consistancy*/
			if(pairs[6 * numPair + 1] == 0 || pairs[6 * numPair + 4] == 0 || pairs[6 * numPair + 2] != pairs[6 * numPair + 5]){
				printf("Error in STK format structure!\n");
				exit(0);
			}
			//	if(IsValidPair(&pairs[6 * numPair], seq_b)){
				//printf("numPair	%d	location:	%d - %d\n", numPair, pairs[6 * numPair], pairs[6 * numPair + 3]);
				++numPair;	
			//	}
		}
	}
	if(stackId != 0){
		printf("Warning: unbalanced structure indicated by STK format: base-pairs may be ignored\n");
	}
	free((void *) stack);
	return numPair;	
}

/*
 *	ReadMotif
 *
 *	the function takes a Motif file name as input, and 3 arrays as result storage(sequence, non-crossing pairings and crossing pairings),
 *	the number of cleavages, and an array storing the positions of the cleavages
 *	and returns the length of the sequence of the motif
 *	Arrays need to be initilized before passing into the function, otherwise updated information will be lost
 *	No strict format checking is implemented here, make sure to specify a correct file.
*/
int ReadMotif_BPSEQ(char *filename, char *seqname, char *sequence, int *pairs, int *crossingPairs, BasepairCount *motifBPcount, int *numCleavage, int *Cleavages){
	FILE *fp;
	char *temp = (char*) calloc(MAXSEQ, sizeof(char));
	if((fp = fopen(filename, "r")) == NULL){
		printf("Cannot open Motif file, please check!\n");
		exit(0);
	}
	// check for format, see whether starts with '>'
	if(fgets(temp, MAXSEQ, fp) == NULL){
		printf("Empty file, motif!\n");
		exit(0);
	}
	if(temp[0] != '>'){
		printf("Please check input file format, motif.\n");
		exit(0);
	}
	sscanf(temp, ">%s", seqname);
	// reads in sequence
	fscanf(fp, "%s\n", temp);
	int seqLength = strlen(temp);
	// process the sequence, find out the location it is broken
	int i,j=0,k=0;
	*numCleavage = 0;
	Cleavages[0] = -1;
	for(i=0; i<seqLength; ++i){
		if(temp[i] != '.'){
			sequence[j] = temp[i];
			++j;
		}else{
			/* the cleavage usually represents by an eclips, we only count the dot once*/
			if(temp[i - 1] != '.'){
				++(*numCleavage);				
				Cleavages[k] = j;
				++k;
			}	
		}
	}
	seqLength = j;
	char *seq_b = calloc(seqLength, sizeof(char));
	for(i=0; i<seqLength; ++i){
		seq_b[i] = Char2int(sequence[i]);
	}
	// check for format, see whether ends with '//'
	fscanf(fp, "%s\n", temp);
	if(temp[0] != '/' || temp[1] != '/'){
		printf("Please check input file format, motif.\n");
		exit(0);
	}
	free(temp);
	
	char loc;				// indicator of non-crossing pairs: '#' for non-crossing pairs, '@' for crossing pairs
	int index1;				// index of the first pairing nucleotide
	int index2;				// index of the second pairing nucleotide
	char nt1;				// first pairing nucleotide, just used for checking
	char nt2;				// second pairing nucleotide, just used for checking
	char eg1;				// pairing edge of the first nucleotide
	char eg2;				// pairing edge of the second nucleotide
	char orientation[6];			// base orientation: 'cis' or 'trans'

	int pairIndex = 0;			// index used in the pairs array
	int crossIndex = 0;			// index used in the crossingPairs array

	// reads in pairing information in the motif file
	while(fscanf(fp, "%c	%d-%d : %c-%c %c/%c %s\n", &loc, &index1, &index2, &nt1, &nt2, &eg1, &eg2, orientation) != EOF){
		// check for inconsistencies in motif, comparing nucleotide in pairing and nucleotide in sequence
		if(nt1 != sequence[index1] || nt2 != sequence[index2]){
			printf("Inconsistency in Motif Input, please check index %d and %d.\n", index1, index2);
			exit(0);
		}else{
			// check whether the base-pair is assigned to be crossing
			// data structure of 'pairs' and 'crossingPairs':
			// 	6 elements per a pair
			//	index1	edge1	cis/trans	index2	edge2	cis/trans 
			if(loc == '#'){
				pairs[pairIndex*6] = index1;
				switch(eg1){
					case 'W':
						pairs[pairIndex*6+1] = 1;
						break;
					case 'H':
						pairs[pairIndex*6+1] = 2;
						break;
					case 'S':
						pairs[pairIndex*6+1] = 3;
						break;
					default:
						printf("Error! Please check motif or target input file: edge error.\n");
						exit(0);
				}			
				pairs[pairIndex*6+3] = index2;
				switch(eg2){
					case 'W':
						pairs[pairIndex*6+4] = 1;
						break;
					case 'H':
						pairs[pairIndex*6+4] = 2;
						break;
					case 'S':
						pairs[pairIndex*6+4] = 3;
						break;
					default:
						printf("Error! Please check motif or target input file: edge error.\n");
						exit(0);
				}	
				if(strcmp(orientation, "cis") == 0){
					pairs[pairIndex*6+2] = 0;
					pairs[pairIndex*6+5] = 0;
				}else if(strcmp(orientation, "trans") == 0){
					pairs[pairIndex*6+2] = 1;
					pairs[pairIndex*6+5] = 1;
				}
				//	if(IsValidPair(&pairs[6 * pairIndex], seq_b)){
				++pairIndex;
				//	}
			}else if(loc == '@'){
				crossingPairs[crossIndex*6] = index1;
				switch(eg1){
					case 'W':
						crossingPairs[crossIndex*6+1] = 1;
						break;
					case 'H':
						crossingPairs[crossIndex*6+1] = 2;
						break;
					case 'S':
						crossingPairs[crossIndex*6+1] = 3;
						break;
					default:
						printf("Error! Please check motif or target input file: edge error.\n");
						exit(0);
				}			
				crossingPairs[crossIndex*6+3] = index2;
				switch(eg2){
					case 'W':
						crossingPairs[crossIndex*6+4] = 1;
						break;
					case 'H':
						crossingPairs[crossIndex*6+4] = 2;
						break;
					case 'S':
						crossingPairs[crossIndex*6+4] = 3;
						break;
					default:
						printf("Error! Please check motif or target input file: edge error.\n");
						exit(0);
				}
				if(strcmp(orientation, "cis") == 0){
					crossingPairs[crossIndex*6+2] = 0;
					crossingPairs[crossIndex*6+5] = 0;
				}else if(strcmp(orientation, "trans") == 0){
					crossingPairs[crossIndex*6+2] = 1;
					crossingPairs[crossIndex*6+5] = 1;
				}
				//	if(IsValidPair(&crossingPairs[6 * crossIndex], seq_b)){
				++crossIndex;
				//	}
			}
		}
	}
	/* set the number of base-pairs */
	motifBPcount->pairCount = pairIndex;
	motifBPcount->crossCount =crossIndex; 
	fclose(fp);
	/* sort the base-pairs */
	SortBasepair(pairs, pairIndex);
	free((void *) seq_b);
	return(seqLength);
}

int ReadMotif_STK(char *filename, char *seqname, char *sequence, int *pairs, int *crossingPairs, BasepairCount *motifBPcount, int *numCleavage, int *Cleavages){	
	FILE *fp;
	int i,j=0,k=0;
	char *temp1 = (char *) calloc(MAXSEQ, sizeof(char));
	char *temp2 = (char *) calloc(MAXSEQ, sizeof(char));
	if((fp = fopen(filename, "r")) == NULL){
		printf("Cannot open Target file, please check!\n");
		exit(0);
	}
	// check for format, see whether starts with '>'
	if(fgets(temp1, MAXSEQ, fp) == NULL){
		printf("Empty file, Target!\n");
		exit(0);
	}
	if(temp1[0] != '>'){
		printf("Please check input file format, motif.\n");
		exit(0);
	}
	sscanf(temp1, ">%s", seqname);
	// reads in sequence
	fscanf(fp, "%s\n", temp1);
	int seqLength = strlen(temp1);
	char *seq_b = calloc(seqLength, sizeof(char));
	for(i=0; i<seqLength; ++i){
		if(temp1[i] != '.'){
		seq_b[i] = Char2int(temp1[i]);
		}else{
			seq_b[i] = '.';
		}
	}
	/* define index to deal with the omited sequence case*/
	int *index = (int *) calloc(seqLength, sizeof(int));
	// process the sequence, find out the location it is broken
	
	*numCleavage = 0;
	Cleavages[0] = -1;
	for(i=0; i<seqLength; ++i){
		index[i] = -1;
		if(temp1[i] != '.'){
			sequence[j] = temp1[i];
			index[i] = j;
			++j;
		}else{
			/* the cleavage usually represents by an eclips, we only count the dot once*/
			if(temp1[i - 1] != '.'){
				++(*numCleavage);				
				Cleavages[k] = j;
				++k;
			}	
		}
	}
	if(fgets(temp1, MAXSEQ, fp) == NULL){
		printf("No pairing information, please check Motif input!\n");
		exit(0);
	}
	/* check for format*/
	if(temp1[0] != '/' || temp1[1] != '/'){
		printf("Invalid format, please check Motif input!\n");
		exit(0);
	}
	/* dealing with the nested case*/
	int numPairs = 0;
	while(fscanf(fp, "%s", temp1) != EOF && temp1[0] != '@' && temp1[1] != '@' && fscanf(fp, "%s", temp2) != EOF){
		if(strlen(temp1) != seqLength || strlen(temp2) != seqLength){
			printf("Incompatable structure length in Motif input!\n");
			exit(0);
		}
		numPairs += STK2Bpseq(&pairs[6 * numPairs], seq_b, temp1, temp2, seqLength, '(', ')');
	}
	/* change the basepair's index into actual sequence index (after removing the cleavage points)*/
	for(i=0; i<numPairs; ++i){
		pairs[6 * i] = index[pairs[6 * i]];
		pairs[6 * i + 3] = index[pairs[6 * i + 3]];
		if(pairs[6 * i] == -1 || pairs[6 * i + 3] == -1){
			printf("Wrong structure input: structures may exists in cleavage sites\n");
		}
	}
	/* records the basepair count*/
	motifBPcount->pairCount = numPairs;
	/* dealing with the crossing basepair case*/
	int numCrossing = 0;
	while(fscanf(fp, "%s", temp1) != EOF && fscanf(fp, "%s", temp2) != EOF){
		if(strlen(temp1) != seqLength || strlen(temp2) != seqLength){
			printf("Incompatable structure length in Motif input!\n");
			exit(0);
		}
		numCrossing += STK2Bpseq(&crossingPairs[6 * numCrossing], seq_b, temp1, temp2, seqLength, '<', '>');
	}
	/* change the basepair's index into actual sequence index (after removing the cleavage points)*/
	for(i=0; i<numCrossing; ++i){
		crossingPairs[6 * i] = index[crossingPairs[6 * i]];
		crossingPairs[6 * i + 3] = index[crossingPairs[6 * i + 3]];
		if(crossingPairs[6 * i] == -1 || crossingPairs[6 * i + 3] == -1){
			printf("Wrong structure input: structures may exists in cleavage sites\n");
		}
	}
	motifBPcount->crossCount = numCrossing;
	/* gets the actual sequence length, which is the last element's index*/
	seqLength = index[seqLength - 1] + 1;
	sequence[seqLength] = '\0';
	SortBasepair(pairs, numPairs);
	free((void *) temp1);
	free((void *) temp2);
	free((void *) index);
	free((void *) seq_b);
	return seqLength;
}

/*
 *	ReadTarget
 *
 *	the function takes target file name as input, two arrays to store the result(sequence and pairing), number of cleavages and array storing 
 *	the positions of the cleavages
 *	Arrays need to be initilized before passing into the function, otherwise updated information will be lost
 *	The function is similar with ReadMotif, except that it reads all pairings and do not distinguish crossing and non-crossing
 *	No strict format checking is implemented here, make sure to specify a correct file.
*/
int ReadTarget_BPSEQ(char *filename, char *seqname, char *sequence, int *pairs, BasepairCount *targetCount){
	FILE *fp;
	char *temp = (char*) calloc(MAXSEQ, sizeof(char));
	if((fp = fopen(filename, "r")) == NULL){
		printf("Cannot open Target file, please check!\n");
		exit(0);
	}
	// check for format, see whether starts with '>'
	if(fgets(temp, MAXSEQ, fp) == NULL){
		printf("Empty file, Target!\n");
		exit(0);
	}
	if(temp[0] != '>'){
		printf("Please check input file format, motif.\n");
		exit(0);
	}
	sscanf(temp, ">%s", seqname);
	// reads in sequence
	fscanf(fp, "%s\n", temp);
	int seqLength = strlen(temp);
	// process the sequence, find out the location it is broken
	int i,j=0,k=0;
	for(i=0; i<seqLength; ++i){
		if(temp[i] != '.'){
			sequence[j] = temp[i];
			++j;
		}
	}
	seqLength = j;
	char *seq_b = calloc(seqLength, sizeof(char));
	for(i=0; i<seqLength; ++i){
		seq_b[i] = Char2int(sequence[i]);
	}
	// check for format, see whether ends with '//'
	fscanf(fp, "%s\n", temp);
	if(temp[0] != '/' || temp[1] != '/'){
		printf("%c	%c\n", temp[0], temp[1]);
		printf("Please check input file format.\n");
		exit(0);
	}
	free(temp);
	
	char loc;				// indicator of non-crossing pairs: '#' for non-crossing pairs, '@' for crossing pairs
	int index1;				// index of the first pairing nucleotide
	int index2;				// index of the second pairing nucleotide
	char nt1;				// first pairing nucleotide, just used for checking
	char nt2;				// second pairing nucleotide, just used for checking
	char eg1;				// pairing edge of the first nucleotide
	char eg2;				// pairing edge of the second nucleotide
	char orientation[6];			// base orientation: 'cis' or 'trans'

	int pairIndex = 0;			// index used in the pairs array

	// reads in pairing information in the motif file
	while(fscanf(fp, "%c	%d-%d : %c-%c %c/%c %s\n", &loc, &index1, &index2, &nt1, &nt2, &eg1, &eg2, orientation) != EOF){
		// check for inconsistencies in motif, comparing nucleotide in pairing and nucleotide in sequence
		if(nt1 != sequence[index1] || nt2 != sequence[index2]){
			printf("Inconsistency in Target Input, please check index %d and %d.\n", index1, index2);
			exit(0);
		}else{
			// check whether the base-pair is assigned to be crossing
			// data structure of 'pairs' and 'crossingPairs':
			// 	6 elements per a pair
			//	index1	edge1	cis/trans	index2	edge2	cis/trans 
			if(loc == '#'){
				pairs[pairIndex*6] = index1;
				switch(eg1){
					case 'W':
						pairs[pairIndex*6+1] = 1;
						break;
					case 'H':
						pairs[pairIndex*6+1] = 2;
						break;
					case 'S':
						pairs[pairIndex*6+1] = 3;
						break;
					default:
						printf("Error! Please check motif or target input file: edge error.\n");
						exit(0);
				}			
				pairs[pairIndex*6+3] = index2;
				switch(eg2){
					case 'W':
						pairs[pairIndex*6+4] = 1;
						break;
					case 'H':
						pairs[pairIndex*6+4] = 2;
						break;
					case 'S':
						pairs[pairIndex*6+4] = 3;
						break;
					default:
						printf("Error! Please check motif or target input file: edge error.\n");
						exit(0);
				}	
				if(strcmp(orientation, "cis") == 0){
					pairs[pairIndex*6+2] = 0;
					pairs[pairIndex*6+5] = 0;
				}else if(strcmp(orientation, "trans") == 0){
					pairs[pairIndex*6+2] = 1;
					pairs[pairIndex*6+5] = 1;
				}
				//	if(IsValidPair(&pairs[6 * pairIndex], seq_b)){
					//printf("pairIndex	%d	location:	%d - %d\n", pairIndex, pairs[6 * pairIndex], pairs[6 * pairIndex + 3]);
				++pairIndex;
				//	}
			}
		}
	}
	// set the number of base-pairs
	targetCount->pairCount = pairIndex;
	targetCount->crossCount = 0;
	fclose(fp);
	/*
	for(i = 0; i < pairIndex; ++ i)	{
		printf("%d	%d	%d	%d	%d	%d\n", pairs[i * 6], pairs[i * 6  + 1], pairs[i * 6 + 2], pairs[i * 6 + 3], pairs[i * 6 + 4], pairs[i * 6 + 5]);
	}
	printf("@@@@@\n");
	*/
	// sort the base-pairs
	SortBasepair(pairs, pairIndex);
	
	
	free((void *) seq_b);
	return(seqLength);
}


int ReadTarget_STK(char *filename, char *seqname, char *sequence, int *pairs, BasepairCount *targetCount){	
	FILE *fp;
	int i,j=0,k=0;
	char *temp1 = (char *) calloc(MAXSEQ, sizeof(char));
	char *temp2 = (char *) calloc(MAXSEQ, sizeof(char));
	if((fp = fopen(filename, "r")) == NULL){
		printf("Cannot open Target file, please check!\n");
		exit(0);
	}
	// check for format, see whether starts with '>'
	if(fgets(temp1, MAXSEQ, fp) == NULL){
		printf("Empty file, Target!\n");
		exit(0);
	}
	if(temp1[0] != '>'){
		printf("Please check input file format, motif.\n");
		exit(0);
	}
	sscanf(temp1, ">%s", seqname);
	// reads in sequence
	fscanf(fp, "%s\n", temp1);
	int seqLength = strlen(temp1);
	char *seq_b = calloc(seqLength, sizeof(char));
	for(i=0; i<seqLength; ++i){
		if(temp1[i] != '.'){
		seq_b[i] = Char2int(temp1[i]);
		}else{
			seq_b[i] = '.';
		}
	}
	/* define index to deal with the omited sequence case*/
	int *index = (int *) calloc(seqLength, sizeof(int));
	// process the sequence, find out the location it is broken
	
	for(i=0; i<seqLength; ++i){
		index[i] = -1;
		if(temp1[i] != '.'){
			sequence[j] = temp1[i];
			index[i] = j;
			++j;
		}
	}
	
	if(fgets(temp1, MAXSEQ, fp) == NULL){
		printf("No pairing information, please check Target input!\n");
		exit(0);
	}
	/* check for format*/
	if(temp1[0] != '/' || temp1[1] != '/'){
		printf("Invalid format, please check Target input!\n");
		exit(0);
	}
	int numPairs = 0;
	while(fscanf(fp, "%s", temp1) != EOF && fscanf(fp, "%s", temp2) != EOF){
		if(strlen(temp1) != seqLength || strlen(temp2) != seqLength){
			printf("Incompatable structure length in Target input!\n");
			exit(0);
		}
		numPairs += STK2Bpseq(&pairs[6 * numPairs], seq_b, temp1, temp2, seqLength, '(', ')');
	}
	/* change the basepair's index into actual sequence index (after removing the cleavage points)*/
	for(i=0; i<numPairs; ++i){
		pairs[6 * i] = index[pairs[6 * i]];
		pairs[6 * i + 3] = index[pairs[6 * i + 3]];
		if(pairs[6 * i] == -1 || pairs[6 * i + 3] == -1){
			printf("Wrong structure input: structures may exists in cleavage sites\n");
		}
	}
	targetCount->pairCount = numPairs;
	SortBasepair(pairs, numPairs);
	/* gets the actual sequence length, which is the last element's index*/
	seqLength = index[seqLength - 1] + 1;
	sequence[seqLength] = '\0';
	free((void *) temp1);
	free((void *) temp2);
	free((void *) index);
	free((void *) seq_b);
	return seqLength;
}

/*
 * ReadIsostericMatrix
 *
 * The function reads in the isosteric matrix and put it in an 3D array
 * The function takes the Isosteric Matrix as input, and an three-dimensional array as result storage.
 * The size of the array is fixed to be 12*4*4 (12 isosteric matrix)
 * No strict format checking is performed, make sure to specify the correct file
*/
void ReadIsostericMatrix(char *filename){
	extern int isoIndex[18][4][4];
	FILE *fp;
	if((fp = fopen(filename, "r")) == NULL){
		printf("Cannot open isostericity file, please check!\n");
		exit(0);
	}
	char *temp = (char*) calloc (50, sizeof(char));
	// ignore the heading information
	while(1){
		if(fscanf(fp, "%s\n", temp) == EOF){
			printf("Error! Please check format of input file of isostericity matrix.\n");
			exit(0);
		}
		// check for ending character
		if(strcmp(temp, "@") == 0){
			break;
		}
	}
	free(temp);
	// reads indices into the matrix
	int i, j, k;
	for(i=0; i<18; ++i){
		for(j=0; j<4; ++j){
			if(fscanf(fp, "%d	%d	%d	%d\n", 
					&isoIndex[i][j][0], &isoIndex[i][j][1], &isoIndex[i][j][2], &isoIndex[i][j][3]) == EOF){
				printf("Error Please check the numbers in the isostericity matrix.\n!");
				exit(0);
			}
		}
	}
	fclose(fp);
	return;
}

/*
 * ReadScoringFunction
 *
 * The function reads the scoring function into an array
 * The input of the function is the file name, and the 2-dimensional array storing the results
 * The size of the array is fixed to be 44*44, because there are totally 44 different isosteric groups
 * No format checking in the function, make sure to specify the right file name
*/
void ReadScoringFunction(char *filename){

	extern double scoreMatrix[66][66];


	FILE *fp;
	if((fp = fopen(filename, "r")) == NULL){
		printf("Cannot find the file specified, program runs with default score.\n");
		return;
	}
	int i,j;
	// reads the score one by one, the score should be integers
	for(i=0; i<66; ++i){
		for(j=0; j<66; ++j){
			if(fscanf(fp, "%lf", &scoreMatrix[i][j]) == EOF){
				printf("Double: %lf\n",scoreMatrix[i][j]);
				printf("Error! Please check the scoring matrix!\n");
				exit(0);
			}
		}
	}
	fclose(fp);


	return;

}

int QueryIsoIndex(int i, int j, int k){
	return (isoIndex[i][j][k]);
}

double QueryScore(int i, int j){
	return (scoreMatrix[i][j]);
}
