#define MAXPAIR 3000
typedef struct PARASCORE{
        int index1, index2;
        double score;
        struct PARASCORE *next;
}PARASCORE;

typedef struct MATCH{
	int	pair[MAXPAIR];
	int	numbasepair;
	double	score;
}MATCH;

extern int W[15][15];
extern int total_nuc;
extern char na_name[17];

static double **HairpinSim;
static double **BasepairSim;
static struct PARASCORE ***ParaScore;
static double **LeftLoopSim;
static double **RightLoopSim;



/*
 * FillHairpinSim
 *
 * The function computes the hairpin loop sequence similarity between any two base-pairs
 * and fills the result in the matrix 'HairpinSim', which is defined in 'ComputeScore.h'
*/
void FillHairpinSim(char *seq1, char *seq2, int *pair1, int numPair1, int *pair2,  int numPair2, int numCleavage, int *Cleavages);
double GetHairpinSim(int i, int j);
/*
 * FillBasepairSim
 *
 * The function computes the base-pair similarity between any two base-pairs
 * and fill the results in the matrix 'BasepairSim', which is defined in 'ComputeScore.c'
*/
void FillBasepairSim(char *seq1, char *seq2, int *pair1, int numPairs1, int *pair2, int numPairs2);
double GetBasepairSim(int i, int j);
/*
 * FillParaScore
 *
 * The function is an interface of calling 'ComputeParaScore' several times
 * It is used to fill the 'ParaScore' matrix
 * The input list contains: sequence 1, length of sequence 1, sequence 2, length of sequence 2, basepairs in sequence 1, basepairs in sequence 2,
 * number of basepairs in sequence 1, number of basepairs in sequence 2 
 * No value is returned
*/
void FillParaScore(char *seq1, int len_seq1, char *seq2, int len_seq2, int *pairs1, int numPairs1, int *pairs2, int numPairs2,
			int numCleavage, int *Cleavages);
PARASCORE *GetParaHead(int i, int j);
/*
 *	CalChainScore
 *
 *	Given two basepairs, the function calcualtes the optimum score of matching the basepairs enclosed by the two given pairs
*/
double CalChainScore(
			/* index of the two stacks*/
			int mout1, 			/* index of the stack in the first sequence that enclosing the chain*/
			int mout2, 			/* index of the stack in the second sequence that enclosing the chain*/
			/* information of the first pair*/
			int *pair1, 			/* predicted basepairs*/
			int *downarray1, 		/* the basepairs that are enclosed by mout1*/
			int sizedownarray1, 		/* number of basepairs enclosed by mout1*/
			int numPair1,			/* number of predicted basepairs in sequence 1*/
			/* information of the second pair*/
			/* commets refers to the comments of the first pair*/
			int *pair2, 
			int *downarray2, 
			int sizedownarray2, 
			int numPair2,
			/* match is the score of matching base-pairs, corresponding to M[Pa, Pb] is the recursive function*/
			/* this is a global variable, passed in for dynamic programming*/
			MATCH **match, 
			int **numContStacking,
			/* matchchain is the matrix for doing dynamic programming of the chains, locally used*/
			MATCH *matchchain, 
			/* the score is the score of matching the sequence of the regions, assuming there is no basepair inside*/
			/* the score is compared inside the function, if it is higher, it is returned back*/
			double mscore0,
			/* the hairpin region inside the stack*/
			int range11, int range12, 
			int range21, int range22, 
			/* the sequences*/
			char *seq_b1, 
			char *seq_b2, 
			/* length of sequence*/
			int len_seq1, 
			int len_seq2,
			int numCleavage,
			int *Cleavages,
			int LastTag
);

PARASCORE *FreeParaLoop(PARASCORE *parascore);
void FreeLoopsim(int numPairs1, int numPairs2);
double AfterCheck(char *seq1, char *seq2, int seqLeng1, int seqLeng2, 
		int *pair1, int *pair2, int numPair1, int numPair2, 
		int *crossPair, int numCrossPair, MATCH *matchPairs, int *matchedpFin1, int *matchedpFin2);
void SetScore(char p, double v);
double CheckScore(int *pair1, int numPairs1, int *pair2, int numPairs2, int *crossPair, int numCross, 
			int *matchedpFin1, int *matchedpFin2, int Prematched, int nummatched, 
			char *seq1, int seqLeng1, char *seq2, int seqLeng2, int numCleavage, int *Cleavages);
