#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "./inc/ReadInput.h"
#include "./inc/BasepairMan.h"
#include "./inc/NucleotideHash.h"
#include "./inc/WeightNC.h"
#include "./inc/ALIGN.h"
#include "./inc/BuildHashes.h"
#include "./inc/RetrieveScore.h"
#include "./inc/ComputeScore.h"
#include "./inc/AlignRNAPairs.h"
#include "./inc/OutputMPairs.h"
#include "./inc/Parameter.h"


int main(int argc, char **argv){
	int i, j, k;
	/* the file name of motif and target*/
	char *file1 = (char*) calloc(200, sizeof(char));
	char *file2 = (char*) calloc(200, sizeof(char));
	/* allocate memory for the two sequence and their name*/
	char *seqname1 = (char*) calloc(200, sizeof(char));
	char *seqname2 = (char*) calloc(200, sizeof(char));
	char *sequence1 = (char*) calloc(MAXSEQ, sizeof(char));
	char *sequence2 = (char*) calloc(MAXSEQ, sizeof(char));
	/* allocate memory for storing the basepair information*/
	int *pairs1 = (int*) calloc(6 * MAXPAIR, sizeof(int));
	int *pairs2 = (int*) calloc(6 * MAXPAIR, sizeof(int));
	/* allocate memory for storing crossing basepair information*/
	int *crosspairings = (int *) calloc(600, sizeof(int));
	/* allocate memory for recording cleavage sites*/
	int numcleavage;
	int *cleavage = (int *) calloc(200, sizeof(int));
	/* allocate memory for recording matched basepairs*/
	int *matchedpFin1 = (int*) calloc(MAXPAIR * 6, sizeof(int));
	int *matchedpFin2 = (int*) calloc(MAXPAIR * 6, sizeof(int));
	/* records the length of the sequences*/
	int seqleng1, seqleng2;
	/* records number of matched basepairs in the nested structure*/
	int numPrematch;
	/* records how many basepairs are there in each sequence*/
	BasepairCount *bpcMotif = (BasepairCount*) calloc(1, sizeof(BasepairCount));
	BasepairCount *bpcTarget = (BasepairCount*) calloc(1, sizeof(BasepairCount));
	/* allocate memory for the name of the isosteric matrix and score of isosteric groups*/
	char *isofile = (char*) calloc(200, sizeof(char));
	char *scorefile = (char*) calloc(200, sizeof(char));
	/* match is used to store which basepairs are matched*/
	MATCH *match = (MATCH*) calloc(1, sizeof(MATCH));
	/* score is used to store the alignment score between query and target*/
	double score, checkscore;
	/* variables for the environment seting*/
	int copt;	
	extern char *optarg;
	int inpseq1B = 0, inpseq2B = 0, inpseq1S = 0, inpseq2S = 0;
	double tempValue;
	while ((copt=getopt(argc,argv,"q:t:i:j:s:e:k:l:y:u:v:a:b:c:w:o:p:")) != EOF)	{
		switch(copt) {
			case 'q':
				inpseq1B = 1;
				sscanf(optarg, "%s", file1);
				seqleng1 = ReadMotif_BPSEQ(file1, seqname1, sequence1, pairs1, crosspairings, bpcMotif, &numcleavage, cleavage);
				continue;
			case 't':
				inpseq2B = 1;
				sscanf(optarg, "%s", file2);
				seqleng2 = ReadTarget_BPSEQ(file2, seqname2, sequence2, pairs2, bpcTarget);
				continue;
			case 'i':
				inpseq1S = 1;
				sscanf(optarg, "%s", file1);
				seqleng1 = ReadMotif_STK(file1, seqname1, sequence1, pairs1, crosspairings, bpcMotif, &numcleavage, cleavage);
				continue;
			case 'j':
				inpseq2S = 1;
				sscanf(optarg, "%s", file2);
				seqleng2 = ReadTarget_STK(file2, seqname2, sequence2, pairs2, bpcTarget);
				continue;
			case 'w':
				sscanf(optarg, "%s", isofile);
				ReadIsostericMatrix(isofile);
				continue;
			case 'o':
				sscanf(optarg, "%s", scorefile);
				ReadScoringFunction(scorefile);
				continue;
			case 's':
			case 'e':
			case 'k':
			case 'l':
			case 'y':
			case 'u':
			case 'v':
			case 'a':
			case 'b':
			case 'c':
			case 'p':
				sscanf(optarg, "%lf", &tempValue);
				SetScore(copt, tempValue);
				continue;
			default:
			 printf("RNAMotifAlign -q [queryBPSEQ] -t [targetBPSEQ] -i [querySTK] -j [targetSTK] -s [GAP start] \n	-e [GAP extend] -k [Same G group bonus] -l [Same H group bonus] -y [stacking bonus] -u [Penalty query] -v [Penalty target] \n	-a [weight1] -b [weight2] -c [weight3] -g [Penalty G group] -h [Penalty H group] \n	-w [isosteric group file] -o [isosteric group score file] -p [Penalty for stacking violation]\n");
			 printf("You need to and can only specify one query input and one target input.\n");
			 printf("-q queryBPSEQ: the query file name in BPSEQ format\n");
			 printf("-t targetBPSEQ: the target file name in BPSEQ format\n");
			 printf("-i querySTK: the query file name in STK format\n");
			 printf("-j targetSTK: the target file name in STK format\n");
			 printf("\n");
			 printf("-s GAP start (optional): the gap open penalty in terms of sequence similarity (default 3)\n");
			 printf("-e GAP extend (optional): the gap extension penalty in terms of sequence similarity (default 2)\n");
			 printf("-k Same G group bonus (optional): the bonus for 1-n pairing in the 'G' situation (default 3)\n");
			 printf("-l Same H group bonus (optional): the bonus for 1-n pairing in the 'H' situation (default 4)\n");
			 printf("-y Stacking bonus (optional): the unit bonus for stacking effect (default 1)\n");
			 printf("-u Penalty query (optional): the penalty of missing one base-pair in query structure (default 2)\n"); 
			 printf("-v Penalty target (optional): the penalty of missing one base-pair in target structure (default 1)\n");
			 printf("-a weight1 (optional): the weight for loop region sequence similarity (default 0.05)\n");
			 printf("-b weight2 (optional): the weight for paired region sequence similarity (default 0.05)\n");
			 printf("-c weight3 (optional): the weight for base-pair similarity in terms of isoteric group (default 0.90)\n");
			 printf("-w isosteric group file (optional): the file encoding all isosteric information between base-pairs\n");
			 printf("-o isosteric score file (optional): the file specifies the matching score between two isosteric groups\n");
			 printf("-p Penalty for staking violation (optional): penalty score for an empty sequence matching a non-empty sequence, weighted with weight1 (default 200)\n");
			 exit(-1);
		}
		optind--;
	}	
	if(inpseq1B && inpseq1S){
		printf("Error! Multiple motif file specified!\n");
		exit(0);
	}
	if(inpseq2B && inpseq2S){
		printf("Error! Multiple target file specified!\n");
		exit(0);
	}
	if(!inpseq1B && !inpseq1S || !inpseq2B && !inpseq2S){
		printf("Not enough input has been specified: cannot start program\n");
		printf("RNAMotifAlign -q [queryBPSEQ] -t [targetBPSEQ] -p [querySTK] -r [targetSTK] -s [GAP start] -e [GAP extend] -u [Penalty query] -v [Penalty target]\n	-a [weight1] -b [weight2] -c [weight3] -g [Penalty G group] -h [Penalty H group] -w [isosteric group file] -o [isosteric group score file]\n");
		printf("You need to and can only specify one query input and one target input.\n");
		printf("-q queryBPSEQ: the query file name in BPSEQ format\n");
		printf("-t targetBPSEQ: the target file name in BPSEQ format\n");
		printf("-i querySTK: the query file name in STK format\n");
		printf("-j targetSTK: the target file name in STK format\n");
		printf("\n");
		printf("-s GAP start (optional): the gap open penalty in terms of sequence similarity (default 3)\n");
		printf("-e GAP extend (optional): the gap extension penalty in terms of sequence similarity (default 2)\n");
		printf("-k Same G group bonus (optional): the bonus for 1-n pairing in the 'G' situation (default 3)\n");
		printf("-l Same H group bonus (optional): the bonus for 1-n pairing in the 'H' situation (default 4)\n");
		printf("-y Stacking bonus (optional): the unit bonus for stacking effect (default 1)\n");
		printf("-u Penalty query (optional): the penalty of missing one base-pair in query structure (default 2)\n"); 
		printf("-v Penalty target (optional): the penalty of missing one base-pair in target structure (default 1)\n");
		printf("-a weight1 (optional): the weight for loop region sequence similarity (default 0.05)\n");
		printf("-b weight2 (optional): the weight for paired region sequence similarity (default 0.05)\n");
		printf("-c weight3 (optional): the weight for base-pair similarity in terms of isoteric group (default 0.95)\n");
		printf("-w isosteric group file (optional): the file encoding all isosteric information between base-pairs\n");
		printf("-o isosteric score file (optional): the file specifies the matching score between two isosteric groups\n");
		exit(0);
	}
	
	
	score = AlignRNAPairs(sequence1, seqleng1, sequence2, seqleng2, pairs1,
		bpcMotif->pairCount, pairs2, bpcTarget->pairCount, numcleavage, cleavage, match);
	/*
	printf("score before AfterCheck: %f\n", score);
	*/
	//printf("finish alignRNApairs\n");
	numPrematch = match->numbasepair;
	if(numPrematch <= 0){
		return;
	}
	//printf("numPrematch: %d\n", numPrematch);
	score = AfterCheck(sequence1, sequence2, seqleng1, seqleng2, pairs1, pairs2, bpcMotif->pairCount, bpcTarget->pairCount, 
			crosspairings, bpcMotif->crossCount, match, matchedpFin1, matchedpFin2);
	/*
	printf("score after AfterCheck: %f\n", score);
	*/
	/*
	checkscore = CheckScore(	pairs1, bpcMotif->pairCount, pairs2, bpcTarget->pairCount, crosspairings, bpcMotif->crossCount, 
			matchedpFin1, matchedpFin2, numPrematch, match->numbasepair,  
			sequence1, seqleng1, sequence2, seqleng2, numcleavage, cleavage);
	if(score > checkscore + 0.1 || score < checkscore - 0.1){
		printf("Check Score error! Alignment Score: %f; Checked Score: %f.\n", score, checkscore);
	}
	*/
	/*
	for(i=0; i<seqleng1; ++i){
		printf("%c", sequence1[i]);
	}	
	printf("\n");
	for(i=0; i<seqleng2; ++i){
		printf("%c", sequence2[i]);
	}	
	printf("\n");
	printf("matched pairs\n");
	for(i=0; i<numPrematch; ++i){
		printf("%d	%d	|	%d	%d\n", matchedpFin1[6 * i], matchedpFin1[6 * i + 3], matchedpFin2[6 * i], matchedpFin2[6 * i + 3]);
	}
	printf("\n");
	*/
	/*
	OutputMPairs(sequence1, seqname1, seqleng1, sequence2, seqname2, seqleng2, pairs1, pairs2, bpcMotif->pairCount, bpcTarget->pairCount,
			crosspairings, bpcMotif->crossCount, match, numPrematch, matchedpFin1, matchedpFin2, cleavage, numcleavage);
	*/
	//printf("Alignment Score: %.3lf\n\n", score);
	printf("program successed:	%s	%s	", seqname1, seqname2);
	printf("score:	%.3lf\n",score);
	free((void*) file1);
	free((void*) file2);
	free((void*) seqname1);
	free((void*) seqname2);
	free((void*) isofile);
	free((void*) scorefile);
	free((void*) sequence1);
	free((void*) sequence2);
	free((void*) pairs1);
	free((void*) pairs2);
	free((void*) matchedpFin1);
	free((void*) matchedpFin2);
	free((void*) match);
	free((void*) bpcMotif);
	free((void*) bpcTarget);
	free((void*) crosspairings);
	free((void*) cleavage);
	return;
}

