##############################################################################
#
#	The program is used to extract RNA secondary structure information
#	from the result of MC-Annotate.
#
#	Writtetn by Cun-cong Zhong @ Jan,27,2008
#	
#
###############################################################################

#!usr/bin/perl -w
use strict;
use Cwd 'abs_path';

my $MCDir = shift;
$MCDir = abs_path($MCDir);
open SEQUENCE, "<pdb_seqres.txt" or die "Cannot open file\n";
my %Type_hash;			#Hash table that determins each chain's type
my %Ref_Seq_hash;
while(<SEQUENCE>)
{
	chomp;
	my $chain_name;
	if(/mol:protein/)
	{
		$chain_name=substr($_,1,6);
		$Type_hash{$chain_name}='P';
	}
	elsif(/mol:na/)
	{
		$chain_name=substr($_,1,6);
		$Type_hash{$chain_name}='N';
		my $seq;
		$seq=<SEQUENCE>;
		chomp $seq;
		$Ref_Seq_hash{$chain_name}=$seq;
	}
	else
	{
		next;
	}
}
#	sort by number
sub by_num
{
	$a <=> $b
}
my %canonical_hash=
(
	"A" => 'U',
	"U" => 'A',
	"C" => 'G',
	"G" => 'C',
);
##################################################################
#
#	read old MC-Annotate results
#
##################################################################
foreach(<$MCDir/*.ent.out>){
	$_ =~ /pdb(.*)\.ent\.out/;
	my @MC_Results;		#Array that accepts the whole file
	my %Sequence_hash;	#chain as key, stores sequence
	my $id;			#ID of PDB file
	$id=$1;			
	my $i;
	my $j;
	my $k;
	my $l;
	$i=0;
	open MCR, "<$_" or die "Cannot open file\n";
##################################################################
	while(<MCR>)		#Copy all file into the array
	{
		chomp;
		$MC_Results[$i++]=$_;
	}
	if($i < 2){
		next;		# the structure probabally does not contain RNA
	}
##################################################################
#	Put different information into different blocks
##################################################################
	my @Residue_Block;
	for($i=0;$i<scalar(@MC_Results);$i++){
		if($MC_Results[$i]=~/^Residue conformation/){
			my $j;
			my $k;
			$k=0;
			for($j=$i+1;$j<(@MC_Results);$j++){
				if($MC_Results[$j]=~/^(\w+) \: \w/ || $MC_Results[$j]=~/^(\'\w+\'\w+) \: \w/){
					if($1 =~ /[.-]/){
						next;
					}else{
						$Residue_Block[$k++]=$MC_Results[$j];
					}
				}elsif($MC_Results[$j]=~/\w+/){
					next;
				}else{
					last;
				}
			}
		}
	}
	my @Basepair_Block;
        for($i=0;$i<scalar(@MC_Results);$i++){
		my $lf = 0;
                if($MC_Results[$i]=~/^Base-pairs/){
                        my $j;
                        my $k;
                        $k=0;
                        for($j=$i+1;$j<(@MC_Results);$j++){
                                if($MC_Results[$j]=~/\w+/){
					if(($MC_Results[$j] =~ /^(\w+)\-(\w+) \: \w\-\w/ || $MC_Results[$j] =~ /^(\'\w+\'\w+)\-(\'\w+\'\w+) \: \w\-\w/)
						&& !($1 =~ /\./) && !($2 =~ /\./)){
						#print "$1	$2\n";
                                	        $Basepair_Block[$k]=$MC_Results[$j];
						++$k;
					}
                                }else{
					$lf = 1;
                                	last;
                                }
                        }
                }
		if($lf == 1){
			last;
		}
        }
	
	my @Helices_Block;
	for($i=0;$i<scalar(@MC_Results);$i++){
                if($MC_Results[$i]=~/^Helices/){
                        my $j;
                        my $k;
                        $k=0;
                        for($j=$i+1;$j<(@MC_Results);$j++){
                                if($MC_Results[$j]=~/\w+/){
                                        $Helices_Block[$k++]=$MC_Results[$j];
                                }else{
                                        last;
                                }
                        }
                }
        }
	my @Sequence_Block;
	for($i=0;$i<scalar(@MC_Results);$i++){
                if($MC_Results[$i]=~/^Sequences/){
                        $k=0;
                        for($j=$i+1;$j<(@MC_Results);$j++){
                                if($MC_Results[$j]=~/\w+/){
                                        $Sequence_Block[$k++]=$MC_Results[$j];
                                }else{
                                        next;
                                }
                        }
                }
        }
##############################################################################
#	Residue Block
#	Basepair Block
#	Helices Block
#	Sequence Block
##############################################################################
#
#	Processing Residue block, assign index for each Nuclear Acid residue
#	Protein residues are ignored
#
##############################################################################
	my %Index_hash;		#index hash for residues
	my $chain;		#the processing chain
	my $pre_chain;		#previous processed chain
	my $sequence_chain;	#anchor for identifying starting point
	my $indexing_num;	#indexing number
	my $fwtag;
	$fwtag=0;
	$indexing_num=0;
	foreach (@Residue_Block){	
	#	print "$id	$pre_chain\n";
		if(/^[A-Za-z]/ || /^\'/){
			if(/^\'(\w+)\'/){
				$chain=$1;
			}else{
				$chain=substr($_,0,1);
			}
		#	print "$id      $pre_chain\n";
			my $type_id;
			$type_id=$id.'_'.$chain;
			if($Type_hash{$type_id} eq 'P'){
				next;
			}
			elsif($Type_hash{$type_id} eq 'N'){
			#	print "$id      $pre_chain\n";
				my @local_decom;		#Local decomposition
				@local_decom=split / /,$_;
#				print "local decom	$local_decom[2]\n";
				if(length($local_decom[2]) != 1){
					my $i;
					for($i=0; $i<length($local_decom[2]); $i++){
						my $c = substr($local_decom[2], $i, 1);
						if($c =~ /[AUCG]/){
							$local_decom[2] = $c;
						}
					}
					if(length($local_decom[2]) != 1){
						$local_decom[2] = undef;
					}
				}
				#print "$id      $pre_chain\n";
				if($chain eq $pre_chain){
					if(defined($local_decom[2])){
						$indexing_num++;
						$Index_hash{$local_decom[0]}=$indexing_num;
						$sequence_chain.=$local_decom[2];
					}else{
						next;
					}
				}else{	
					if($fwtag==0){
						$fwtag=1;
					}else{
					#	print "$id      $pre_chain\n";
						if(exists $Sequence_hash{$pre_chain}){
							$Sequence_hash{$pre_chain}.=$sequence_chain;
						}else{
							$Sequence_hash{$pre_chain}=$sequence_chain;
						}
					}
					$sequence_chain=$local_decom[2];
					$indexing_num=0;
					$pre_chain=$chain;
					$Index_hash{$local_decom[0]}=$indexing_num;
				}
			}
		}
	}
#	print "Before alignment\n";
#	foreach(sort keys %Index_hash){
#		print "$_	$Index_hash{$_}\n";
#	}
#	print "$id      $pre_chain\n";
######################################################################################################
#
#	Finding the proper index order for interpreting MC-Annotate results
#
######################################################################################################
#	print "before sequence hash: X40 => $Index_hash{'X40'}\n";
	if(exists $Sequence_hash{$pre_chain}){
		$Sequence_hash{$pre_chain}.=$sequence_chain;
	}else{
		$Sequence_hash{$pre_chain}=$sequence_chain;
	}

	# remove the '?' symbols
	foreach(keys %Sequence_hash){
		$Sequence_hash{$_} =~ s/\?//g;
	}
	my $chn;
	my $idx;
	foreach	(keys %Sequence_hash){
		$chn=$id.'_'.$_;
		my $local_chain;
		$local_chain=$_;
		$idx=index($Ref_Seq_hash{$chn},$Sequence_hash{$_});
#		print "$Ref_Seq_hash{$chn}\n$Sequence_hash{$_}\n";
#		print "$_	$idx\n";
		if($idx>0){
			foreach(keys %Index_hash){
				#print "$_	$local_chain\n";
				if(($_=~/$local_chain/ && $local_chain=~/[A-Za-z]/) || ($_=~/\'$local_chain\'/ && $local_chain=~/[0-9]/)){
					$Index_hash{$_}+=$idx;	#change the sequence reference
				}
			}
		}
		elsif($idx<0){
			if(length($Ref_Seq_hash{$chn}) <= length($Sequence_hash{$_})){
				my $u;
				$u = index($Sequence_hash{$_},$Ref_Seq_hash{$chn});
				if($u > 0){
	                       		$Sequence_hash{$_}=$Ref_Seq_hash{$chn};
					next;
				}
               		}
			#do alignment
			my @ref_seq;
			my @fetched_seq;
			@ref_seq=split //, $Ref_Seq_hash{$chn};
			@fetched_seq=split //, $Sequence_hash{$_};
			my @DP_matrix;
			my @I_matrix;
			my @D_matrix;
			for($i=0;$i<scalar(@fetched_seq)+1;$i++){
				for($j=0;$j<scalar(@ref_seq)+1;$j++){
					$DP_matrix[$i][$j] = 0;
					$I_matrix[$i][$j] = 0;
					$D_matrix[$i][$j] = 0;
				}
			}
			my $match;
			my $mismatch;
			my $gap;
			my $extend;
			my @recs;
			$match = 3;
			$mismatch = -5;
			$gap = -4;
			$extend = -1;
			my $t = $gap;
			# Fill boundary cases
			for($i=1;$i<scalar(@ref_seq)+1;$i++){
				$t = $t + $extend;
				$DP_matrix[$i][0] = $t;
				$D_matrix[$i][0] = $t + $gap;
				$recs[$i][0]='I';
			}
			# Fill boundary cases
			$t = $gap;
			for($j=1;$j<scalar(@fetched_seq)+1;$j++){
				$t = $t + $extend;
                                $DP_matrix[0][$j] = $t;
				$I_matrix[0][$j] = $t + $gap;
				$recs[0][$j]='D';
                        }
			$recs[0][0]='X';
			
			# Filling the matrix
			for($i=1;$i<scalar(@ref_seq)+1;$i++){
				for($j=1;$j<scalar(@fetched_seq)+1;$j++){
					
					# fill the insertion table
					if($DP_matrix[$i-1][$j]+$gap > $I_matrix[$i-1][$j]){
						$I_matrix[$i][$j] = $DP_matrix[$i-1][$j] + $gap + $extend;
					}else{
						$I_matrix[$i][$j] = $I_matrix[$i-1][$j] + $extend;
					}	

					# fill the deletion table
					if($DP_matrix[$i][$j-1]+$gap > $D_matrix[$i][$j-1]){
						$D_matrix[$i][$j] = $DP_matrix[$i][$j-1] + $gap + $extend;
					}else{
						$D_matrix[$i][$j] = $D_matrix[$i][$j-1] + $extend;
					}
					my @candidates;
					push @candidates, $I_matrix[$i][$j];
					push @candidates, $D_matrix[$i][$j];
					if($ref_seq[$i-1] eq $fetched_seq[$j-1]){
						$DP_matrix[$i][$j] = $DP_matrix[$i-1][$j-1]+$match;
						push @candidates,$DP_matrix[$i][$j];
					}else{
						$DP_matrix[$i][$j] = $DP_matrix[$i-1][$j-1]+$mismatch;
						push @candidates,$DP_matrix[$i][$j];
					}
					@candidates=sort by_num @candidates;
				#	print "@candidates	||DP:$DP_matrix[$i][$j]	I:$I_matrix[$i][$j]	D:$D_matrix[$i][$j]\n";
					if($candidates[2] == $I_matrix[$i][$j]){
						$DP_matrix[$i][$j] = $I_matrix[$i][$j];
						$recs[$i][$j] = 'I';
					}elsif($candidates[2] == $D_matrix[$i][$j]){
						$DP_matrix[$i][$j] = $D_matrix[$i][$j];
						$recs[$i][$j] = 'D';
					}elsif($candidates[2] == $DP_matrix[$i][$j]){
                                                $recs[$i][$j] = 'M';
					}
				}
			}
#			for($i=0;$i<scalar(@ref_seq)+1;$i++){
#				for($j=0;$j<scalar(@fetched_seq)+1;$j++){
#					print "$recs[$i][$j] ";
#				}	
#				print "\n";
#			}
			my @ops;
			$i=scalar(@ref_seq);
			$j=scalar(@fetched_seq);
			while($i>0 || $j>0){
				if($recs[$i][$j] eq 'M'){
					push @ops, 'M';
					--$i;
					--$j;
				}elsif($recs[$i][$j] eq 'D'){
                                        push @ops, 'D';
                                        --$j;
                                }elsif($recs[$i][$j] eq 'I'){
                                        push @ops, 'I';
                                        --$i;
                	        }elsif($recs[$i][$j] eq 'X'){
					last;
				}
			}
#			for($i=0;$i<@ref_seq+1;$i++){
#				for($j=0;$j<@fetched_seq+1;$j++){
#					print "$DP_matrix[$i][$j]	";
#				}
#				print "\n";
#			}
#			print "\n";
#			for($i=0;$i<@ref_seq;$i++)
#                        {
#                                for($j=0;$j<@fetched_seq;$j++)
#                                {
#                                        print "$I_matrix[$i][$j]	";
#                                }
#                                print "\n";
#                        }
#			print "\n";
#			for($i=0;$i<@ref_seq;$i++)
#                        {
#                                for($j=0;$j<@fetched_seq;$j++)
#                                {
#                                        print "$D_matrix[$i][$j]	";
#                                }
#                                print "\n";
#                       }
#			print "\n";
			@ops= reverse @ops;
#			print "@ops\n";
#			print "____________________\n";
			my $i_f;	#index for fetched
			my $i_r;	#index for reference
			$i_f=0;
			$i_r=0;
			my %i_fr_hash;
			foreach(@ops){
				if($_ eq 'M'){
					$i_fr_hash{$i_f}=$i_r;
					$i_f++;
					$i_r++;
				}elsif($_ eq 'D'){
					$i_f++;
				}elsif($_ eq 'I'){
					$i_r++;
				}
			}
#			foreach(sort keys %i_fr_hash)
#			{
#				print "$_	=>	$i_fr_hash{$_}\n";
#			}
#			print "\n";
			foreach(keys %Index_hash){
				if(/$local_chain/ && $local_chain=~/[A-Za-z]/ || /\'$local_chain\'/ && $local_chain=~/[0-9]/){
					$Index_hash{$_}=$i_fr_hash{$Index_hash{$_}};
				}
			}
		}
	}
#	print "After alignment\n";
#	foreach(sort keys %Index_hash){
#		print "$_	$Index_hash{$_}\n";
#	}
#	find offset information ends here
#############################################################################
#
#	To know chains are intereacting with each other 	
#	Parse the Basepair Block
#
#############################################################################	
	my @Interact;					#each record represents chains that interact with each other
	my @Pairing_Info;				#the pairing information
							#the structure:
							#[0]and[3]	ID
							#[1]and[4]	sequence number
							#[2]and[5]	chain
#	for($i = 0; $i < scalar(@Basepair_Block); ++ $i){
#		print "$Basepair_Block[$i]\n";
#	}
	foreach ($i=0;$i<scalar(@Basepair_Block);$i++){
	#	print "$Basepair_Block[$i]\n";
		if($Basepair_Block[$i]=~/pairing/ && ($Basepair_Block[$i]=~/cis/ || $Basepair_Block[$i]=~/trans/) ||
			$Basepair_Block[$i] =~ /bhbond/){
	#		print "$Basepair_Block[$i]\n";
			my @local_decom;
			@local_decom=split / /,$Basepair_Block[$i];
			my @basepair;				#one basepair
			my $left_chain;				#left component:chain
			my $right_chain;			#right component:chain
			my $left_num;				#left component:number
			my $right_num;				#right component:numver
			@basepair=split /\-/,$local_decom[0];
			my $first_letter;
			$left_chain=substr($basepair[0],0,1);
			$right_chain=substr($basepair[1],0,1);
			$left_num=substr($basepair[0],1);
			$right_num=substr($basepair[1],1);
			if($left_chain eq '\'')			#If the chain ID is a number, say '2'
			{
				$basepair[0]=~/\'(.*)\'(.*)/;
				$left_chain=$1;
				$left_num=$2;
			}
			if($right_chain eq '\'')
			{
				$basepair[1]=~/\'(.*)\'(.*)/;
				$right_chain=$1;
				$right_num=$2;
			}
			my $typename1 = $id.'_'.$left_chain;
			my $typename2 = $id.'_'.$right_chain;
		
			if($Type_hash{$typename1} eq 'N' && $Type_hash{$typename2} eq 'N'){
				$Pairing_Info[$i][0]=$basepair[0];
				$Pairing_Info[$i][2]=$left_chain;
				$Pairing_Info[$i][3]=$basepair[1];
				$Pairing_Info[$i][5]=$right_chain;
			}else{
				next;
			}
			
			if($Basepair_Block[$i] =~ /(\w)-(\w) (\S+)\/(\S+) /){
				#print "$1	$2	$3	$4\n";
				if($3 eq 'Hh' || $3 eq 'Hw' || $3 eq 'Bh'){
					$Pairing_Info[$i][6] = 'H';
				}elsif($3 eq 'Ws' || $3 eq 'Ww' || $3 eq 'Wh'){
					$Pairing_Info[$i][6] = 'W'
				}elsif($3 eq 'Sw' || $3 eq 'Bs' || $3 eq 'Ss' || $3 eq "O2\'"){
					$Pairing_Info[$i][6] = 'S'
				}

				if($4 eq 'Hh' || $4 eq 'Hw' || $4 eq 'Bh'){
					$Pairing_Info[$i][7] = 'H';
				}elsif($4 eq 'Ws' || $4 eq 'Ww' || $4 eq 'Wh'){
					$Pairing_Info[$i][7] = 'W'
				}elsif($4 eq 'Sw' || $4 eq 'Bs' || $4 eq 'Ss' || $3 eq "O2\'"){
					$Pairing_Info[$i][7] = 'S'
				}	

				$Pairing_Info[$i][8] = $1;
				$Pairing_Info[$i][9] = $2;			
			}

			if($Basepair_Block[$i] =~ /cis/){
				$Pairing_Info[$i][10] = 'cis';
			}elsif($Basepair_Block[$i] =~ /trans/){
				$Pairing_Info[$i][10] = 'trans';
			}elsif($Basepair_Block[$i] =~ /bhbond/){
				if($Pairing_Info[$i][6] eq 'W' && $Pairing_Info[$i][7] eq 'S' || 
					$Pairing_Info[$i][6] eq 'S' && $Pairing_Info[$i][7] eq 'W' ||
					$Pairing_Info[$i][6] eq 'S' && $Pairing_Info[$i][7] eq 'S'){
					$Pairing_Info[$i][10] = 'cis';
				}elsif($Pairing_Info[$i][6] eq 'H' && $Pairing_Info[$i][7] eq 'S' ||
					$Pairing_Info[$i][6] eq 'S' && $Pairing_Info[$i][7] eq 'H'){
					$Pairing_Info[$i][10] = 'trans';
				}else{
					$Pairing_Info[$i][10] = 'cis';
				}
			}
			# should add cis-trans information here
			# should add pairing edge here 
			my $exist_tag;				#identify whether new chains have already exists
			$exist_tag=0;
			for($j=0;$j<scalar(@Interact);$j++){
				if($Interact[$j]=~/$left_chain/ && $Interact[$j]=~/$right_chain/){
					$exist_tag=1;
					last;
				}elsif($Interact[$j]=~/$left_chain/ && !($Interact[$j]=~/$right_chain/)){
                                	$exist_tag=1;
					$Interact[$j].=$right_chain;
        	                        last;
                	        }elsif(!($Interact[$j]=~/$left_chain/) && $Interact[$j]=~/$right_chain/){
                                	$exist_tag=1;
					$Interact[$j].=$left_chain;
        	                        last;
                	        }

			}
			if($exist_tag==0)
			{
				if($left_chain eq $right_chain){
					$Interact[$j].=$left_chain;
				}else{
					$Interact[$j].=$left_chain;
					$Interact[$j].=$right_chain;
				}
			}
		}else{
			next;
		}
	}
#	print "@Interact\n";
	for($i=0;$i<@Interact;$i++){
		for($j=$i+1;$j<@Interact;$j++){
			if(length($Interact[$i])>length($Interact[$j])){
				if($Interact[$i]=~/$Interact[$j]/ && $Interact[$i] && $Interact[$j]){
					$Interact[$j]=undef;
				}
			}elsif(length($Interact[$i])<length($Interact[$j])){
				if($Interact[$j]=~/$Interact[$i]/ && $Interact[$i] && $Interact[$j]){
					$Interact[$i]=undef;
				}
			}
		}
	}
	my $conflict_tag = 1;
	while($conflict_tag){
#		print "hello\n";
		my @revised_interact;
		$conflict_tag = 0;
		for($i=0;$i<@Interact;++$i){
			for($j=$i+1;$j<@Interact;++$j){
				my @chain1 = split //,$Interact[$i];
				my @chain2 = split //,$Interact[$j];
				for($k=0;$k<@chain1;++$k){
					for($l=0;$l<@chain2;++$l){
						if($chain1[$k] eq $chain2[$l]){
							$conflict_tag = 1;
							my $temp1 = $Interact[$i];
							my $temp2 = $Interact[$j];
							$temp2 =~ s/$chain2[$l]//g;
							$temp1 = $temp1.$temp2;
							push @revised_interact, $temp1;
							my $m;
							for($m=$j+1; $m<@Interact; ++$m){
								push @revised_interact, $Interact[$m];
							}
							$k = scalar(@chain1);
							$l = scalar(@chain2);
							$i = scalar(@Interact);
							$j = scalar(@Interact);
						}
					}
				}
			}
			if($conflict_tag == 0){
				push @revised_interact, $Interact[$i];
			}	
		}
		@Interact = undef;
		@Interact = @revised_interact;
#		print "@revised_interact\n";
	}
#	print "@Interact\n";	
#	foreach(keys %Index_hash){
#		print "$_	$Index_hash{$_}\n";
#	}
#####################################################################################
#
#	Change the array Pairing_Info with the Index_hash
#
#####################################################################################	
		
	my %Group_hash;
	for($i=0;$i<scalar(@Pairing_Info);$i++)
	{
		my $first_chain;
		my $second_chain;
		$first_chain=$Pairing_Info[$i][2];
		$second_chain=$Pairing_Info[$i][5];
		my $loif;
		my $lois;
		$Pairing_Info[$i][1]=$Index_hash{$Pairing_Info[$i][0]};
		$Pairing_Info[$i][4]=$Index_hash{$Pairing_Info[$i][3]};
		#print "$Pairing_Info[$i][0]	$Pairing_Info[$i][1]	$Pairing_Info[$i][2]	$Pairing_Info[$i][3]	$Pairing_Info[$i][4]	$Pairing_Info[$i][5]	$Pairing_Info[$i][6]	$Pairing_Info[$i][7]	$Pairing_Info[$i][8]	$Pairing_Info[$i][9]	$Pairing_Info[$i][10]\n";
		foreach(@Interact)
		{
#			print "Interact	$_\n";
			if(/$first_chain/ || /$second_chain/)
			{
				$loif=index($_,$first_chain);
				$lois=index($_,$second_chain);
				my $q;
				for($q=0;$q<$loif;$q++)
				{
					my $lo_len;
					$lo_len=substr($_,$q,1);
					my $lo;
					$lo=$id.'_'.$lo_len;
					$Pairing_Info[$i][1]+=length($Ref_Seq_hash{$lo});
				}
				for($q=0;$q<$lois;$q++)
                                {
                                        my $lo_len;
                                        $lo_len=substr($_,$q,1);
                                        my $lo;
                                        $lo=$id.'_'.$lo_len;
                                        $Pairing_Info[$i][4]+=length($Ref_Seq_hash{$lo});
                                }
				$Group_hash{$_}.=$Pairing_Info[$i][1].'|'.$Pairing_Info[$i][4].'|';
			}
		}
	}
#	foreach(keys %Index_hash){
#		print "$_	$Index_hash{$_}\n";
#	}
#	foreach(keys %Group_hash)
#	{
#		print "$_ => $Group_hash{$_}\n";
#	}
#####################################################################################
#
#	Check for non-representable pairs: pseudo-knots, 1-N pairing,
#	RNA-Protein pairing
#
#####################################################################################
	
######################################################################################
#
#	The following are wittern for output
#
#####################################################################################
	my %Output_Sequence_hash;
	my %Output_Parentheses_hash;
	my $Chains;			#all existing chain of the compound
	foreach(keys %Group_hash){
		$Chains=$_;
		my @symbol= qw /( ) < > { } [ ]/;
#	Initializing the Output sequence hash and Parentheses hash
		my %visited_chain;
		my %increased_base;
		for($i=0;$i<length($Chains);$i++){
			%increased_base = undef;
			my $local_chain;
			$local_chain=substr($Chains,$i,1);
			$visited_chain{$local_chain} = 1;
			my $local_id;
			$local_id=$id.'_'.$local_chain;
			$Output_Sequence_hash{$Chains}.=$Ref_Seq_hash{$local_id};
			$Output_Parentheses_hash{$Chains}.=('.' x length($Ref_Seq_hash{$local_id}));
			my $j;
			for($j=0; $j<@Pairing_Info; ++$j){
				if(!(exists $increased_base{$Pairing_Info[$j][0]}) && $Pairing_Info[$j][2] ne $local_chain && $Chains =~ /$Pairing_Info[$j][2]/ && !(exists $visited_chain{$Pairing_Info[$j][2]})){
					$increased_base{$Pairing_Info[$j][0]} = 1;
					$Index_hash{$Pairing_Info[$j][0]} += length($Ref_Seq_hash{$local_id});
				}
				if(!(exists $increased_base{$Pairing_Info[$j][3]}) && $Pairing_Info[$j][5] ne $local_chain && $Chains =~ /$Pairing_Info[$j][5]/ && !(exists $visited_chain{$Pairing_Info[$j][5]})){
					$increased_base{$Pairing_Info[$j][3]} = 1;
					$Index_hash{$Pairing_Info[$j][3]} += length($Ref_Seq_hash{$local_id});
				}
			}
		}

	}
#	foreach(keys %Index_hash){
#		print "$_ 	$Index_hash{$_}\n";
#	}
#####################################################################################
#
#	The following are just test for output
#
#####################################################################################
#	foreach($i=0;$i<@Helices_Pairing;$i++)
#	{
#		for($j=0;$j<5;$j++)
#		{
#			print "$Helices_Pairing[$i][$j] | ";
#		}
#		print "\n";
#	}
	$i=0;
	foreach	(keys %Output_Sequence_hash){
		$i++;
	}
	if($i==0){
		next;
	}else{
		foreach (keys %Output_Sequence_hash){	
			print ">$id";
			print "_$_\n";
			print "$Output_Sequence_hash{$_}\n";
			print "//\n";
			for($i=0; $i<@Pairing_Info; ++$i){
				if($_ =~ $Pairing_Info[$i][2] && $_ =~ $Pairing_Info[$i][5] && $Pairing_Info[$i][8]=~/\w/ && $Pairing_Info[$i][9]=~/\w/ && $Pairing_Info[$i][6]=~/\w/ && $Pairing_Info[$i][7]=~/\w/){
					print "#	$Index_hash{$Pairing_Info[$i][0]}-$Index_hash{$Pairing_Info[$i][3]} : $Pairing_Info[$i][8]-$Pairing_Info[$i][9] $Pairing_Info[$i][6]\/$Pairing_Info[$i][7] $Pairing_Info[$i][10] $Pairing_Info[$i][0]-$Pairing_Info[$i][3]\n";
				}
			}
		}
	}
}
