##############################################################################
#
#	The program is used to extract RNA secondary structure information
#	from the result of RNAVIEW.
#
#	Writtetn by Cun-cong Zhong @ Feb,10,2009
#	
#
###############################################################################

#!/usr/bin/perl -w
use strict;
use Cwd 'abs_path';

my $RNAVIEWDir = shift;
$RNAVIEWDir = abs_path($RNAVIEWDir);
open my $RHash, "<ResidueHash.txt" or die "Cannot open file: $!\n";
my @contents;
while(<$RHash>){
	chomp;
	push @contents, $_;
}
close $RHash;
my %ntHash = (
	'A' => 1,
	'C' => 1,
	'G' => 1,
	'U' => 1
);

my %edgeHash = (
	'W' => 1,
	'S' => 1,
	'H' => 1
);
my $i;
for($i = 0; $i < scalar(@contents); ++$i){
	if($contents[$i] =~ /^>(.{4})\_(.+)/){
		# reads in the hash index file
		my $id = $1;
		my $chains = $2;
		my $sequence = $contents[$i + 1];
		$i += 3;
		my %indexHash;
		while(!($contents[$i] =~ /^>/) && $i < scalar(@contents)){
			my @decom = split /	/, $contents[$i];
			$indexHash{$decom[0]} = $decom[1];
			++ $i;
		}
		-- $i;
		# check for basepair property
		my $altFilename1 = 'pdb' . $id . '.ent.out';
		my $altFilename2 = 'pdb' . $id . '.ent_nmr.pdb.out';
		my $BPFILEHANDLE;
		open $BPFILEHANDLE, "<$RNAVIEWDir/$altFilename1" or open $BPFILEHANDLE, "<$RNAVIEWDir/$altFilename2" or next;
		# assign basepair and format output
		print ">$id";
		print "_";
		print "$chains\n";
		print "$sequence\n";
		print "\/\/\n";
		my @tempholding;
		while(<$BPFILEHANDLE>){
			if(/^BEGIN_base-pair/){
				my $temp;
				while(<$BPFILEHANDLE>){
					if(/^END_base-pair/){
						last;
					}
					chomp;
					#print "Processing:	$_\n";
					$temp = $_;
					$temp =~ s/^\s+//g;
					my @decom = split /\s+/, $temp;
					my $chain1; my $chain2; my $nt1; my $nt2; my $index1; my $index2;
					if(length($decom[1]) == 2){
						$chain1 = substr($decom[1], 0, 1);
					}else{
						next;
					}
					if(length($decom[5]) == 2){
						$chain2 = substr($decom[5], 0, 1);
					}else{
						next;
					}
					#print "chain test passed\n";
					$index1 = $decom[2]; $index2 = $decom[4];
					if(length($decom[3]) == 3){
						$nt1 = uc(substr($decom[3], 0, 1));
						$nt2 = uc(substr($decom[3], 2, 1));
						if(!(exists $ntHash{$nt1}) || !(exists $ntHash{$nt2})){
							next;
						}
					}
					#print "nucleotide test passed\n";
					#print "$chain1	$chain2	$nt1	$nt2	$index1	$index2\n";
					my $leftkey;
					my $rightkey;
					if($chain1 =~ /\d+/){
						$leftkey = '\'' . $chain1 . '\'' . $index1;
					}else{
						$leftkey = $chain1 . $index1;
					}
					if($chain2 =~ /\d+/){
						$rightkey = '\'' . $chain2 . '\'' . $index2;
					}else{
						$rightkey = $chain2 . $index2;
					}
					my $innerIndex1;
					my $innerIndex2;
					if(exists $indexHash{$leftkey}){
						$innerIndex1 = $indexHash{$leftkey};
					}else{
						next;
					}
					if(exists $indexHash{$rightkey}){
						$innerIndex2 = $indexHash{$rightkey};
					}else{
						next;
					}
					#print "nucleotide hash existence test passed\n";
					if(defined($decom[6]) && defined($decom[7])){
						#print "edge and orientation test passed\n";
						#print "$decom[6]	$decom[7]\n";
						if($decom[6] =~ /.\/./ && ($decom[7] eq 'cis' || $decom[7] eq 'tran')){
							if($decom[7] eq 'tran'){
								$decom[7] = 'trans';
							}
							#print "edge existance test passed\n";
							$decom[6] = uc($decom[6]);
							$decom[6] =~ s/\+/W/g;
							$decom[6] =~ s/\-/W/g;
							my $le = substr($decom[6], 0, 1);
							my $re = substr($decom[6], 2, 1);
							if(!(exists $edgeHash{$le}) || !(exists $edgeHash{$re})){
								next;
							}
							my $checknt1 = substr($sequence, $innerIndex1, 1);
							my $checknt2 = substr($sequence, $innerIndex2, 1);
							if($checknt1 eq $nt1 && $checknt2 eq $nt2){
								my $bpinfo = sprintf("%s-%s %s %s %s-%s", $nt1, $nt2, $decom[6], $decom[7], $leftkey, $rightkey);
								#print "$bpinfo\n";
								push @tempholding, [$innerIndex1, $innerIndex2, $bpinfo];
								#print "#	$innerIndex1-$innerIndex2 : $nt1-$nt2 $decom[6] $decom[7] $leftkey-$rightkey\n";
							}
						}else{
							next;
						}
					}else{
						next;
					}
				}
			}
		}
		my $k; my $l;
		for($k = 0; $k < scalar(@tempholding) - 1; ++ $k){
			for($l = $k + 1; $l < scalar(@tempholding); ++ $l){
				if($tempholding[$k][0] > $tempholding[$l][0] || ($tempholding[$k][0] == $tempholding[$l][0] && $tempholding[$k][1] > $tempholding[$l][1])){
					my $swap;
					$swap = $tempholding[$k][0];
					$tempholding[$k][0] = $tempholding[$l][0];
					$tempholding[$l][0] = $swap;
					$swap = $tempholding[$k][1];
					$tempholding[$k][1] = $tempholding[$l][1];
					$tempholding[$l][1] = $swap;
					$swap = $tempholding[$k][2];
					$tempholding[$k][2] = $tempholding[$l][2];
					$tempholding[$l][2] = $swap;
				}
			}
		}
		for($k = 0; $k < scalar(@tempholding); ++ $k){
			print "#	$tempholding[$k][0]-$tempholding[$k][1] : $tempholding[$k][2]\n";
		}
		close $BPFILEHANDLE,
	}
}
